package com.iflytek.cloud;

import android.content.Context;

import com.iflytek.cloud.msc.module.SpeechModule;
import com.iflytek.cloud.msc.msp.MscCommon;
import com.iflytek.cloud.msc.util.log.DebugLog;
/**
 * <h3>数据上传类</h3>
 * <p>
 * 		使用此类任何接口前，需要先调用{@link SpeechUtility#createUtility }
 * 进行SDK初始化。
 * </p>
 * 
 * @deprecated 此类的上传数据功能，已被{@link SpeechRecognizer}类的成员函数代替：
 * 		<p>识别语法上传，请参考{@link SpeechRecognizer#buildGrammar}</p>
 * 		<p>听写联系人、用户热词上传，请参考{@link SpeechRecognizer#updateLexicon}</p>
 * @since version:1073
 * @author <script>gShowAuthor();</script>
 * @see SpeechUtility
 * @see SpeechRecognizer
 * @see DataDownloader
 */
public class DataUploader extends SpeechModule{
	
	/**
	 * <h4>构造函数</h4>
	 */
	public DataUploader(Context context) {
		super(context);
	}

	/**
	 * <h4>上传数据</h4>
	 * @param listener 监听器,通过此类的回调函数，获取上传状态
	 * @param name 上传的文本名称，当上传命令词识别的词典时，设为“userword”
	 * @param uploadData 上传的数据内容，需要转成gb2312格式
	 * 
	 * @return 是否成功开始上传，若返回{@link ErrorCode#SUCCESS}则是，否则开始上传失败。
	 * @see SpeechListener
	 * @see SpeechRecognizer#buildGrammar
	 * @see SpeechRecognizer#updateLexicon
	 * @see SpeechRecognizer#startListening
	 */
	public int uploadData(SpeechListener listener,String name,byte[] uploadData) {
		int ret = ErrorCode.SUCCESS;
		
		try{
			mscer = new MscCommon(mContext,mSessionParams,getHandlerThread( "upload" ));
			((MscCommon)mscer).startUpload(new InnerSpeechListener(listener),name, uploadData);
		}catch(SpeechError e){
			ret = e.getErrorCode();
			DebugLog.LogE( e );
		}catch( Throwable e ){
			ret = ErrorCode.ERROR_UNKNOWN;
			DebugLog.LogE( e );
		}//end of try-catch
		
		return ret;
	}
	
	@Override
	protected boolean onFini() {
		return true;
	}
}
