package com.iflytek.cloud;

import com.iflytek.cloud.param.MscKeys;

/**
 * <h3>公共常量</h3>
 * <p>
 * 		本类定义语音SDK用到的参数名。
 * </p>
 * <p>
 * 		除了初始化SDK时，可通过{@link SpeechUtility#createUtility}设置参数外，
 * 其他如合成，识别等会话的参数设置需要通过相关类的 setParameter 函数来设置。
 * </p>
 * <p>
 * 		SDK设置参数值均以String型的字符串传入，对于本类中说明的值范围中可能涉及的
 * int等类型的数值，请自行转换到对应的String字符串值后传给SDK，如
 * {@link java.lang.String#valueOf(int)}。同样的，通过获取参数值函数返回的值，也以
 * String值表示，请自行转换到对应的值，如{@link java.lang.Integer#parseInt(String)}。
 * </p>
 * <p>
 * 		在参数值域中，如果只有{ null, 0, 1 }的范围，则表示这是布尔(boolean)型的参数，
 * 其中，null表示空，0表示false，1表示true。+∞、-∞指int型或long型可表示的最大
 * 正负整数值。设置某参数值为null时，表示不设置此参数值，SDK会删除此参数已设置的
 * 值，存在默认值时，在会话中使用默认值。
 * </p>
 * 
 * @since version:1073
 * @author <script>gShowAuthor();</script>
 */
public class SpeechConstant {

	private SpeechConstant() {
	}

	// ///////////////////////////////////业务共用参数//////////////////////////////////////////
	/**
	 * <h4>应用ID</h4>
	 * <p>
	 * 		初始化SDK时，需要传入APPID值。服务器将根据应用的APPID认证应用接入的权
	 * 限和个性化的数据，如用户词表等。APPID需要与使用的SDK一致，否则，将报
	 * {@link ErrorCode#MSP_ERROR_DB_INVALID_APPID}的错误。SDK和APPID的申请
	 * 可通过开发者网站申请： <script>gShowDeveloperWebsite();</script>
	 * </p>
	 * <p>是否必须设置：是</p>
	 * <p>默认值：无</p>
	 * <p>值范围：与SDK一致的十六进制String类型字符串</p>
	 * @see SpeechUtility#createUtility
	 */
	public static final String APPID = "appid";
	
	/**
	 * <h4>网络类型</h4>
	 * <p>
	 * 		设置当前应用使用的网络类型。
	 * </p>
	 * <p>
	 * 		SDK通过应用设置的不同网络类型，采用不同的优化，提高网络交互速度，减少
	 * 带宽占用。默认的，SDK内会获取当前设备的网络类型（在Android等平台时），
	 * 应用可不设置此参数。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：无</p>
	 * <p>
	 * 		值范围：
	 * { null, "wifi", "cmwap", "ctwap", "uniwap", "cmnet", "uninet", "ctnet", "3gnet" }
	 * </p>
	 * 
	 * @see SpeechUtility#createUtility
	 */
	public static final String NET_TYPE = "net_type";

	/**
	 * <h4>是否强制login</h4>
	 * <p>
	 * 		在初始化调用{@link SpeechUtility#createUtility}时会对进程名称进行检查，如果名称
	 * 与应用包名不一致则不进行login操作，返回null，用以规避在子进程反复进行调用的问题。
	 * 此参数设置是否强制login。
	 * </p>
	 * <p>是否必须设置：否</p>
	 * <p>默认值：false</p>
	 * <p>值范围：{ null, true, false }</p>
	 * @see SpeechUtility#createUtility
	 */
	public static final String FORCE_LOGIN = "force_login";

	/**
	 * <h4>共享库名字</h4>
	 * <p>
	 * 		自定义初始化SDK时，要加载的库名字。
	 * </p>
	 * <p>
	 * 		在SDK使用时，与服务器交互、音频编解码等模块，都在共享库中，在SDK使用时，
	 * 必需使用这些库才可以正常使用功能。关于java使用C共享库函数的知识，请自行查找JNI
	 * （Java Native Interface）的相关资料。
	 * </p>
	 * <p>
	 * 		SDK默认的库文件名为：libmsc.so。
	 * </p>
	 * <p>
	 * 		若应用层需要自定义库名，文件名为“libmyname.so”时，
	 * 则设置此参数值为“myname”。
	 * </p>
	 * <p>
	 * 		如果库文件参数设置不正常，会引起使用时加载库失败的错误：
	 * java.lang.Unsatisfie.lang.UnsatisfiedLinkError no injava.library.path。
	 * </p>
	 * <p>是否必须设置：否</p>
	 * <p>默认值："msc"</p>
	 * <p>值范围：与共享库名一致的字符串</p>
	 * 
	 * @see SpeechUtility#createUtility
	 */
	public static final String LIB_NAME = "lib_name";

	/**
	 * <h4>结果类型</h4>
	 * <p>
	 * 		听写，识别（语法、命令词），语义理解（语音、文本），评测返回文本结果类
	 * 型。关于评测结果的等级，请参考{@link #RESULT_LEVEL}。
	 * </p>
	 * <p>
	 * 		结果类型包括：xml, json, plain。xml和json即对应的结构化文本结构，plain即
	 * 自然语言的文本，多用于听写结果。关于xml和json格式的参数说明，请参考新手指
	 * 南文档。xml和json格式多用于语法和语义中，在结果中获取置信度值或语义参数值。
	 * </p>
	 * <p>
	 * 		识别会话，支持所有结果类型，默认json。听写会话支持json和plain。
	 * </p>
	 * <p>
	 * 		语义理解会话，支持xml和json（在设置语义版本{@link #NLP_VERSION}值为
	 * 1.0时，结果只支持xml格式）。
	 * </p>
	 * <p>
	 * 		评测会话，仅支持xml格式。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值："json"</p>
	 * <p>值范围：{ "json", "xml", "plain" }</p>
	 * 
	 * @see SpeechRecognizer#setParameter(String, String)
	 * @see SpeechUnderstander#setParameter(String, String)
	 * @see SpeechEvaluator#setParameter(String, String)
	 * @see TextUnderstander#setParameter(String, String)
	 * @see RecognizerResult
	 * @see #RESULT_LEVEL
	 */
	public static final String RESULT_TYPE = "result_type";

	/**
	 * <h4>结果详细等级</h4>
	 * <p>
	 * 		返回结果等级（详细程度） ，暂时只有评测用到。可选：<br>
	 *  精简：plain <br>
	 *  完全：complete ，目前中文评测仅支持此等级。
	 * </p>
	 * <p>
	 * 		关于评测结果的详细说明，请参考<script>gShowIseResultFile();</script>。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值："complete"</p>
	 * <p>值范围：{ "complete", "plain" }</p>
	 * 
	 * @see #RESULT_TYPE
	 * @see SpeechEvaluator#setParameter(String, String)
	 */
	public static final String RESULT_LEVEL = "result_level";
	
	/**
	 * <h4>语言区域</h4>
	 * <p>
	 * 		在听写、语音语义理解和评测时，可通过设置此参数，选择要使用的语言区域，参考
	 * Java标准组织定义 {@link java.util.Locale}。关于合成的语言选择，请参考
	 * {@link #VOICE_NAME}。关于方言选择，请参考{@link #ACCENT}。
	 * </p>
	 * <p>
	 * 		当前支持：<br>
	 * 简体中文：zh_cn（默认）<br>
	 * 美式英文：en_us
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值："zh_cn"</p>
	 * <p>值范围：{ "zh_cn", "en_us" }</p>
	 * 
	 * @see #ACCENT
	 * @see SpeechRecognizer#setParameter(String, String)
	 * @see SpeechUnderstander#setParameter(String, String)
	 * @see SpeechEvaluator#setParameter(String, String)
	 */
	public static final String LANGUAGE = "language";

	/**
	 * <h4>方言</h4>
	 * <p>
	 * 		每一种语言区域，一般还有不同的方言，通过此参数，在听写和语音语义理解时，
	 * 设置不同的方言参数。
	 * </p>
	 * <p>
	 * 		当前仅在{@link #LANGUAGE}为简体中文时，支持方言选择，其他语言区域时，
	 * 请把此参数值设为null。
	 * </p>
	 * <p>
	 * 		当前支持的中文方言有：<br>
	 * 普通话：mandarin(默认)<br>
	 * 粤   语：cantonese<br>
	 * 四川话：lmz
	 * </p>
	 * <p>
	 * 		关于合成的方言选择，请参考{@link #VOICE_NAME}。关于语言区域的选择，请
	 * 参考{@link #LANGUAGE}。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值："mandarin"</p>
	 * <p>值范围：{ null, "mandarin", "cantonese", "lmz" }</p>
	 * 
	 * @see #LANGUAGE
	 * @see SpeechRecognizer#setParameter(String, String)
	 * @see SpeechUnderstander#setParameter(String, String)
	 */
	public static final String ACCENT = "accent";

	/**
	 * <h4>应用领域</h4>
	 * <p>
	 * 		服务器为不同的应用领域，定制了不同的听写匹配引擎，使用对应的领域能获取更
	 * 高的匹配率。
	 * </p>
	 * <p>
	 * 		应用领域用于听写和语音语义服务。当前支持的应用领域有：<br>
	 * 短信和日常用语：iat (默认)<br>
	 * 视频：video <br>
	 * 地图：poi<br>
	 * 音乐：music
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：无</p>
	 * <p>值范围：{ "iat", "video", "poi", "music" }</p>
	 * 
	 * @see SpeechRecognizer#setParameter(String, String)
	 * @see SpeechUnderstander#setParameter(String, String)
	 */
	public static final String DOMAIN = "domain";

	/**
	 * <h4>是否允许VAD</h4>
	 * <p>
	 * 		VAD（Voice Activity Detection,静音抑制）是用于在音频传输时，通过控制音频
	 * 的静音时长，减少在网络传输没有意义的数据，以减少网络带宽使用等。关于VAD的
	 * 具体介绍，请自行查找相关的资料。
	 * </p>
	 * <p>
	 * 		仅在允许VAD时，{@link #VAD_BOS}, {@link #VAD_EOS}才会起作用，且各监听
	 * 的音量变化回调（如{@link RecognizerListener#onVolumeChanged}）才会有音量
	 * 检测值返回。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：1(是)</p>
	 * <p>值范围：{ null, 0, 1 }</p>
	 * 
	 * @see SpeechRecognizer#setParameter(String, String)
	 * @see SpeechUnderstander#setParameter(String, String)
	 * @see #VAD_BOS
	 * @see #VAD_EOS
	 */
	public static final String VAD_ENABLE = "vad_enable";
	
	/**
	 * <h4>前端点超时</h4>
	 * <p>
	 * 		开始录入音频后，音频前面部分最长静音时长。
	 * </p>
	 * <p>
	 * 		VAD（Voice Activity Detection,静音抑制）是用于在音频传输时，通过控制音频
	 * 的静音时长，减少在网络传输没有意义的数据，以减少网络带宽使用等。关于VAD的
	 * 具体介绍，请自行查找相关的资料。
	 * </p>
	 * <p>
	 * 		SDK为了减少网络带宽，同时又可以自动的根据用户停止说话时的静音来自动结束
	 * 音频输入，使用了VAD技术。本参数正是在应用VAD时用于录入音频开始后（包括使
	 * 用SDK录音机模式或应用写入音频流模式，见{@link #AUDIO_SOURCE}），前面部
	 * 份静音的最长时间控制，当音频的前静音时长超过设置值时，SDK就会认为音频输入已
	 * 完成。此时，SDK会通过{@link RecognizerListener#onError(SpeechError)}结束会话，
	 * 并携带错误码为{@link ErrorCode#MSP_ERROR_NO_DATA}的错误信息。
	 * </p>
	 * 
	 * <p>
	 * 		当在识别会话开始前，设置了参数{@link com.iflytek.cloud.util.AudioDetector#VAD_ENGINE}
	 * 值为{@link com.iflytek.cloud.util.AudioDetector#TYPE_META}时，将启用前端点消除功能，并使
	 * 此前端点超时参数不再起作用。（前端点消除功能暂未支持）
	 * </p>
	 * <p>
	 * 		前端点静音消除，与前端点超时相反，会把开始说话前的静音全部过滤——无论时间多长，
	 * 减少在一直等待识别的场景下，出现{@link ErrorCode#MSP_ERROR_NO_DATA}的错误，并减少
	 * 静音识别产生的流量。从此处亦可知道，启用此功能后，将会使{@link #VAD_BOS}不再起作用。
	 * </p>
	 * <p>
	 * 		如果已经存在 {@link com.iflytek.cloud.util.AudioDetector}的实例，则SDK直接使用已经存在
	 * 的实例进行前端点静音消除；否则，要启用前端点静音消除，还
	 * 	必须设置 {@link com.iflytek.cloud.util.AudioDetector#RES_PATH}参数，以使
	 * SDK 能成功创建 {@link com.iflytek.cloud.util.AudioDetector}的实例。所以，在启用了前端点静音
	 * 消除时，需要保证应用没有使用  {@link com.iflytek.cloud.util.AudioDetector}，否则将出现不可预
	 * 期的数据错误。
	 * </p>
	 * <p>
	 * 		前端点静音消除对  {@link com.iflytek.cloud.util.AudioDetector} 实例的占用，仅在有启用该功能
	 * 的会话期间。且SDK不会主动销毁已创建的 {@link com.iflytek.cloud.util.AudioDetector} 实例——
	 * 无论是否其由SDK来创建。所以，若应用需要使用  {@link com.iflytek.cloud.util.AudioDetector} 的实例时，
	 * 应该参考关于该类的销毁函数 {@link com.iflytek.cloud.util.AudioDetector#destroy() } 的说明，并确保
	 * 没有在启用前端点静音消除的会话期间时，来销毁并重新创建。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：听写5000，其他4000</p>
	 * <p>值范围：[1000, 10000]</p>
	 * 
	 * @see #VAD_EOS
	 * @see #VAD_ENABLE
	 * @see SpeechRecognizer#setParameter(String, String)
	 * @see SpeechUnderstander#setParameter(String, String)
	 */
	public static final String VAD_BOS = "vad_bos";

	/**
	 * <h4>后端点超时</h4>
	 * <p>是否必须设置：否</p>
	 * <p>
	 * 		开始录入音频后，音频后面部分最长静音时长。其他详情，请参考{@link #VAD_BOS}。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：听写1800，其他700</p>
	 * <p>值范围：[0, 10000]</p>
	 * 
	 * @see #VAD_ENABLE
	 * @see #VAD_BOS
	 * @see SpeechRecognizer#setParameter(String, String)
	 * @see SpeechUnderstander#setParameter(String, String)
	 */
	public static final String VAD_EOS = "vad_eos";

	/**
	 * <h4>音频采样率</h4>
	 * <p>
	 * 		通过此参数设置音频的采样率。
	 * </p>
	 * <p>
	 * 		音频的采样率是音频属性的其中一个，一般来说，采样率越高音频的质量越好，识
	 * 别的匹配率越高，但上传带宽消耗也越大。关于音频采样率的具体说明请自行查找相关
	 * 的资料。
	 * </p>
	 * <p>
	 * 		采样率设置应该与实际传入的音频一致，否则将无法识别。在使用SDK录音模式时，
	 * SDK将根据设置的采样率录取对应音频。同时，不同的业务功能支持的采样率可能不一
	 * 样，本参数只列举普遍支持的值，对于具体业务不支持的采样率，请参考对应的类的
	 * setParameter函数的说明。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：16000</p>
	 * <p>值范围：{8000, 16000}</p>
	 * 
	 * @see SpeechRecognizer#setParameter(String, String)
	 * @see SpeechSynthesizer#setParameter(String, String)
	 * @see SpeechUnderstander#setParameter(String, String)
	 */
	public static final String SAMPLE_RATE = "sample_rate";

	/**
	 * <h4>多参数设置</h4>
	 * <p>
	 * 		通过本参数可以同时设置多个参数值。特别地，如果设置本参数值为null，将清空
	 * 所有设置过的参数，还原为默认值。
	 * </p>
	 * <p>
	 * 		设置多参数时，参数名与参数值间，以"="连接，各参数间以","连接。（参数名与
	 * 参数值间，各参数间无空格）如"vad_bos=1000,vad_eos=700"。为了减少字符串的
	 * 手误，我们建议在没有必要时，还是使用普通的一次设置单个参数值的方式。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：无</p>
	 * <p>值范围：有效的{@link java.lang.String}类型参数集成</p>
	 */
	public static final String PARAMS = "params";

	/**
	 * <h4>网络检查</h4>
	 * <p>
	 * 		是否需要检查网络状态再进行会话。（部分系统和电视平台，获取的网络状态可能不
	 * 准确）在设置为进行检查时，检查到网络断开时，直接返回20002错误。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：true</p>
	 * <p>值范围：{ null, true ,false }</p>
	 * 
	 * @see SpeechRecognizer#setParameter
	 * @see SpeechSynthesizer#setParameter
	 * @see SpeechUnderstander#setParameter
	 * @see SpeechEvaluator#setParameter
	 * @see SpeakerVerifier#setParameter
	 */
	public static final String NET_CHECK = "net_check";

	/**
	 * <h4>网络连接超时时间</h4>
	 * <p>
	 * 		设置等待结果最长的时间（毫秒）。当超过设置的时间还没返回结果时，SDK将
	 * 主动结束会话。Msc.jar和共享库都有超时控制，默认下，Msc.jar超时时间为20000
	 * ms，共享库超时时间为10000ms，此参数将同时设置Msc.jar和共享库的超时时间。
	 * 如果Msc.jar超时将回调{@link ErrorCode#ERROR_NETWORK_TIMEOUT}错误码。
	 * 共享库报的网络连接超时的错误码是{@link ErrorCode#MSP_ERROR_TIME_OUT}。
	 * 因为Msc.jar使用的超时判断策略和共享库的不一样，即使SDK会把值设置一样，但
	 * 两边的超时时间会有先后，当回调最先的一个错误后，另一个错误将被忽略。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：20000</p>
	 * <p>值范围：[0, 30000]</p>
	 * 
	 * @see SpeechRecognizer#setParameter(String, String)
	 * @see SpeechSynthesizer#setParameter(String, String)
	 */
	public static final String NET_TIMEOUT = "timeout";

	/**
	 * <h4>语音输入超时时间</h4>
	 * <p>
	 * 		设置录取音频的最长时间。在听写、识别、语音语义和声纹等需要录入音频的业务
	 * 下，在录音模式时，录取音频的最长时间。当录音超过这个时间时，SDK会自动结束
	 * 录音，并等待结果返回。当此参数值设为-1时，表示超时时间为无限，仅在评测和转
	 * 写时生效，在其他业务中，服务器最长仅支持60秒的音频，超过的音频将被忽略。
	 * 关于转写支持的最长时长，请参考转写类{@link SpeechTranscripter}说明。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：60000 （评测、转写、音频检测时，为-1）</p>
	 * <p>值范围：[-1, 60000] （评测、转写、音频检测时，为[-1，+∞)）</p>
	 * 
	 * @see SpeechRecognizer#setParameter(String, String)
	 */
	public static final String KEY_SPEECH_TIMEOUT = "speech_timeout";

	/**
	 * <h4>客户端引擎选择模式</h4>
	 * <script>gShowEngineMode();</script>
	 * 
	 * @see #ENGINE_TYPE
	 * @see SpeechRecognizer#setParameter(String, String)
	 * @see SpeechSynthesizer#setParameter(String, String)
	 * @see SpeechUtility#createUtility
	 */
	public static final String ENGINE_MODE = "engine_mode";

	/**
	 * <h4>引擎类型</h4>
	 * <p>
	 * 		设置使用的引擎类型：在线、离线、混合。在申请了离线合成资源和权限，
	 * <script>gShowEngineTypeUsePlus();</script>可以选择使用本地或在线的方式进行语音服务。
	 * </p>
	 * <p>
	 * 		使用在线模式（{@link #TYPE_CLOUD} 又称云端模式）时，需要使用网络，产生一定流量，但有更好的识别
	 * 或合成的效果，如更高的识别匹配度，更多的发音人等。
	 * </p>
	 * <p>
	 * 		使用离线模式（{@link #TYPE_LOCAL} 又称本地模式）时，不需要使用网络，且识别和合成的速度更快，但同
	 * 时要求使用对应的离线资源<script>gShowEngineTypeIntallPlus();</script>。
	 * </p>
	 * <p>
	 * 		在混合模式时（{@link #TYPE_MIX} 目前仅支持识别），可以通过{@link #MIXED_TYPE 混合类型}、
	 * {@link #MIXED_TIMEOUT 云端超时}、{@link #MIXED_THRESHOLD 本地置信门限}
	 * 使用对应的策略，提高识别准确度与成功率。详情参见前面的几个参数说明。
	 * </p>
	 * <p>
	 * 		分布式模式时（{@link #TYPE_DISTRIBUTED} 目前仅支持合成），合成将通过服务器获取合成信息后，结合本地
	 * 的资源文件合成文本。即，分布式模式与本地模式一样需要设置资源文件，不同的时，分布式模式必须联网获取合成
	 * 信息方可合成。相比本地模式，其合成效果更好；相比在线模式，其合成占用的带宽要少得多。（注意：分布式模式
	 * 与离线模式的资源不通用）
	 * </p>
	 * 
	 * <p>
	 * 		在离线或混合、分布式模式下，需要设置对应的资源。请参考
	 * <br>合成资源路径：{@link com.iflytek.cloud.util.ResourceUtil#TTS_RES_PATH}；
	 * <br>识别资源路径：{@link com.iflytek.cloud.util.ResourceUtil#ASR_RES_PATH}；
	 * <br>唤醒资源路径：{@link #IVW_RES_PATH}；
	 * </p>
	 * <p>
	 * 		关于离线资源的其他介绍，参考{@link com.iflytek.cloud.util.ResourceUtil}类的说明。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值："cloud"</p>
	 * <p>值范围：{ "cloud", "local","mixed", "distributed" }</p>
	 * 
	 * @see #ENGINE_MODE
	 * @see #MIXED_THRESHOLD
	 * @see #MIXED_TIMEOUT
	 * @see #MIXED_TYPE
	 * @see SpeechRecognizer#setParameter(String, String)
	 * @see SpeechSynthesizer#setParameter(String, String)
	 * @see SpeechUtility#createUtility
	 * @see com.iflytek.cloud.util.ResourceUtil
	 */
	public static final String ENGINE_TYPE = "engine_type";

	/**
	 * <h4>音频源</h4>
	 * <p>
	 * 		设置SDK是使用 Android自带的录音机录制音频方式，还是使用应用传入音频流方式。
	 * </p>
	 * <p>
	 * 		Android自带录音机具体支持方式请见 {@link android.media.MediaRecorder.AudioSource}。当通过
	 * 麦克风录入音频时，应用需拥有以下权限：
	 * <br>{@link android.Manifest.permission#RECORD_AUDIO}
	 * </p>
	 * <p>
	 * 		SDK在系统类型基础上扩充以下音频流方式：
	 * </p>
	 * <p>
	 * 		1、在写音频流方式(-1)下，应用层通过writeAudio函数送入音频；
	 * </p>
	 * <p>
	 * 		2、在传文件路径方式（-2）下，SDK通过应用层设置的{@link #ASR_SOURCE_PATH}值，
	 * 直接读取音频文件。目前仅在{@link SpeechRecognizer}中支持。
	 * </p>
	 * <p>
	 * 		3、在使用 AIMIC 方式( -3 )下（仅唤醒下有效），SDK通过麦克风硬件录音。需要购买硬件支持。
	 * 此时SDK需要使用 alsa 的 so 文件（一般命名为 libalsa-jni.so），  且当 Msc.jar 不包含 alsa 的类时（默认不包含），
	 * 应用工程中，还需要包含 alsa 的 jar 文件（一般命名为 alsarecorder.jar）；否则 msc.jar 将因无法使用 alsa 录音而报错。
	 * 另参考{@link #IVW_ALSA_CARD}, {@link #IVW_ALSA_RATE} 关于录音设备参数说明。
	 * </p>
	 * 	<p>
	 * 		在唤醒使用-3值方式时，还必须设置 {@link #IVW_CHANNEL_NUM}的值为大于0。
	 * 	</p>
	 * 	
	 * <p>是否必须设置：否</p>
	 * <p>默认值：1  </p>
	 * <p>值范围：SDK扩展方式{ -3, -2, -1} 和 Android系统自带方式{@link android.media.MediaRecorder.AudioSource}</p>
	 * 
	 * @see SpeechRecognizer#setParameter(String, String)
	 * @see #ASR_SOURCE_PATH
	 * @see #BLUETOOTH
	 */
	public static final String AUDIO_SOURCE = "audio_source";

	/**
	 * <h4>音频源路径</h4>
	 * <p>
	 * 		外部音频文件的路径，在直接听写、识别音频文件时可通过设置此参数，传入文件的路
	 * 径，SDK自动读取文件进行识别，不用应用层调用{@link SpeechRecognizer#writeAudio}
	 * 函数传入音频流。仅在{@link #AUDIO_SOURCE}值为-2时生效。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：null</p>
	 * <p>值范围：{@link java.lang.String}类型值的文件路径（含文件名）</p>
	 * 
	 * @see #AUDIO_SOURCE
	 * @see SpeechRecognizer#setParameter(String, String)
	 */
	public static final String ASR_SOURCE_PATH = "asr_source_path";

	/**
	 * <h4>需要过滤的音频时长</h4>
	 * <p>
	 * 		部分系统存在启动录音后，有短暂的音频跳变，通过设置此参数，可以过滤掉此部分音频，
	 * 提高识别效果。
	 * </p>
	 * <p>是否必须设置：否</p>
	 * <p>默认值：0</p>
	 * <p>值范围：[0, 60000]</p>
	 * 
	 * @see SpeechRecognizer#setParameter(String, String)
	 */
	public static final String FILTER_AUDIO_TIME = "filter_audio_time";
	
	/**
	 *  <h4>使用蓝牙通道</h4>
	 *  <p>
	 *  		在录音时，通过蓝牙来录音。目前此参数只在会话使用SDK录音时生效。设置此参数前，终端
	 *  设备应已与蓝牙设备连接成功——SDK 不会进行开启蓝牙选项的操作。同时，应用应拥有以下权限：
	 *  <br>android.permission.MODIFY_AUDIO_SETTINGS
	 *  <br>android.permission.BLUETOOTH
	 *  <br>部份手机上可能需要 android.permission.BROADCAST_STICKY
	 *  <br>当此参数设为真，又没有对应的权限时，将由于无法使用蓝牙而返回 {@link ErrorCode#ERROR_PERMISSION_DENIED}
	 *  错误。
	 *  </p>
	 *  
	 * <p>是否必须设置：否</p>
	 * <p>默认值：0</p>
	 * <p>值范围：{0, 1}</p>
	 *  
	 *  @see SpeechRecognizer#setParameter(String, String)
	 *  @see VoiceWakeuper#setParameter(String, String)
	 *  @see SpeechEvaluator#setParameter(String, String)
	 *  @see SpeechUnderstander#setParameter(String, String)
	 *  @see #AUDIO_SOURCE
	 */
	public static final String BLUETOOTH = "bluetooth";

	// ///////////////////////////////////识别相关设置/////////////////////////////////////
	/**
	 * <h4>本地语法ID</h4>
	 * <p>
	 * 		在使用本地功能时，通过设置此参数，使用本地语法识别。当此参数值为 null 时SDK
	 * 将使用本地听写。
	 * </p>
	 * <p>是否必须设置：否</p>
	 * <p>默认值：null</p>
	 * <p>值范围：有效的{@link java.lang.String}类型语法ID</p>
	 * @see #CLOUD_GRAMMAR
	 * @see SpeechRecognizer#buildGrammar(String, String, GrammarListener)
	 * @see SpeechRecognizer#setParameter(String, String)
	 * @see SpeechRecognizer#startListening(RecognizerListener)
	 * @see #GRAMMAR_TYPE
	 */
	public static final String LOCAL_GRAMMAR = "local_grammar";

	/**
	 * <h4>云端语法ID</h4>
	 * <p>即原GRAMMAR_ID参数。</p>
	 * <p>
	 * 		在线语法识别时，通过设置此参数，使用对应的语法文件。语法ID由应用层使用{@link 
	 * SpeechRecognizer#buildGrammar(String, String, GrammarListener) }构造语法后，通
	 * 过{@link GrammarListener#onBuildFinish(String, SpeechError)}回调返回。在应用层不
	 * 设置此参数，或此参数值为null，且{@link #SUBJECT}参数不为{@link #ENG_ASR}时，
	 * 将不使用任何语法，使用的是听写功能。另参考{@link SpeechRecognizer}的介绍。
	 * </p>
	 * <p><script>gShowGrammarGuide();</script></p>
	 * <p>是否必须设置：否</p>
	 * <p>默认值：null</p>
	 * <p>值范围：有效的{@link java.lang.String}类型语法ID</p>
	 * 
	 * @see SpeechRecognizer#buildGrammar(String, String, GrammarListener)
	 * @see SpeechRecognizer#setParameter(String, String)
	 * @see SpeechRecognizer#startListening(RecognizerListener)
	 * @see SpeechRecognizer
	 * @see #LOCAL_GRAMMAR
	 * @see #GRAMMAR_TYPE
	 */
	public static final String CLOUD_GRAMMAR = "cloud_grammar";

	/**
	 * <h4>语法类型</h4>
	 * <p>
	 * 		在使用识别识别时，需要设置与使用的类型一致语法类型的参数。构建本地语法
	 * 仅支持"bnf"，构建在线语法支持“abnf”，在线语法识别还可以直接在识别时，设置为
	 * 语法类型为"url-list"，并设置{@link #CLOUD_GRAMMAR}为由服务端提供的语法链接
	 * 地址，使用服务器已经存在的语法路径（但目前此用法很少，不建议使用）。
	 * </p>
	 * <p>是否必须设置：否</p>
	 * <p>默认值：null</p>
	 * <p>值范围：{ null, "abnf", "bnf", "url-list" }</p>
	 * 
	 * @see #CLOUD_GRAMMAR
	 * @see #LOCAL_GRAMMAR
	 * @see SpeechRecognizer#buildGrammar(String, String, GrammarListener)
	 * @see SpeechRecognizer#startListening(RecognizerListener)
	 * @see SpeechRecognizer#setParameter(String, String)
	 */
	public static final String GRAMMAR_TYPE = "grammar_type";
	
	/**
	 * <h4>语法名称</h4>
	 * <p>
	 * 		上传语法时的语法名字。目前上传语法后，暂不能通过语法名字使用，此参数为可选
	 * 参数。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：null</p>
	 * <p>值范围：任意字符串值</p>
	 * 
	 * @see SpeechRecognizer#buildGrammar(String, String, GrammarListener)
	 * @see SpeechRecognizer#startListening(RecognizerListener)
	 * @see SpeechRecognizer#setParameter(String, String)
	 * @see #CLOUD_GRAMMAR
	 * @see #LOCAL_GRAMMAR
	 */
	public static final String GRAMMAR_NAME = "grammar_name";

	/**
	 * <h4>语法列表名</h4>
	 * <p>
	 * 		在本地语法识别时，在构建语法后，可通过
	 * {@link SpeechRecognizer#updateLexicon(String, String, LexiconListener)}更新语法中
	 * 指定的词表，如本地语法“call”中的"contact"，则指定本参数值为"call"，在
	 * {@link SpeechRecognizer#updateLexicon(String, String, LexiconListener)}的参数1中
	 * 指定值为"contact"。
	 * </p>
	 * <p>是否必须设置：否</p>
	 * <p>默认值：null</p>
	 * <p>值范围：有效的{@link java.lang.String}类型语法名字</p>
	 * @see SpeechRecognizer#updateLexicon(String, String, LexiconListener)
	 * @see SpeechRecognizer#setParameter(String, String)
	 */
	public static final String GRAMMAR_LIST = "grammar_list";

	/**
	 * <h4>本地语法存储的子路径</h4>
	 * <p>
	 * 		应对不同的应用包名，建立子路径，存储本地语法。仅在内部使用到，应用层可
	 * 忽略此参数。
	 * </p>
	 */
	public static final String LOCAL_GRAMMAR_PACKAGE = "local_grammar_package";

	/**
	 * <h4>抛出录音数据</h4>
	 * <p>
	 * 		设置通过onEvent接口抛出听写、识别、语义业务的录音数据。当设为抛出时，会通过
	 * 监听器的onEvent函数，如{@link RecognizerListener#onEvent}，回调返回音频数据，
	 * 事件ID为{@link SpeechEvent#EVENT_RECORD_DATA}。事件详情请参考该事件ID的说
	 * 明。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：0</p>
	 * <p>值范围：{ null, 0，1 }</p>
	 * 
	 * @see SpeechEvent#EVENT_RECORD_DATA
	 * @see RecognizerListener#onEvent
	 * @see RecognizerListener#onVolumeChanged
	 * @see SpeechUnderstanderListener#onEvent
	 * @see SpeechRecognizer#setParameter
	 * @see SpeechUnderstander#setParameter
	 */
	public static final String NOTIFY_RECORD_DATA = "notify_record_data";
	
	/**
	 * <h4>混合识别门限</h4>
	 * <p>
	 * 		在混合模式，且混合策略为"realtime"时使用。当本地引擎给出结果大于此门限值时，
	 * 不用等待云端超时，即马上返回本地结果。
	 * </p>
	 * <p>是否必须设置：否</p>
	 * <p>默认值：60</p>
	 * <p>值范围：[0, 100]</p>
	 * @see #ENGINE_TYPE
	 * @see #MIXED_TYPE
	 * @see #MIXED_THRESHOLD
	 * @see SpeechRecognizer#setParameter(String, String)
	 */
	public static final String MIXED_THRESHOLD = "mixed_threshold";
	
	/**
	 * <h4>混合类型</h4>
	 * <p>java平台暂不支持此参数。</p>
	 * <p>
	 * 		在混合模式下，通过设置此参数，选择对应的混合类型：<br>
	 * realtime：实时，同时向云端和本地发送音频，在云端超时，或本地
	 * {@link #MIXED_THRESHOLD 置信门限}大于指定值时，使用本地结果；<br>
	 * 	delay: 延时，在云端识别超时后，向本地发送音频。
	 * </p>
	 * <p>是否必须设置：否</p>
	 * <p>默认值："realtime"</p>
	 * <p>值范围：{ null, "realtime", "delay" }</p>
	 * 
	 * @see #ENGINE_TYPE
	 * @see #MIXED_TIMEOUT
	 * @see #MIXED_THRESHOLD
	 * @see SpeechRecognizer#setParameter(String, String)
	 */
	public static final String MIXED_TYPE = "mixed_type";
	
	/**
	 * <h4>混合超时</h4>
	 * <p>java平台暂不支持此参数。</p>
	 * <p>
	 * 		在混合模式下，通过设置此参数，选择云端超时时间。
	 * </p>
	 * <p>是否必须设置：否</p>
	 * <p>默认值：2000</p>
	 * <p>值范围：[0, 30000]</p>
	 * @see #ENGINE_TYPE
	 * @see #MIXED_TYPE
	 * @see #MIXED_THRESHOLD
	 * @see SpeechRecognizer#setParameter(String, String)
	 */
	public static final String MIXED_TIMEOUT = "mixed_timeout";
	
	/**
	 * <h4>识别门限值</h4>
	 * <p>
	 * 		用在本地识别中，设置引擎只给出大于此门限值的识别结果。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：0</p>
	 * <p>值范围：[0, 100]</p>
	 */
	public static final String ASR_THRESHOLD = "asr_threshold";

	/**
	 * <h4>词典类型</h4>
	 * <p>
	 * 		设置个性化数据上传的数据类型。在使用
	 * {@link SpeechRecognizer#updateLexicon(String, String, LexiconListener)}上
	 * 传用户个性化数据时，通过此参数，设置要更新的是用户词表（"userword"），
	 * 还是联系人列表（"contact"）。在匹配应用定制的个性化数据时，相似的音将优
	 * 先匹配联系人列表。
	 * </p>
	 * 
	 * <p>是否必须设置：是</p>
	 * <p>默认值：null</p>
	 * <p>值范围：{ "contact", "userword" }</p>
	 * 
	 * @see SpeechRecognizer#updateLexicon(String, String, LexiconListener)
	 */
	public static final String LEXICON_TYPE = "lexicon_type";

	/**
	 * <h4>句子多侯选</h4>
	 * <p>
	 * 		在听写和语法、命令词识别时，通过设置此参数，获取在发音相似时的句子多侯选结
	 * 果。设置多候选会影响性能，响应时间延迟200ms左右。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：听写：1，语法或命令词：在线 3，本地 1</p>
	 * <p>值范围：听写：[1, 5]，语法或命令词：在线[0, 10]， 本地{0, 1}</p>
	 * 
	 * @see #ASR_WBEST
	 * @see SpeechRecognizer#setParameter(String, String)
	 */
	public static final String ASR_NBEST = "asr_nbest";

	/**
	 * <h4>词语多侯选</h4>
	 * <p>
	 * 		在听写和语法、命令词识别时，通过设置此参数，获取在发音相似时的词语多侯选结
	 * 果。设置多候选会影响性能，响应时间延迟200ms左右。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：听写：1，语法或命令词：在线 3，本地 1</p>
	 * <p>值范围：听写：[1, 5]，语法或命令词：在线[0, 10]， 本地{0, 1}</p>
	 * 
	 * @see #ASR_NBEST
	 * @see SpeechRecognizer#setParameter(String, String)
	 */
	public static final String ASR_WBEST = "asr_wbest";

	/**
	 * <h4>标点符号</h4>
	 * <p>
	 * 		通过此参数，设置听写文本结果是否含标点符号。0表示不带标点，1则表示带标点。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：1</p>
	 * <p>值范围：{ null, 0, 1 }</p>
	 * 
	 * @see SpeechRecognizer#setParameter(String, String)
	 */
	public static final String ASR_PTT = "asr_ptt";
	
	/**
	 * <h4>识别语义/翻译参数设置</h4>
	 * <p>
	 * 		通过此参数，设置识别接口进行语音语义或者翻译操作，并返回相应结果
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：0</p>
	 * <p>值范围：{ null, 0, 1 }</p>
	 * 
	 * @see SpeechConstant#NLP_VERSION
	 * @see SpeechRecognizer#setParameter(String, String)
	 * @see #ADD_CAP
	 */
	public static final String ASR_SCH = "asr_sch";

	/**
	 * <h4>动态修正 （暂不支持）</h4>
	 * <p>
	 * 		设置听写结果是否动态修正。为“1”则在听写过程中动态递增地返回结果，否则只
	 * 在听写结束之后返回最终结果。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：0</p>
	 * <p>值范围：{ null, 0, 1 }</p>
	 */
	public static final String ASR_DWA = "asr_dwa";
	
	/**
	 * <h4>设备信息</h4>
	 * <p>
	 * 		设置此参数为真时，在每次会话开始时，由SDK获取设备信息给服务器。在希望通过
	 * 听写或语义结果返回广告信息内容时，必须设置此参数为真。关于广告信息内容展示的广告控件
	 * SDK，请通过讯飞开放平台下载。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：0</p>
	 * <p>值范围：{ null, 0, 1 }</p>
	 * 
	 * @see SpeechRecognizer#setParameter(String, String)
	 */
	public static final String DVC_INFO = "dvc_info";

	/**
	 * <h4>语义版本</h4>
	 * <p>
	 * 		通过此参数，设置开放语义协议版本号。不参的版本号，返回的结果格式将不一样。
	 * 在 1.0 版本时，返回结果只支持xml格式；2.0 版本时，返回结果同时支持 xml和json，
	 * 在 3.0 版本时，返回结果只支持json。
	 * <script>gShowSchApply();</script>
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：<script>gShowDefaultNlpVersion();</script></p>
	 * <p>值范围：{ null, "1.0", "2.0", "3.0" }</p>
	 * 
	 *  @see TextUnderstander#setParameter(String, String)
	 *  @see SpeechUnderstander#setParameter(String, String)
	 *  @see #RESULT_TYPE
	 *  @see #SCENE
	 */
	public static final String NLP_VERSION = "nlp_version";
	
	/**
	 * <h4>语义情景</h4>
	 * <p>
	 * 		应用可以通此参数，使用不同的情景。每个不同的情景可以选择不同的语义处理。
	 * 服务器会为所有开通语义的应用自动配置一个 "main" 情景，在应用未设置此参数时，
	 * 服务器将默认应用该情景。
	 * </p>
	 * <p><script>gShowSetupNlpScene();</script></p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：无</p>
	 * <p>值范围：{ null, "main", 或其他通过平台配置的自定义情景名 }</p>
	 * 
	 *  @see TextUnderstander#setParameter(String, String)
	 *  @see SpeechUnderstander#setParameter(String, String)
	 *  @see #RESULT_TYPE
	 *  @see #NLP_VERSION 
	 */
	public static final String SCENE = "scene";

	/**
	 * <h4>常量值：本地引擎</h4>
	 * @see #ENGINE_TYPE
	 */
	public static final String TYPE_LOCAL = "local";

	/**
	 * <h4>常量值：云端引擎</h4>
	 * @see #ENGINE_TYPE
	 */
	public static final String TYPE_CLOUD = "cloud";

	/**
	 * <h4>常量值：混合引擎</h4>
	 * @see #ENGINE_TYPE
	 */
	public static final String TYPE_MIX = "mixed";
	
	/**
	 * <h4>常量值：分布式</h4>
	 * <p>
	 * 		目前仅在合成支持。
	 * </p>
	 * @see #ENGINE_TYPE
	 */
	public static final String TYPE_DISTRIBUTED = "distributed";
	
	/**
	 * <h4>常量值：自动引擎模型</h4>
	 * @see #ENGINE_MODE
	 */
	public static final String TYPE_AUTO = "auto";

	// ///////////////////////////////////声纹相关///////////////////////////////////////

	/**
	 * @Deprecated
	 * <h4>声纹业务类型</h4>
	 * <p>
	 * 		设置声纹业务类型。声纹业务，包括：train（训练），verify（验证），identify（
	 * 鉴别），详情请参考{@link SpeakerVerifier}类的说明。
	 * </p>
	 * 
	 * <p>是否必须设置：是</p>
	 * <p>默认值：null</p>
	 * <p>值范围：{ null, "train", "verify", "identify" }</p>
	 * 
	 * @see SpeakerVerifier#setParameter(String, String)
	 * @see SpeakerVerifier#startListening(VerifierListener)
	 */
	public static final String ISV_SST = "sst";

	/**
	 * @Deprecated
	 * <h4>声纹密码类型</h4>
	 * <p>
	 * 		设置声纹密码类型。声纹密码类型，包括：1（文本密码），2（自由说），
	 * 3（数字密码）。详情请参考{@link SpeakerVerifier} 类的说明。
	 *  (当前只支持数字密码，文本密码及自由说正在研发中)
	 * </p>
	 * 
	 * <p>是否必须设置：是</p>
	 * <p>默认值：null</p>
	 * <p>值范围：{ null, 1, 2, 3 }</p>
	 * 
	 * @see SpeakerVerifier#setParameter(String, String)
	 * @see SpeakerVerifier#startListening(VerifierListener)
	 */
	public static final String ISV_PWDT = "pwdt";

	/**
	 * <h4>声纹密码类型</h4>
	 * <p>
	 * 		设置声纹密码类型。声纹密码类型，包括：1（文本密码），2（自由说），
	 * 3（数字密码）。
	 * </p>
	 *
	 * <p>是否必须设置：是</p>
	 * <p>默认值：null</p>
	 * <p>值范围：{ null, 1, 2, 3 }</p>
	 *
	 * @see IdentityVerifier#setParameter(String, String)
	 * @see IdentityVerifier#startWorking(IdentityListener)
	 */
	public static final String MFV_PWDT = "pwdt";

	/**
	 * <h4>声纹密码自由说模式必须参数</h4>
	 * <p>
	 * 		设置声纹密码类型。声纹密码类型为 2（自由说）时，必须带入此参数，目前只支持8k音频
	 * </p>
	 *
	 * <p>是否必须设置：是</p>
	 * <p>默认值：null</p>
	 * <p>值范围：{8000}</p>
	 *
	 * @see IdentityVerifier#setParameter(String, String)
	 */
	public static final String MFV_DATA_FORMAT = "data_foramt";

	/**
	 * <h4>声纹id</h4>
	 * <p>
	 * 		注册成功服务端返回的此次注册的id。目前此参数值在声纹交互中尚未曾使用到，
	 * 应用层可忽略此参数。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：null</p>
	 * <p>值范围：有效的声纹ID值</p>
	 * 
	 * @see SpeakerVerifier#setParameter(String, String)
	 * @see SpeakerVerifier#startListening(VerifierListener)
	 */
	public static final String ISV_VID = "vid";

	/**
	 * @Deprecated
	 * <h4>声纹模型训练次数</h4>
	 * <p>
	 * 		在注册声纹模型时，需要设置模型训练的次数。引擎通过用户上传的音频，获取
	 * 用户的声纹特征，训练次数越多，识别越精准，越难被模仿。当前支持的训练次数是
	 * 2~9次。
	 * </p>
	 * 
	 * 	<p>是否必须设置：是</p>
	 * <p>默认值：null</p>
	 * <p>值范围：[2, 9]</p>
	 * 
	 * @see SpeakerVerifier#setParameter(String, String)
	 * @see SpeakerVerifier#startListening(VerifierListener)
	 * @see #ISV_SST
	 */
	public static final String ISV_RGN = "rgn";

	/**
	 *  <h4>声纹模型训练次数</h4>
	 *  <p>
	 * 		在注册声纹模型时，需要设置模型训练的次数。引擎通过用户上传的音频，获取
	 * 用户的声纹特征，训练次数越多，识别越精准，越难被模仿。当前支持的训练次数是
	 * 2~9次。
	 * </p>
	 *
	 * 	<p>是否必须设置：是</p>
	 * <p>默认值：null</p>
	 * <p>值范围：[2, 9]</p>
	 *
	 * @see IdentityVerifier#setParameter(String, String)
	 */
	public static final String MFV_RGN = "rgn";

	/**
	 * <h4>声纹密码</h4>
	 * <p>
	 * 		在声纹注册和验证时，除自由说类型外，其余均需要设置对应类型的待注册或验
	 * 证的密码，且，密码需要从服务器中获取（见
	 * {@link SpeakerVerifier#getPasswordList(SpeechListener)}）。随机数字密码
	 * 验证时，通过{@link SpeakerVerifier#generatePassword(int)}获取密码。
	 * </p>
	 * 
	 * 	<p>是否必须设置：是</p>
	 * <p>默认值：null</p>
	 * <p>值范围：有效的密码字符串</p>
	 * 
	 * @see SpeakerVerifier#setParameter(String, String)
	 * @see SpeakerVerifier#startListening(VerifierListener)
	 * @see SpeakerVerifier#getPasswordList(SpeechListener)
	 * @see #ISV_SST
	 * @see #ISV_PWDT
	 */
	public static final String ISV_PWD = "ptxt";

	public static final String MFV_PWD = "ptxt";

	/**
	 * <h4>声纹音频保存路径</h4>
	 * <p>
	 * 		通过此参数，可以在注册或验证完成后在本地保存一个音频文件（目前保存为
	 *  Windows pcm）。在Android平台上，如保存到SDCARD，需添加
	 *  WRITE_EXTERNAL_STORAGE的权限。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：null</p>
	 * <p>值范围：有效的文件相对或绝对路径（含文件名）</p>
	 * 
	 * @see SpeakerVerifier#setParameter(String, String)
	 */
	public static final String ISV_AUDIO_PATH = "isv_audio_path";
	
	/**
	 * <h4>声纹模型操作命令</h4>
	 * <p>
	 * 		在进行声纹操作时，需要设置操作的命令：que（查询），del（删除）。
	 * </p>
	 * 
	 * 	<p>是否必须设置：是</p>
	 * <p>默认值：null</p>
	 * <p>值范围：{ "que", "del" }</p>
	 * 
	 * @see SpeakerVerifier#sendRequest(String, String, SpeechListener)
	 */
	public static final String ISV_CMD = "cmd";
	
	/**
	 * <h4>声纹会话中断报错</h4>
	 * <p>
	 * 		设置声纹会话被中断时（如当前会话未结束就开启了新会话等），是否通
	 * 过{@link VerifierListener#onError(SpeechError)}回调{@link 
	 * ErrorCode#ERROR_INTERRUPT}错误。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：false</p>
	 * <p>值范围：{ null, true, false }</p>
	 * 
	 * @see SpeakerVerifier#setParameter(String, String)
	 */
	public static final String ISV_INTERRUPT_ERROR = "isv_interrupt_error";
	// ///////////////////////////////////人脸相关///////////////////////////////////////
	/**
	 * <h4>人脸识别业务类型</h4>
	 * <p>
	 * 		在人脸认别会话开始前，通过设置此参数， 决定接下来的会话业务类型
	 * ，详情请参考{@link FaceRequest}类的说明。业务类型包括：<br>
	 * 注册：reg，<br>
	 * 验证：verify，<br>
	 * 检测：detect，<br>
	 * 聚焦：align
	 * </p>
	 * 
	 * 	<p>是否必须设置：是</p>
	 * <p>默认值：null</p>
	 * <p>值范围：{ "reg", "verify", "detect", "align" }</p>
	 * 
	 * @see FaceRequest#setParameter(String, String)
	 */
	public static final String WFR_SST = "sst";

	// ///////////////////////////////////评测相关///////////////////////////////////////
	/**
	 * <h4>评测用户模型id</h4>
	 * <p>
	 * 		评测用户模型，是保存在服务器端的模型。由于目前暂不支持应用创建用户模型，
	 * 可忽略此参数。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：null</p>
	 * <p>值范围：有效的用户模型ID</p>
	 * 
	 * @see SpeechEvaluator#setParameter(String, String)
	 */
	public static final String ISE_USER_MODEL_ID = "user_model_id";
	
	/**
	 * <h4>评测题型</h4>
	 * <p>
	 * 		指定当前评测会话的题型，包括以下值：<br>
	 * read_syllable：朗读单字，英文评测不支持；<br>
	 * read_word：朗读词语；<br>
	 * read_sentence：朗读句子；<br>
	 * read_chapter：朗读篇章，暂不支持；<br>
	 * speaker_adapt：自适应，暂不支持；<br>
	 * choice_question：选择题型，暂不支持；<br>
	 * </p>
	 * <p>
	 * 		关于题型的详细说明，请参考<script>gShowIsePaperFile();</script>。
	 * </p>
	 * 
	 * 	<p>是否必须设置：是</p>
	 * <p>默认值：null</p>
	 * <p>
	 * 		值范围：{ null ,"read_syllable", "read_word", "read_sentence", "read_chapter"
	 * , "speaker_adapt", "choice_question" }
	 * </p>
	 * 
	 * @see SpeechEvaluator#setParameter(String, String)
	 */
	public static final String ISE_CATEGORY = "category";
	
	/**
	 * <h4>试题是否已标注过</h4>
	 * <p>
	 * 		试卷未标注，云端会先调用标注引擎进行标注，然后把标注结果传给引擎；
	 * 试卷已标注，云端将直接传给引擎进行评测。标注指试题是否包含拼音标注。
	 * 关于试题的说明，请参考{@link #ISE_CATEGORY}。
	 * </p>
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：0</p>
	 * <p>值范围：{ null, 0, 1 }</p>
	 * 
	 * @see SpeechEvaluator#setParameter(String, String)
	 */
	public static final String ISE_PARSED = "parsed";
	
	/**
	 * <h4>朗读跟踪</h4>
	 * <p>
	 * 		设置朗读跟踪，只对句子和篇章有效。关于跟踪模式设置，请参考
	 * {@link #ISE_TRACK_TYPE}。在开启朗读跟踪时，每次的朗读都会实时的
	 * 通过{@link EvaluatorListener#onResult(EvaluatorResult, boolean)}返回
	 * 跟踪信息，指示当前朗读的文本信息。值得注意的是，跟踪信息并不是评测
	 * 结果，跟踪信息的数据首个 xml 标签为"Data"，而评测结果则为"xml_result"。
	 * 关于跟踪信息和评测结果的说明，请参考{@link EvaluatorResult}。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值："disable"</p>
	 * <p>值范围：{ null, "enable", "disable" }</p>
	 * 
	 * @see SpeechEvaluator#setParameter(String, String)
	 * @see #ISE_TRACK_TYPE
	 * @see EvaluatorResult
	 */
	public static final String ISE_AUTO_TRACKING = "auto_tracking";
	
	/**
	 * <h4>跟踪模式</h4>
	 * <p>
	 * 		跟踪模式包括严格模式（hard）和宽松模式（easy）。在宽松模式下，读音与
	 * 试题大概一致便会返回当前的跟踪位置；相反，在严格模式下，则要求读音更准确
	 * 才会返回当前跟踪的位置。
	 * </p>
	 * <p>
	 * 		此参数仅在{@link #ISE_AUTO_TRACKING 朗读跟踪}开启时生效。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：null</p>
	 * <p>值范围：{ null, "easy", "hard" }</p>
	 * 
	 * @see SpeechEvaluator#setParameter(String, String)
	 * @see #ISE_AUTO_TRACKING
	 */
	public static final String ISE_TRACK_TYPE = "track_type";
	
	/**
	 * <h4>评测会话中断报错</h4>
	 * <p>
	 * 		设置评测会话被中断时（如当前会话未结束就开启了新会话等），是否通
	 * 过{@link EvaluatorListener#onError}回调{@link 
	 * ErrorCode#ERROR_INTERRUPT}错误。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：false</p>
	 * <p>值范围：{ null, true, false }</p>
	 * 
	 * @see SpeechEvaluator#setParameter
	 */
	public static final String ISE_INTERRUPT_ERROR = "ise_interrupt_error";
	
	/**
	 * <h4>评测音频保存路径</h4>
	 * <p>
	 * 		通过此参数，可以在评测完成后在本地保存一个音频文件。关于音频文件的格式，
	 * 请参考{@link #AUDIO_FORMAT}。在Android平台上，如保存到SDCARD，需添加
	 *  {@link android.Manifest.permission#WRITE_EXTERNAL_STORAGE}的权限。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：null</p>
	 * <p>值范围：有效的文件相对或绝对路径（含文件名）</p>
	 * 
	 * @see SpeechEvaluator#setParameter(String, String)
	 */
	public static final String ISE_AUDIO_PATH = "ise_audio_path";
	
	/**
	 * <h4>评测音频源路径</h4>
	 * <p>
	 * 		外部音频文件的路径，在直接听写、识别音频文件时可通过设置此参数，传入文件的路
	 * 径，SDK自动读取文件进行识别，不用应用层调用{@link SpeechEvaluator#writeAudio}
	 * 接口传入音频流。仅在{@link #AUDIO_SOURCE}值为-2时生效。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：null</p>
	 * <p>值范围：{@link java.lang.String}类型值的文件路径（含文件名）</p>
	 * 
	 * @see #AUDIO_SOURCE
	 * @see SpeechEvaluator#setParameter(String, String)
	 */
	public static final String ISE_SOURCE_PATH = "ise_source_path";
	
	// ///////////////////////////////////唤醒相关///////////////////////////////////////
	/**
	 * <h4>唤醒业务类型</h4>
	 * <p>
	 * 		唤醒业务类型，分为如下几种：<br>
	 * 唤醒：wakeup；（单唤醒词，多唤醒词，或特定人唤醒）<br>
	 * 注册：enroll；（特定人唤醒注册，又称为“训练”，用于特定人唤醒时，注册用户唤醒词）<br>
	 * 唤醒识别：oneshot；<br>
	 * 详情请参考{@link VoiceWakeuper}类的说明。
	 * </p>
	 * <p>特定人唤醒暂不支持。</p>
	 * 
	 * <p>是否必须设置：是</p>
	 * <p>默认值：null</p>
	 * <p>值范围：{ null, "wakeup", "enroll", "oneshot" }</p>
	 * 
	 * @see VoiceWakeuper#setParameter(String, String)
	 * @see VoiceWakeuper
	 */
	public static final String IVW_SST = "sst";

	/**
	 * <h4>合并资源目标路径</h4>
	 * <p>
	 * 		合并资源目标路径，用于在使用 {@link VoiceWakeuper#updateWords} 更新唤醒词
	 * 时，设置合并资源后返回的资源文件保存的路径。需要注意的是，此参数并不通过{@link 
	 * VoiceWakeuper#setParameter(String, String)}设置，而是设置在
	 * {@link VoiceWakeuper#updateWords}的参数2中。且由于是用来保存的路径，不必通过
	 * {@link com.iflytek.cloud.util.ResourceUtil#generateResourcePath}生成路径值，而是
	 * 直接设置即可，如"/sdcard/dest.jet"。
	 * </p>
	 * 
	 * <p>是否必须设置：是(在合并资源时，必须设置，其他不必)</p>
	 * <p>默认值：null</p>
	 * <p>值范围：有效的{@link java.lang.String}类型路径</p>
	 * 
	 */
	public static final String IVW_WORD_PATH = "ivw_word_path";
	
	/**
	 * <h4>门限值</h4>
	 * <p>
	 *  	格式：id:门限值;* (*代表可多个)，根据资源携带的唤醒词个数按照“id:门限;id:门限”
	 *  的格式传入。门限值越高，则要求匹配度越高，才能唤醒。
	 *  </p>
	 *  
	 * <p>是否必须设置：否</p>
	 * <p>默认值：1450</p>
	 * <p>值范围：[0, 3000]</p>
	 * 
	 * @see VoiceWakeuper#setParameter(String, String)
	 */
	public static final String IVW_THRESHOLD = "ivw_threshold";
	
	/**
	 * <h4>持续唤醒</h4>
	 * <p>
	 * 		在持续唤醒时，唤醒一次后，当前会话依然继续，录音还在继续，当有匹配的唤醒
	 * 时，会返回结果；否则，唤醒一次后，当前会话已结束，录音也结束，不会再监听。
	 * </p>
	 * <p>
	 * 		此参数对唤醒识别不起作用，见{@link #IVW_SST}。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：0</p>
	 * <p>值范围：{null, 0, 1}</p>
	 * 
	 * @see VoiceWakeuper#setParameter(String, String)
	 */
	public static final String KEEP_ALIVE = "keep_alive";
	
	/**
	 * <h4>包含唤醒词</h4>
	 * <p>
	 * 		设置唤醒识别时，用于识别的音频是否包含唤醒词。
	 * </p>
	 * <p>
	 * 		当包含时，唤醒词作为识别语法的一部分，在语法中必须包含唤醒词，否则将没有
	 * 匹配结果。相反的，在不包含时，唤醒词不参与识别，在说完唤醒词后，需要完整的说出
	 * 语法中的句子。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：1</p>
	 * <p>值范围：{null, 0, 1}</p>
	 * 
	 * @see #IVW_SST
	 * @see VoiceWakeuper#setParameter(String, String)
	 */
	public static final String IVW_SHOT_WORD = "ivw_shot_word";

	/**
	 * <h4>唤醒注册辅助资源路径</h4>
	 * <p>
	 * 		唤醒注册功能在注册唤醒模型时，需要辅助资源，才能进行。通过此参数，设置
	 * 辅助资源路径值。资源路径值为由
	 * {@link com.iflytek.cloud.util.ResourceUtil#generateResourcePath}生成标准的路
	 * 径值，请参考该函数的说明。
	 * </p>
	 * 
	 * <p>是否必须设置：是（仅在注册模型时）</p>
	 * <p>默认值：null</p>
	 * <p>值范围：有效的资源路径</p>
	 * 
	 * @see #IVW_SST
	 * @see #IVW_ENROLL_DEST_PATH
	 * @see VoiceWakeuper#setParameter(String, String)
	 * @see com.iflytek.cloud.util.ResourceUtil#generateResourcePath
	 */
	public static final String IVW_ENROLL_RES_PATH = "ivw_enroll_res_path";

	/**
	 * <h4>唤醒注册生成资源目标路径</h4>
	 * <p>
	 * 		设置一个文件路径，用于保存训练唤醒词生成的资源路径。与
	 * {@link #IVW_ENROLL_RES_PATH }设置使用资源路径不同的是，这里设置保存
	 * 资源的路径不需要通过{@link com.iflytek.cloud.util.ResourceUtil#generateResourcePath}
	 * 生成，直接设置即可，如"/sdcard/msc/ivw_test.jet"。
	 * </p>
	 * 
	 * 
	 * <p>是否必须设置：是（仅在注册模型时）</p>
	 * <p>默认值：null</p>
	 * <p>值范围：有效的文件路径</p>
	 *  
	 * @see #IVW_SST
	 * @see #IVW_ENROLL_RES_PATH
	 * @see VoiceWakeuper#setParameter(String, String)
	 */
	public static final String IVW_ENROLL_DEST_PATH = "ivw_enroll_dest_path";

	/**
	 * <h4>唤醒注册的最短语音时间</h4>
	 * <p>
	 * 		唤醒注册时，为了保证模型的唤醒效果，过长或过短的音频都不好，需要一个合适的音频
	 * 长度，目前，在引擎定义这个时长为250~4000毫秒。此音频时长不包括首尾的静音部分，指
	 * 录音打开后，用户开始说话时（由VAD检测到音量非0时），开始算起。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：510</p>
	 * <p>值范围：[250, 4000]</p>
	 *  
	 * @see #IVW_SST
	 * @see #IVW_ENROLL_TMAX
	 * @see VoiceWakeuper#setParameter(String, String)
	 */
	public static final String IVW_ENROLL_TMIN = "ivw_enroll_tmin";

	/**
	 * <h4>唤醒注册的最长语音时间</h4>
	 * <p>
	 * 		请参考{@link #IVW_ENROLL_TMIN}的说明。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：4000</p>
	 * <p>值范围：[250, 4000]</p>
	 *  
	 * @see #IVW_SST
	 * @see #IVW_ENROLL_TMIN
	 * @see VoiceWakeuper#setParameter(String, String)
	 */
	public static final String IVW_ENROLL_TMAX = "ivw_enroll_tmax";

	/**
	 * <h4>唤醒注册是否返回音量</h4>
	 * <p>
	 * 		设置在唤醒注册时，是否返回音量。设置为是时，将通过
	 * {@link WakeuperListener#onVolumeChanged(int)}实时返回音量值；否则不会返回。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：0</p>
	 * <p>值范围：{ null, 0, 1 }</p>
	 *  
	 * @see #IVW_SST
	 * @see VoiceWakeuper#setParameter(String, String)
	 * @see WakeuperListener#onVolumeChanged(int)
	 */
	public static final String IVW_VOL_CHECK = "ivw_vol_check";

	/**
	 * <h4>唤醒注册总次数</h4>
	 * <p>
	 * 		唤醒注册为了建立模型，需要多次采集用户对同一唤醒词的音频，即在一次注册的会话中，
	 * 要求用户多次读出同一唤醒词。在此过程中，用户一次唤醒词音频送入SDK后，SDK通过
	 * {@link WakeuperListener#onResult(WakeuperResult)}告知应用，当次注册的是否成功，应用
	 * 则根据这个结果，提示用户进行下一轮读唤醒词，或是重新开始。
	 * </p>
	 * <p>
	 * 		目前仅支持三次。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：3</p>
	 * <p>值范围：{ null, 3 }</p>
	 *  
	 * @see #IVW_SST
	 * @see VoiceWakeuper#setParameter(String, String)
	 * @see WakeuperListener#onResult(WakeuperResult)
	 * @see WakeuperResult
	 */
	public static final String IVW_ENROLL_TIMES = "ivw_enroll_times";

	/**
	 * <h4>唤醒资源路径</h4>
	 * <p>
	 * 		唤醒需要使用本地资源，通过此参数设置本地资源所在的路径。多个资源间，以英文分号";"分隔。
	 * 与{@link #IVW_ENROLL_RES_PATH}一样，用于使用的资源，需要通过
	 * {@link com.iflytek.cloud.util.ResourceUtil#generateResourcePath}生成标准的资源路径值。
	 * 请参考 {@link com.iflytek.cloud.util.ResourceUtil#generateResourcePath}。
	 * </p>
	 * 
	 * <p>是否必须设置：是（在非注册时）</p>
	 * <p>默认值：null</p>
	 * <p>值范围：有效的资源文件路径</p>
	 *  
	 * @see #IVW_SST
	 * @see VoiceWakeuper#setParameter(String, String)
	 */
	public static final String IVW_RES_PATH = "ivw_res_path";
	
	/**
	 * <h4>唤醒闭环优化网络模式</h4>
	 * <p>
	 * 		语音唤醒是本地能力，唤醒功能使用时不需要网络。但闭环优化功能需要上传数据，
	 * 故会产生一定的数据流量，建议该功能在WIFI条件下开启。
	 * </p>
	 * <p>
	 * 		唤醒闭环优化即唤醒功能在闭环环境下进行持续优化。唤醒闭环优化是根据开发者
	 * App应用场景， 通过挑选本地唤醒音频数据上传至服务端，进行训练，生成优化唤醒词资源。
	 * 优化资源相比本地资源在App使用场景在唤醒率和抑制误唤醒有良好的表现。
	 * </p>
	 * <p>
	 * 		该参数拥有以下模式：
	 * 		模式0：关闭闭环优化功能，禁止上传优化数据及启动唤醒时进行资源查询下载；
	 * 		模式1：开启闭环优化功能，允许上传优化数据，禁止启动唤醒时进行资源查询下载；
	 * 		<!-- 模式2：开启闭环优化功能，允许上传优化数据及启动唤醒时进行资源查询下载；-->
	 * 		注：目前仅加强型支持，基础性暂不支持闭环优化
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：0</p>
	 * <p>值范围：{0,1<!--,2-->}</p>
	 *  
	 * @see VoiceWakeuper#setParameter(String, String)
	 */
	public static final String IVW_NET_MODE = "ivw_net_mode";
	
	/**
	 * <h4>唤醒路数</h4>
	 * <p>
	 * 		需要通过 AIMIC 模块使用多路唤醒时，设置此参数为大于0的值，且需要 libaimic.so 以使用
	 * AIMIC 模块。最大支持的唤醒路数与 libaimic.so 及硬件的性能相关，目前建议值为 3。默认值
	 * 下，将不使用AIMIC。
	 * </p>
	 * <p>
	 * 		当此参数值大于0时，{@link VoiceWakeuper} 内部将通过 AIMIC 模块，把通过麦克风硬件录取
	 * 的多声道原始音频（SDK直接录取——{@link #AUDIO_SOURCE} 为 -3 时，或应用通过 
	 * {@link VoiceWakeuper#writeAudio(byte[], int, int)} 传入的——{@link #AUDIO_SOURCE} 为 -1 时 ），
	 * 分解出有效的几个通道，并转换至合适的采样率后，进行唤醒，并在任意一个通道命中唤醒时，返回成功
	 * 唤醒的结果，并开始返回可用于识别的 16000HZ 单声道的 PCM 音频。
	 * </p>
	 * <p>
	 * 		关于获取用于识别的音频，参考
	 * {@link WakeuperListener#onEvent(int, int, int, android.os.Bundle)} 以及
	 * {@link #IVW_RESET_AIMIC} 的说明。
	 * </p>
	 * <p>
	 * 		libaimic.so 与硬件的密钥一致方可正常使用，否则将返回 {@link ErrorCode#MSP_ERROR_NO_LICENSE} 的错误，
	 * 此时，请参考硬件附带的相关文档，确认密钥等信息是否正确。同时，5麦、6麦各不同硬件的 libaimic.so 不能相互通
	 * 用，使用时，需要确认 libaimic.so 与硬件一致，否则将可能出现不返回错误，也无法唤醒的情况。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：0</p>
	 * <p>值范围：[0, 3]</p>
	 *  
	 * @see VoiceWakeuper#setParameter(String, String)
	 * @see #AUDIO_SOURCE
	 * @see #IVW_RESET_AIMIC
	 */
	public static final String IVW_CHANNEL_NUM = "ivw_channel_num";
	
	/**
	 * <h4>重置 AIMIC 状态</h4>
	 * <p>
	 * 		AIMIC 状态重置后，直到下次成功唤醒为止，将不再通过
	 * {@link WakeuperListener#onEvent(int, int, int, android.os.Bundle)} 返回识别音频。
	 * </p>
	 * <p>
	 * 		此参数在 {@link VoiceWakeuper} 唤醒会话进行时（参考{@link VoiceWakeuper#isListening()}说明）
	 * 设置，且仅在 {@link #IVW_CHANNEL_NUM} 非0时生效。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>值范围：{1}</p>
	 * 
	 * @see VoiceWakeuper#setParameter(String, String)
	 * @see WakeuperListener#onEvent(int, int, int, android.os.Bundle)
	 */
	public static final String IVW_RESET_AIMIC = MscKeys.AIMIC_ON_PARAM_PREFIX+"reset";
	
	/**
	 * <h4>ALSA 录音卡号</h4>
	 * <p>
	 * 		当指定{@link #AUDIO_SOURCE} 为 -3 时，通过此参数，设置通过麦克风硬件时使用的
	 * CARD ID，此参数值与实际硬件相关，只有设置正确的值时才可以正常录音，请参考硬件资料
	 * 中的相关说明。此参数值仅在 {@link #AUDIO_SOURCE} 值为 -3 时有效。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：2</p>
	 * <p>值范围：视硬件而定</p>
	 * 
	 * @see #AUDIO_SOURCE
	 * @see #IVW_ALSA_RATE
	 * @see VoiceWakeuper#setParameter(String, String)
	 */
	public static final String IVW_ALSA_CARD = MscKeys.AIMIC_SSB_PARAM_PREFIX+MscKeys.AIMIC_ALSA_CARD;
	
	/**
	 * <h4>ALSA 录音采样率</h4>
	 * <p>
	 * 		当指定{@link #AUDIO_SOURCE} 为 -3 时，通过此参数，设置通过麦克风硬件时使用的
	 * 采样率，此参数值与实际硬件相关，只有设置正确的值时才可以正常录音，请参考硬件资料
	 * 中的相关说明。此参数值仅在 {@link #AUDIO_SOURCE} 值为 -3 时有效。
	 * </p>
	 * <p>
	 * 		此参数指定的仅为通过麦克风硬件录制到的多声道原始音频的采样率，最后通过 AIMIC 
	 * 模块分解后给唤醒引擎的音频，可能与此参数指定的值不一样。另参考 {@link #IVW_CHANNEL_NUM}
	 * 和 {@link #SAMPLE_RATE} 的说明。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：16000</p>
	 * <p>值范围：视硬件而定</p>
	 * 
	 * @see #AUDIO_SOURCE
	 * @see #IVW_ALSA_CARD
	 * @see VoiceWakeuper#setParameter(String, String)
	 */
	public static final String IVW_ALSA_RATE = MscKeys.AIMIC_SSB_PARAM_PREFIX+MscKeys.AIMIC_ALSA_SAMPLE_RATE;
	
	/**
	 * <h4>唤醒音频保存路径</h4>
	 * <p>仅在 {@link #AUDIO_SOURCE} 非负值时生效。</p>
	 * <p>
	 * 		通过此参数，可以在唤醒结束后在本地保存一个最近1分钟音频文件。关于音频文件的格式，
	 * 请参考{@link #AUDIO_FORMAT}。在Android平台上，如保存到SDCARD，需添加
	 *  {@link android.Manifest.permission#WRITE_EXTERNAL_STORAGE}的权限。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：null</p>
	 * <p>值范围：有效的文件相对或绝对路径（含文件名）</p>
	 * 
	 * @see VoiceWakeuper#setParameter(String, String)
	 */
	public static final String IVW_AUDIO_PATH = "ivw_audio_path";

	// ///////////////////////////////////合成相关设置//////////////////////////////////////
	/**
	 * <h4>合成发音人</h4>
	 * <p>
	 * 		通过此参数，在合成中使用不同的语言和方言、性别等。设
	 * 置的{@link #VOICE_NAME 发音人}，{@link #SPEED 语速}, {@link #PITCH 语调}，
	 * {@link #VOLUME 音量}，在一次合成时开始后，就会在合成的音频中生效，此时，若
	 * 要中途改变这些参数，则应该从想要改变的文本处重新开始合成，因为已经返回的音频
	 * 是无法再改变的。
	 * </p>
	 * <p>
	 * 		支持的发音人：请参照<script>gShowUserGuideFile();</script>。
	 * </p>
	 * <p>
	 * 		更多内容请参考官方网站:<script>gShowDeveloperWebsite();</script>。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：xiaoyan</p>
	 * <p>值范围：见上文。</p>
	 * 
	 * @see SpeechSynthesizer#setParameter(String, String)
	 * @see SpeechSynthesizer#startSpeaking(String, SynthesizerListener)
	 * @see #SPEED
	 * @see #VOLUME
	 * @see #PITCH
	 */
	public static final String VOICE_NAME = "voice_name";

	/**
	 * <h4>发音人情感</h4>
	 * <p>
	 * 		通过此参数，在合成中使用不同的情感。
	 * </p>
	 * <p>
	 * 		更多内容请参考官方网站:<script>gShowDeveloperWebsite();</script>。
	 * </p>
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：neutral</p>
	 * <p>值范围：{ neutral, happy, sad, angry }。</p>
	 */
	public static final String EMOT = "emot";
	/**
	 * <h4>预合成文本</h4>
	 * <p>
	 * 		预合成文本，用于在线合成时，利用当前合成播放时的网络空余，预合成下一段要
	 * 合成的文本。使用了预合成后，可大幅度提高合成播放的流畅度。预合成的文本要与
	 * 下一次{@link SpeechSynthesizer#startSpeaking}的文本一致，才会使预合成生效。
	 * 此参数仅在设置后的那次{@link SpeechSynthesizer#startSpeaking}的会话中生效，
	 * 下次{@link SpeechSynthesizer#startSpeaking}前没有设置此参数，则此参数值
	 * 为null。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：null</p>
	 * <p>值范围：null，或下一次会话要进行合成的文本</p>
	 * 
	 * @see SpeechSynthesizer#setParameter(String, String)
	 * @see SpeechSynthesizer#startSpeaking(String, SynthesizerListener)
	 */
	public static final String NEXT_TEXT = "next_text";

	/** 
	 * <h4>本地TTS发音人</h4>
	 * <p>仅在内使用到，应用层可忽略。</p>
	 */
	public static final String LOCAL_SPEAKERS = "local_speakers";

	/**
	 * <h4>合成语速</h4>
	 * <p>
	 * 		通过此参数，设置合成返回音频的语速。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：50</p>
	 * <p>值范围：[0, 100]</p>
	 * 
	 * @see #VOLUME
	 * @see #PITCH
	 * @see #VOICE_NAME
	 * @see SpeechSynthesizer#setParameter(String, String)
	 */
	public static final String SPEED = "speed";

	/**
	 * <h4>合成语调</h4>
	 * <p>
	 * 		通过此参数，设置合成返回音频的语调。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：50</p>
	 * <p>值范围：[0, 100]</p>
	 * 
	 * @see #VOLUME
	 * @see #SPEED
	 * @see #VOICE_NAME
	 * @see SpeechSynthesizer#setParameter(String, String)
	 */
	public static final String PITCH = "pitch";

	/**
	 * <h4>合成音量</h4>
	 * <p>
	 * 		通过此参数，设置合成返回音频的音量。合成音量，影响的是合成到的音频本身的
	 * 音量大小（振幅），而非播放时系统的音量。关于播放时操作系统的音量，则请自行
	 * 查找操作系统相关API开发接口说明。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：50</p>
	 * <p>值范围：[0, 100]</p>
	 * 
	 * @see #PITCH
	 * @see #SPEED
	 * @see #VOICE_NAME
	 * @see SpeechSynthesizer#setParameter(String, String)
	 */
	public static final String VOLUME = "volume";

	/**
	 * <h4>背景音乐</h4>
	 * <p>
	 * 		背景音乐 默认值:无，可选：1
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：0</p>
	 * <p>值范围：{ null, 0, 1 }</p>
	 * 
	 * @see SpeechSynthesizer#setParameter(String, String)
	 */
	public static final String BACKGROUND_SOUND = "background_sound";

	/**
	 * <h4>合成播放缓冲时间</h4>
	 * <p>
	 * 		在合成使用SDK进行播放时，设置此参数使合成音频更快播放。
	 * </p>
	 * <p>
	 * 		一般情况下，合成根据第一小句（出现逗号或者句号等断句）返回音频完成时，开
	 * 始播放音频，这样不会出现每小句中播放到中途出现音频未返回完而中断播放的情况。
	 * 然而，这样会因为等待第一小句音频完成，而可能会使播放开始时间慢几百毫秒以上。
	 * 对于某些播报实时性要求很高的应用来说，这个可能是无法接受的。通过此参数，可以
	 * 设置返回音频的时长达到多少毫秒后，就马上开始播放已有音频，而不必等待第一小句的完
	 * 成(当值超过第一小句时间时，将以第一小句完成开始播放)。然而，设置此参数后为大于
	 * 等于0后，合成的播放回调{@link SynthesizerListener#onSpeakProgress(int, int, int)}
	 * 返回的当前播放的文本位置将不再准确。所以，此参数一般用于使用了本地合成，且实
	 * 时性要求很高，但对播报文本位置回调没有要求的应用中，如滴滴打车等。
	 * </p>
	 * <p>
	 * 		当值为-1时，等待第一小句缓冲完成，开始播放；当值为大于等于0时，等待缓冲音
	 * 频时长大于等于设置值时，即开始播放。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：-1</p>
	 * <p>值范围：{ null, [-1, +) }</p>
	 * 
	 * @see SynthesizerListener#onSpeakProgress(int, int, int)
	 * @see SpeechSynthesizer#setParameter(String, String)
	 */
	public static final String TTS_BUFFER_TIME = "tts_buffer_time";

	/**
	 * <h4>合成播放状态</h4>
	 * <p>
	 * 		通过{@link SpeechSynthesizer#getParameter}获取当前的合成播放状态。
	 * 此参数值仅用于获取，不用来设置。
	 * </p>
	 * <p>
	 * 		取值：0（初始化），1（缓冲中），2（播放中），3（暂停中），4（停止）
	 * </p>
	 * 
	 * @see SpeechSynthesizer#getParameter
	 */
	public static final String TTS_PLAY_STATE = "tts_play_state";

	/**
	 * <h4>抛出合成数据</h4>
	 * <p>
	 * 		设置通过{@link SynthesizerListener#onEvent}抛出合成的音频数据。当设为抛出时，
	 * 会通过回调返回音频数据，事件ID为{@link SpeechEvent#EVENT_TTS_BUFFER}。事件
	 * 详情请参考该事件ID的说明。仅在{@link SpeechSynthesizer#synthesizeToUri 合成到
	 * 文件}时生效。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：0</p>
	 * <p>值范围：{ null, 0，1 }</p>
	 * 
	 * @see SpeechSynthesizer#setParameter
	 * @see SpeechSynthesizer#synthesizeToUri
	 */
	public static final String TTS_DATA_NOTIFY = "tts_data_notify";

	/**
	 * <h4>合成中断错误</h4>
	 * <p>
	 * 		合成被异常打断的情况下，是否通知外部错误码 
	 * {@link ErrorCode#ERROR_INTERRUPT}。如应用层在当前合成未结束，就调用 
	 * {@link SpeechSynthesizer#startSpeaking} 开始新的会话。默认为不通知（false）。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：false</p>
	 * <p>值范围：{ null, true，false }</p>
	 * 
	 * @see SpeechSynthesizer#setParameter
	 */
	public static final String TTS_INTERRUPT_ERROR = "tts_interrupt_error";

	/**
	 * <h4>拼音标注信息</h4>
	 * <p>此参数暂不支持。</p>
	 * <p>
	 * 		如外部需要返回拼音标注，则需要设置此值为true。返回结果为JSON结构的文
	 * 本字符串，其中标注信息在 "spell_info" 字段中。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：false</p>
	 * <p>值范围：{ null, true，false }</p>
	 * 
	 * @see SpeechSynthesizer#setParameter
	 */
	public static final String TTS_SPELL_INFO = "tts_spell_info";
	
	/**
	 * <h4>合成播放淡入淡出</h4>
	 * <p>
	 * 		通过此参数，设置是否在合成播放开始、暂停和恢复时，进行声音的淡入淡出。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：false</p>
	 * <p>值范围：{ null, true，false }</p>
	 * 
	 * @see SpeechSynthesizer#setParameter(String, String)
	 */
	public static final String TTS_FADING = "tts_fading";

	/**
	 * <h4>音频格式</h4>
	 * <p>
	 * 		保存的音频格式(合成、听写、识别、语义、声纹、评测业务支持)。仅在
	 * audio_path（{@link #ASR_AUDIO_PATH}, {@link #TTS_AUDIO_PATH}, 
	 * {@link #ISV_AUDIO_PATH}, {@link #ISE_AUDIO_PATH}）非空时生效。
	 * </p>
	 * <p>
	 * 		支持保存为以下格式：<br>
	 * 1，带音频文件头的 Windows Pcm Wav（"wav"）；<br>
	 * 2，不带音频文件头的 Windows Pcm Raw Data （"pcm"）（默认）；
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值："pcm"</p>
	 * <p>值范围：{ null, "wav"，"pcm" }</p>
	 * 
	 * @see SpeechSynthesizer#setParameter
	 * @see SpeechRecognizer#setParameter
	 * @see SpeakerVerifier#setParameter
	 * @see SpeechUnderstander#setParameter
	 * @see SpeechEvaluator#setParameter
	 * @see #TTS_AUDIO_PATH
	 * @see #ASR_AUDIO_PATH
	 * @see #ISV_AUDIO_PATH
	 * @see #ISE_AUDIO_PATH
	 */
	public static final String AUDIO_FORMAT = "audio_format";
	
	/**
	 * <h4>合成播放流类型</h4>
	 * <p>
	 * 		在安卓平台中，播放器{@link android.media.AudioTrack}可以使用多种流类型
	 * 播放音频，以达到音乐、铃声等类型的声音。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：{@link android.media.AudioManager#STREAM_MUSIC}</p>
	 * <p>值范围：{@link android.media.AudioManager}支持的流类型值</p>
	 * 
	 * @see SpeechSynthesizer#setParameter(String, String)
	 */
	public static final String STREAM_TYPE = "stream_type";

	/**
	 * <h4>音频焦点</h4>
	 * <p>
	 * 		在需要录音（如听写）或播音（如合成）的会话中，为了避免出现把音乐播放软件
	 * 播放的声音录入，或与合成声音同时播的问题，需要让SDK获取系统的音频焦点。通过
	 * 此参数，设置是否会话过程中由SDK获取音频焦点。当SDK获取或释放音频焦点时，系
	 * 统通过{@link android.media.AudioManager.OnAudioFocusChangeListener#onAudioFocusChange}
	 * 回调通知所有监听，音频焦点的变化。应用层也可以设置不由SDK获取音频焦点，并自
	 * 己实现音频焦点获取、丢失时，暂停合成播放等处理。
	 * </p>
	 * 
	 * <p>是否必须设置：否</p>
	 * <p>默认值：true  (唤醒默认关闭)</p>
	 * <p>值范围：{ null, true，false }</p>
	 * 
	 * @see SpeechSynthesizer#setParameter
	 * @see SpeechRecognizer#setParameter
	 * @see SpeakerVerifier#setParameter
	 * @see SpeechUnderstander#setParameter
	 * @see SpeechEvaluator#setParameter
	 * @see VoiceWakeuper#setParameter
	 */
	public static final String KEY_REQUEST_FOCUS = "request_audio_focus";

	/**
	 * <h4>合成音频保存路径</h4>
	 * <p>
	 * 		通过此参数，可以在合成完成后在本地保存一个音频文件，关于音频文件的格式，
	 * 请参考{@link #AUDIO_FORMAT}。在Android平台上，如保存到SDCARD，需添加
	 *  {@link android.Manifest.permission#WRITE_EXTERNAL_STORAGE}的权限。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：null</p>
	 * <p>值范围：有效的文件相对或绝对路径（含文件名）</p>
	 * 
	 * @see SpeechSynthesizer#setParameter(String, String)
	 * @see SpeechSynthesizer#synthesizeToUri(String, String, SynthesizerListener)
	 * @see #ASR_AUDIO_PATH
	 * @see #AUDIO_FORMAT
	 */
	public static final String TTS_AUDIO_PATH = "tts_audio_path";

	/**
	 * <h4>数据类型</h4>
	 * <p>已被{@link #LEXICON_TYPE}取代。</p>
	 * <p>
	 * 		设置个性化数据上传的数据类型。在使用
	 * {@link SpeechRecognizer#updateLexicon(String, String, LexiconListener)}及
	 * {@link DataUploader#uploadData(SpeechListener, String, byte[])}上传用户个
	 * 性化数据时，通过此参数，设置要更新的是用户词表（"userword"），还是联系
	 * 人列表（"contact"）。在匹配应用定制的个性化数据时，相似的音将优先匹配联
	 * 系人列表。
	 * </p>
	 * 
	 * <p>是否必须设置：是</p>
	 * <p>默认值：null</p>
	 * <p>值范围：{ "contact", "userword" }</p>
	 * 
	 * @see SpeechRecognizer#updateLexicon(String, String, LexiconListener)
	 * @see DataUploader#uploadData(SpeechListener, String, byte[])
	 */
	public static final String DATA_TYPE = "data_type";

	/**
	 * <h4>业务类型</h4>
	 * <p>
	 * 		会话中区分各业务，除身份认证等在该类的setParameter函数中说明的特
	 * 殊业务外，其他可由SDK自动设置，应用可忽略此参数。业务类型包括：<br>
	 * {@link SpeechRecognizer 听写}：iat<br>
	 * {@link SpeechRecognizer 识别}：asr<br>
	 * {@link SpeechSynthesizer 合成}：tts<br>
	 * {@link SpeakerVerifier 声纹}：ivp<br>
	 * {@link SpeechEvaluator 评测}：ise<br>
	 * {@link IdentityVerifier 多重认证}：mfv<br>
	 * {@link IdentityVerifier 身份认证之声纹}：ivp<br>
	 * {@link IdentityVerifier 身份认证之人脸}：ifr<br>
	 * {@link SpeechRecognizer#updateLexicon 上传}：uup<br>
	 * {@link SpeechTranscripter 转写}：ist<br>
	 * 部分业务可能未开放，或已不再支持，请参考各类的说明。
	 * </p>
	 * 
	 * 	<p>是否必须设置：视不同类而异</p>
	 * <p>默认值：视不同类而异</p>
	 * <p>值范围：见上文</p>
	 */
	public static final String SUBJECT = "subject";

	/**
	 * <h4>识别音频保存路径</h4>
	 * <p>
	 * 		通过此参数，可以在识别完成后在本地保存一个音频文件。关于音频文件的格式，
	 * 请参考{@link #AUDIO_FORMAT}。在Android平台上，如保存到SDCARD，需添加
	 *  {@link android.Manifest.permission#WRITE_EXTERNAL_STORAGE}的权限。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：null</p>
	 * <p>值范围：有效的文件相对或绝对路径（含文件名）</p>
	 * 
	 * @see SpeechRecognizer#setParameter(String, String)
	 */
	public static final String ASR_AUDIO_PATH = "asr_audio_path";

	/**
	 * <h4>识别会话中断报错</h4>
	 * <p>
	 * 		设置识别会话被中断时（如当前会话未结束就开启了新会话等），是否通
	 * 过{@link RecognizerListener#onError}回调{@link 
	 * ErrorCode#ERROR_INTERRUPT}错误。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：false</p>
	 * <p>值范围：{ null, true, false }</p>
	 * 
	 * @see SpeechRecognizer#setParameter
	 */
	public static final String ASR_INTERRUPT_ERROR = "asr_interrupt_error";

	/**
	 * <h4>无结果错误</h4>
	 * <p>
	 * 		通过此参数，设置在识别无结果时，是否通过{@link RecognizerListener#onError}
	 * 返回{@link ErrorCode#MSP_ERROR_NO_DATA}错误。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：true</p>
	 * <p>值范围：{ null, true, false }</p>
	 * 
	 * @see SpeechRecognizer#setParameter
	 */
	public static final String ASR_NOMATCH_ERROR = "asr_nomatch_error";

	/**
	 * <h4>网络状态反馈值</h4>
	 * <p>
	 * 		通过此参数，获取网络连接状态（仅在识别和听写）。网络状态是SDK通过测试
	 * 网络连接返回的[0, 100]范围的状态值，值越高，表示网络状态越好。设置此参数为
	 * 真后，网络状态将通过onEvent函数的{@link SpeechEvent#EVENT_NETPREF}
	 * 事件返回给应用层。
	 * </p>
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：false</p>
	 * <p>值范围：{ null, true, false }</p>
	 * 
	 * @see SpeechEvent#EVENT_NETPREF
	 * @see RecognizerListener#onEvent
	 * @see VerifierListener#onEvent
	 * @see SpeechRecognizer#setParameter(String, String)
	 * @see SpeakerVerifier#setParameter(String, String)
	 */
	public static final String ASR_NET_PERF = "asr_net_perf";

	/**
	 * <h4>常量值：识别引擎</h4>
	 */
	public static final String ENG_ASR = "asr";

	/**
	 * <h4>常量值：合成引擎</h4>
	 */
	public static final String ENG_TTS = "tts";

	/**
	 * <h4>常量值：语义引擎</h4>
	 */
	public static final String ENG_NLU = "nlu";

	/**
	 * <h4>常量值：唤醒引擎</h4>
	 */
	public static final String ENG_IVW = "ivw";

	/**
	 * <h4>常量值：声纹引擎</h4>
	 */
	public static final String ENG_IVP = "ivp";

	/**
	 * <h4>常量值：人脸引擎</h4>
	 */
	public static final String ENG_WFR = "wfr";

	/**
	 * <h4>常量值：评测引擎</h4>
	 */
	public static final String ENG_EVA = "eva";

	/**
	 * <h4>常量值：MSC模式</h4>
	 */
	public static final String MODE_MSC = "msc";

	/**
	 * <h4><script>gShowModePlus();</script></h4>
	 */
	public static final String MODE_PLUS = "plus";

	/**
	 * <h4>常量值：自动模式</h4>
	 */
	public static final String MODE_AUTO = "auto";

	/**
	 * <h4>文本编码格式</h4>
	 * <p>
	 * 		合成传入文本，或构建语法、个性化词典，听写、识别和语义结果的文本编码格式。
	 * </p>
	 * <p>
	 * 		除在使用{@link DataUploader#uploadData(SpeechListener, String, byte[])}上传
	 * 外，其他函数传入的多为String类型的字符串，SDK会自动根据业务自动选择不同的编
	 * 码格式，在没有特殊说明要设置编码格式的函数，应用层如无必要，不需要设置此参数。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：视业务而异</p>
	 * <p>值范围：{ null, "gb2312", "gbk", "big5", "unicode", "gb18030", "utf-8" }</p>
	 * 
	 * @see SpeechRecognizer#setParameter(String, String)
	 * @see SpeechSynthesizer#setParameter(String, String)
	 * @see SpeechRecognizer#buildGrammar(String, String, GrammarListener)
	 * @see SpeechRecognizer#updateLexicon(String, String, LexiconListener)
	 * @see DataUploader#uploadData(SpeechListener, String, byte[])
	 */
	public static final String TEXT_ENCODING = "text_encoding";

	/**
	 * <h4>文本字节顺序</h4>
	 * <p>
	 * 		当以byte[]类型传入的文本数据时，若是utf-8或unicode格式，则应在数据中有
	 * 字节顺序（Byte Order Mark, BOM）信息，指示数据存储方式。当没有字节顺序
	 * 信息时，unicode编码时，默认数据为“UnicodeLittleUnmarked”格式。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：1</p>
	 * <p>值范围：{ null, 0, 1 }</p>
	 */
	public static final String TEXT_BOM = "text_bom";
	
	/**
	 * <h4>授权ID</h4>
	 * <p>
	 * 		授权ID，是每个应用的用户的唯一标识，声纹密码，人脸识别，以及身份认证时需要设置。
	 * 在获取密码，注册和验证声纹，查询和删除声纹模型时，必须设置的用于区分操作
	 * 的用户模型的标识。授权ID由应用层自定义，应保证每个用户的ID不一样，否则，
	 * 可能在注册时，报{@link ErrorCode#MSP_ERROR_ALREADY_EXIST}的错误。若
	 * 注册时需要覆盖一个已存在的授权ID，可以在注册时，设置参数名“property”的
	 * 值为"del"。
	 * </p>
	 * 
	 * 	<p>是否必须设置：是</p>
	 * <p>默认值：null</p>
	 * <p>
	 * 		值范围：6-18位，以英文字母开头，由英文字母、阿拉伯数字和下划线组成，区分
	 * 字母大小字的字符串。
	 * </p>
	 * 
	 * @see SpeakerVerifier#sendRequest(String, String, SpeechListener)
	 * @see SpeakerVerifier#setParameter(String, String)
	 * @see IdentityVerifier#setParameter(String, String)
	 */
	public static final String AUTH_ID = "auth_id";
	
	// ///////////////////////////////////身份验证相关///////////////////////////////////////
	
	/**
	 * <h4>会话类型</h4>
	 * <p>人脸业务（ifr）</p>
	 * <p>包括注册（enroll），验证（verify），删除（delete）。</p>
	 * <p>暂不支持：重新注册（reenroll）查询（query）。</p>
	 * 
	 * <p>声纹业务（ivp）</p>
	 * <p>包括注册（enroll，或train），验证（verify），查询（query），删除（delete），下载（download）</p>
	 * <p>暂不支持：重新注册（reenroll）</p>
	 * <p>
	 * 		请参考{@link IdentityVerifier}类的说明。
	 * </p>
	 * 
	 * 	<p>是否必须设置：是</p>
	 * <p>默认值：null</p>
	 * <p>值范围：</p>
	 * <p>人脸业务时：{ "enroll", "verify", "reenroll", "query", "delete" }</p>
	 * <p>声纹业务时：{ "enroll", "verify", "reenroll", "query", "delete", "download" }</p>
	 * 
	 * @see IdentityVerifier#setParameter(String, String)
	 * @see IdentityVerifier
	 */
	public static final String MFV_SST = "sst";
	
	/**
	 * <h4>融合验证模式</h4>
	 * <p>
	 * 		仅在融合验证场景下使用。可选值：单一生物特征数据验证（sin），混合生物特征数据验证（mix），
	 * 灵活生物特征数据验证（agi）。
	 * </p>
	 * 
	 * 	<p>是否必须设置：是（仅在融合验证场景下）</p>
	 * <p>默认值：null</p>
	 * <p>值范围：{ "sin", "mix", "agi" }</p>
	 * 
	 * @see IdentityVerifier#setParameter(String, String)
	 * @see IdentityVerifier
	 */
	public static final String MFV_VCM = "vcm";
	
	/**
	 * <h4>特征场景</h4>
	 * <p>
	 * 		用来说明本次会话将涉及的业务。包括：鉴别组管理(ipt)，声纹（ivp），人脸（ifr），人脸声纹融合(ivp|ifr)。
	 * </p>
	 * 
	 * 	<p>是否必须设置：是</p>
	 * <p>默认值：null</p>
	 * <p>值范围：{ null,"ipt","ivp", "ifr", "ivp|ifr" }</p>
	 * 
	 * @see IdentityVerifier#setParameter(String, String)
	 * @see IdentityVerifier
	 */
	public static final String MFV_SCENES = "scenes";
	
	/**
	 * <h4>确认周期</h4>
	 * <p>
	 * 		用户设置的确认超时时间（生命周期，单位：s），仅在灵活融合验证场景下使用。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：30</p>
	 * <p>值范围：[3, 45]</p>
	 * 
	 * @see IdentityVerifier#setParameter(String, String)
	 */
	public static final String MFV_AFC = "afc";
	
	/**
	 * <h4>数据保存路径</h4>
	 * <p>
	 * 		通过此参数，可以在声纹业务完成后在本地保存一个音频文件，关于音频文件的格式，
	 * 请参考{@link #AUDIO_FORMAT}。在Android平台上，如保存到SDCARD，需添加
	 *  {@link android.Manifest.permission#WRITE_EXTERNAL_STORAGE}的权限。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：null</p>
	 * <p>值范围：有效的文件相对或绝对路径（含文件名）</p>
	 * 
	 * @see IdentityVerifier#setParameter(String, String)
	 */
	public static final String MFV_DATA_PATH = "mfv_data_path";
	
	/**
	 * <h4>身份认证会话中断报错</h4>
	 * <p>
	 * 		设置身份认证会话被中断时（如当前会话未结束就开启了新会话等），是否通
	 * 过{@link IdentityListener#onError(SpeechError)}回调{@link ErrorCode#ERROR_INTERRUPT}错误。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值：false</p>
	 * <p>值范围：{ null, true, false }</p>
	 * 
	 * @see IdentityVerifier#setParameter(String, String)
	 */
	public static final String MFV_INTERRUPT_ERROR = "mfv_interrupt_error";
	
	/**
	 * <h4>网络协议</h4>
	 * <p>
	 * 		在使用业务时与服务端的网络交互协议。当默认的协议在网络环境不支持时，可以设置其他
	 * 支持的协议，以正常使用服务。
	 * </p>
	 * 
	 * 	<p>是否必须设置：否</p>
	 * <p>默认值："tcp"</p>
	 * <p>值范围：{ null, "ssl", "tcp" }</p>
	 * 
	 * @see IdentityVerifier#setParameter(String, String)
	 */
	public static final String PROT_TYPE = "prot_type";
	// ///////////////////////////////////语记获取离线资源能力//////////////////////////////////////
	/**
	 * <h4>本地合成资源</h4>
	 * <script>gShowPlusLocal( gTts );</script>
	 * 
	 * @see SpeechUtility#getParameter(String)
	 */
	public static final String PLUS_LOCAL_TTS = "tts";
	
	/**
	 * <h4>本地识别资源</h4>
	 * <script>gShowPlusLocal( gAsr );</script>
	 * 
	 * @see SpeechUtility#getParameter(String)
	 */
	public static final String PLUS_LOCAL_ASR = "asr";
	
	/**
	 * <h4>本地唤醒资源</h4>
	 * <script>gShowPlusLocal( gIvw );</script>
	 * 
	 * @see SpeechUtility#getParameter(String)
	 */
	public static final String PLUS_LOCAL_IVW = "ivw";
	
	/**
	 * <h4>本地所有资源</h4>
	 * <script>gShowPlusLocal( gAll );</script>
	 * 
	 * @see SpeechUtility#getParameter(String)
	 */
	public static final String PLUS_LOCAL_ALL = "all";
	
	/**
	 * <h4>转写会话ID</h4>
	 * <p>在续传时需要指定</p>
	 * <p>
	 * 		如需下次进行续传，请在当前会话 {@link com.iflytek.cloud.SpeechEvent#EVENT_SESSION_ID} 
	 * 获取，或在{@link com.iflytek.cloud.SpeechEvent#EVENT_SESSION_END}事件中，以第4个参数，通
	 * 过{@link android.os.Bundle#getString}指定KEY值为本参数名获取。
	 * </p>
	 * <p>若获取的值为空，表明本次会话已发生错，未生成会话ID。</p>
	 * 
	 * <p>转写功能暂未开放，相关参数设计预留</p>
	 * <p>是否必须设置：否</p>
	 * <p>默认值：null</p>
	 * <p>值范围：有效的{@link java.lang.String}会话ID值</p>
	 * 
	 * @see com.iflytek.cloud.SpeechTranscripter#setParameter
	 * @see com.iflytek.cloud.SpeechTranscripter#getParameter
	 * @see com.iflytek.cloud.SpeechTranscripter#startTranscripting
	 * @see com.iflytek.cloud.TranscripterListener#onEvent
	 */
	public static final String IST_SESSION_ID = "sid";
	
	/**
	 * <h4>转写同步id</h4>
	 * <p>在续传时需要指定</p>
	 * <p>
	 * 		如需下次进行续传，请在当前会话
	 * {@link com.iflytek.cloud.SpeechEvent#EVENT_SESSION_END}
	 * 事件中，以第4个参数，通过{@link android.os.Bundle#getInt}指定KEY值为
	 * 本参数名获取值。
	 * </p>
	 * <p>因为续传时必须指定为上次会话最新的值，所以务必在该事件中获取最新的值</p>
	 * 
	 * <p>转写功能暂未开放，相关参数设计预留</p>
	 * <p>是否必须设置：否</p>
	 * <p>默认值：null</p>
	 * <p>值范围：有效的同步ID值</p>
	 * 
	 * @see com.iflytek.cloud.SpeechTranscripter#setParameter
	 * @see com.iflytek.cloud.SpeechTranscripter#getParameter
	 * @see com.iflytek.cloud.SpeechTranscripter#startTranscripting
	 * @see com.iflytek.cloud.TranscripterListener#onEvent
	 * @see com.iflytek.cloud.SpeechEvent#EVENT_SESSION_END
	 */
	public static final String IST_SYNC_ID = "syncid";
	
	/**
	 * <h4>转写已上传音频字节数</h4>
	 * <p>在续传时需要指定</p>
	 * <p>
	 * 		如需下次进行续传，请在当前会话
	 * {@link com.iflytek.cloud.SpeechEvent#EVENT_SESSION_END}
	 * 事件中，以第4个参数，通过{@link android.os.Bundle#getInt}指定KEY值为
	 * 本参数名获取值。
	 * </p>
	 * <p>
	 * 		因为续传时必须指定为上次会话最新的值，所以务必在该事件中获取最新的值
	 * </p>
	 * 
	 * <p>转写功能暂未开放，相关参数设计预留</p>
	 * <p>是否必须设置：否</p>
	 * <p>默认值：null</p>
	 * <p>值范围：与上次会话事件中获取一致的值</p>
	 * 
	 * @see com.iflytek.cloud.SpeechTranscripter#setParameter
	 * @see com.iflytek.cloud.SpeechTranscripter#getParameter
	 * @see com.iflytek.cloud.SpeechTranscripter#startTranscripting
	 * @see com.iflytek.cloud.TranscripterListener#onEvent
	 * @see com.iflytek.cloud.SpeechEvent#EVENT_SESSION_END
	 */
	public static final String IST_AUDIO_UPLOADED = "spos";
	
	
	/**
	 * <h4>转写音频路径</h4>
	 * <p>
	 * 		在录音模式下，可指定文件保存的目录，或文件的绝对路径。
	 * 当文件或目录不存在时，SDK自动尝试创建，当创建失败时，回调
	 * {@link com.iflytek.cloud.ErrorCode#ERROR_FILE_ACCESS}错误;
	 * 当创建成功时，可通过
	 * {@link com.iflytek.cloud.TranscripterListener#onEvent}
	 * 的
	 * {@link com.iflytek.cloud.SpeechEvent#EVENT_IST_AUDIO_FILE}事件中获取值。
	 * 或在
	 * {@link com.iflytek.cloud.SpeechEvent#EVENT_SESSION_END}
	 * 事件中，以第4个参数，通过{@link android.os.Bundle#getString}指定KEY值为
	 * 本参数名获取值。
	 * </p>
	 * <p>
	 * 		在转写指定文件模式({@link #AUDIO_SOURCE}值为－2时)下，当指定的文件不存在时，回调
	 * {@link com.iflytek.cloud.ErrorCode#ERROR_FILE_ACCESS}错误;
	 * </p>
	 * 
	 * <p>转写功能暂未开放，相关参数设计预留</p>
	 * <p>是否必须设置：否</p>
	 * <p>默认值：null</p>
	 * <p>值范围：有效的文件路径值</p>
	 * 
	 * @see com.iflytek.cloud.SpeechTranscripter#setParameter
	 * @see com.iflytek.cloud.SpeechTranscripter#getParameter
	 * @see com.iflytek.cloud.SpeechTranscripter#startTranscripting
	 * @see com.iflytek.cloud.TranscripterListener#onEvent
	 * @see com.iflytek.cloud.SpeechEvent#EVENT_SESSION_END
	 * @see com.iflytek.cloud.SpeechEvent#EVENT_IST_AUDIO_FILE
	 */
	public static final String IST_AUDIO_PATH = "ist_audio_path";
	
	
	/**
	 * <h4>转写会话重启尝试次数</h4>
	 * <p>
	 * 		指定当会话失败时，重试的次数。在转写音频源{@link #AUDIO_SOURCE}为负数时为全过程的重试次数，
	 * 当音频源{@link #AUDIO_SOURCE}为麦克风时，为停止录音后的重试次数。
	 * </p>
	 * <p>默认值为1，如需指定为无限次，则设置值为-1。</p>
	 * 
	 * <p>转写功能暂未开放，相关参数设计预留</p>
	 * <p>是否必须设置：否</p>
	 * <p>默认值：1</p>
	 * <p>值范围：[-1, +∞)</p>
	 * 
	 * @see com.iflytek.cloud.SpeechTranscripter#setParameter
	 * @see com.iflytek.cloud.SpeechTranscripter#startTranscripting
	 * @see #AUDIO_SOURCE
	 */
	public static final String IST_SESSION_TRY = "ist_session_try";
	
	/**
	 * <h4>后处理结果通道</h4>
	 * <script>gShowUnSupportInLxy();</script>
	 * <p>
	 * 		应用可以通此参数，获取不同的后处理结果。
	 * 		在听写和识别业务中，只有在
	 * 		{@link #ASR_SCH} 为"1" 时，此参数设置才会生效,
	 * 		若取值为"translate"，则获取翻译结果，若为"kcloud"，则为语义结果。
	 * </p>
	 *
	 * <p>是否必须设置：仅在翻译时必须设置</p>
	 * <p>默认值：无</p>
	 * <p>值范围：{ null, "translate","kcloud" }</p>
	 *
	 *  @see SpeechRecognizer#setParameter(String, String)
	 *  @see #ORI_LANG
	 *  @see #TRANS_LANG
	 *  @see #TRS_SRC
	 */
	public static final String ADD_CAP = "addcap";

	/**
	 * <h4>翻译原始语种</h4>
	 * <script>gShowUnSupportInLxy();</script>
	 * <p>
	 *     	此参数仅在翻译情景下使用，即设置了参数：
	 *     {@link	#ADD_CAP} 为"translate" 的情况下，此参数才会生效。
	 *	"cn"表示汉语，"en"表示英语。
	 * </p>
	 * 
	 * <p>是否必须设置：仅在翻译时必须设置</p>
	 * <p>默认值：无</p>
	 * <p>值范围：{ null, "cn","en" }</p>
	 *
	 * @see SpeechRecognizer#setParameter(String, String)
	 * @see #TRANS_LANG
	 * @see #ADD_CAP
	 */
	public static final String ORI_LANG = "orilang";

	/**
	 * <h4>翻译目标语种</h4>
	 * <script>gShowUnSupportInLxy();</script>
	 * <p>
	 * 		此参数仅在翻译情景下使用，即设置了参数：
	 *     	{@link	#ADD_CAP} 为"translate" 的情况下，此参数才会生效。
	 *     "cn"表示汉语，"en"表示英语。
	 * </p>
	 *
	 * 	<p>是否必须设置：仅在翻译时必须设置</p>
	 * <p>默认值：无</p>
	 * <p>值范围：{ null, "cn","en" }</p>
	 *
	 *  @see SpeechRecognizer#setParameter(String, String)
	 *  @see #ORI_LANG
	 *  @see #ADD_CAP
	 */
	public static final String TRANS_LANG = "translang";

	/**
	 * <h4>翻译结果格式</h4>
	 * <script>gShowUnSupportInLxy();</script>
	 * <p>
	 *		设置翻译结果格式，目前仅支持一种。
	 * </p>
	 * <p>是否必须设置:否</p>
	 * <p>默认值:无</p>
	 * <p>取值范围：{null,"its"}</p>
	 * 
	 * @see #ADD_CAP
	 * @see SpeechRecognizer#setParameter(String, String)
	 */
	public static final String TRS_SRC= "trssrc";

	/**
	 * <h4>sdk音频编解码格式</h4>
	 * <p>
	 *    设置音频流编解码格式，目前支持speex，speex-wb，ico，raw等4种格式
	 * </p>
	 * <p>是否必须设置:否</p>
	 * <p>默认值:speex-wb</p>
	 * <p>取值范围:{"speex","speex-wb","raw","ico"}</p>
	 * @see SpeechRecognizer#setParameter(String, String)
	 * @see SpeechSynthesizer#setParameter(String, String)
	 */
	public static final String AUDIO_FORMAT_AUE = "aue";
}
