package com.iflytek.cloud;

/**
 * <h3>事件类</h3>
 * <p>
 * 		事件类定义用于扩展事件回调中返回的事件ID。所有事件在设置生效后，通过
 * 各业务监听类的onEvent函数回调，如
 * {@link RecognizerListener#onEvent}。应用层可选择性的处
 * 理这些事件。
 * </p>
 * @since version:1073
 * @author <script>gShowAuthor();</script>
 * @see com.iflytek.cloud.RecognizerListener#onEvent
 * @see com.iflytek.cloud.SynthesizerListener#onEvent
 * @see com.iflytek.cloud.SpeechUnderstanderListener#onEvent
 * @see com.iflytek.cloud.TranscripterListener#onEvent
 * @see com.iflytek.cloud.VerifierListener#onEvent
 * @see com.iflytek.cloud.WakeuperListener#onEvent
 */
public class SpeechEvent {

	/**
	 *  <h4>网络状态消息</h4>
	 *  <p>
	 *  		网络状态是SDK通过测试网络连接返回的[0, 100]范围的状态值，值越高，
	 * 表示网络状态越好。网络状态值通过onEvent的参数2获取。
	 *  </p>
	 *  
	 *  @see SpeechConstant#ASR_NET_PERF
	 */
	public static final int EVENT_NETPREF = 10001;
	
	/**
	 * <h4>服务端会话id</h4>
	 * <p>
	 * 		在消息到达时,可通过onEvent(int, int, int, android.os.Bundle)的第4个参数obj,调用
	 *  {@link android.os.Bundle#getString}指定Key为{@link #KEY_EVENT_SESSION_ID},
	 *  获取服务端会话id.
	 * </p>
	 * @see #KEY_EVENT_SESSION_ID
	 */
	public static final int EVENT_SESSION_ID = 20001;
	
	/**
	 * <h4>服务端会话 key</h4>
	 * <p>
	 * 		用于在{@link #EVENT_SESSION_ID}事件到达时，获取会话ID值的KEY。
	 * </p>
	 * 
	 * @see #EVENT_SESSION_ID
	 */
	public static final String KEY_EVENT_SESSION_ID = "session_id";
	
	
	/**
	 * <h4>TTS合成数据消息</h4>
	 * <p>
	 * 		在消息到达时,{@link SynthesizerListener#onEvent}
	 * 的第4个参数obj中包含数据，通过{@link android.os.Bundle#getByteArray}
	 * 指定KEY为{@link #EVENT_TTS_BUFFER}获取.
	 * </p>
	 * 
	 * @see #KEY_EVENT_TTS_BUFFER
	 * @see SpeechConstant#TTS_DATA_NOTIFY
	 */
	public static final int EVENT_TTS_BUFFER = 21001;
	
	/**
	 * <h4>TTS 取音频数据key</h4>
	 * <p>
	 * 		用于在{@link #EVENT_TTS_BUFFER}事件到达时，获取音频数据的KEY。
	 * </p>
	 * @see #EVENT_TTS_BUFFER
	 */
	public static final String KEY_EVENT_TTS_BUFFER = "buffer";
	
	/**
	 * <h4>合成取消事件</h4>
	 * <p>
	 * 		合成会话被取消时，返回此消息。
	 * </p>
	 * 
	 * @see SpeechSynthesizer#stopSpeaking()
	 */
	public static final int EVENT_TTS_CANCEL = 21002;
	
	
	/**
	 * <h4>录音数据消息</h4>
	 * <p>
	 * 		听写、识别、语义、唤醒业务的录音数据消息。在消息到达时, onEvent
	 * 的第3个参数为音量（非唤醒业务时），第4个参数obj中包含音频数据，通
	 * 过{@link android.os.Bundle#getByteArray}
	 * 指定KEY为{@link #KEY_EVENT_RECORD_DATA}获取。
	 * </p>
	 * 
	 * @see #KEY_EVENT_RECORD_DATA
	 */
	public static final int EVENT_RECORD_DATA = 21003;
	
	/**
	 * <h4>音频数据key</h4>
	 * <p>用于在{@link #EVENT_RECORD_DATA}事件到达时，获取音频数据的KEY。</p>
	 * @see #EVENT_RECORD_DATA
	 */
	public static final String KEY_EVENT_RECORD_DATA = "data";

	/**
	 * <h4>唤醒识别结果</h4>
	 * <p>
	 * 		在消息到达时,{@link com.iflytek.cloud.WakeuperListener#onEvent}第2个参数arg1包含是否为
	 * 最后一个结果:1为是,0为否;第4个参数obj中包含数据，通过{@link android.os.Bundle#get(String)}
	 * 指定KEY为{@link #KEY_EVENT_IVW_RESULT}获取.
	 * </p>
	 * 
	 * @see #KEY_EVENT_IVW_RESULT
	 * @see SpeechConstant#IVW_SST
	 */
	public static final int EVENT_IVW_RESULT = 22001;
	
	/**
	 * <h4>唤醒识别结果KEY</h4>
	 * <p>用于在{@link #EVENT_IVW_RESULT}事件到达时，获取结果数据的KEY。</p>
	 * 
	 * @see #EVENT_IVW_RESULT
	 */
	public static final String KEY_EVENT_IVW_RESULT = "rec_result";
	
	/**
	 * <h4>开始处理录音数据</h4>
	 * <p>
	 * 		在录音开始后，SDK获取到第一段音频数据时，返回此消息。随后，如果有设置
	 * {@link SpeechConstant#NOTIFY_RECORD_DATA 抛出录音数据}，则会通过
	 * {@link #EVENT_RECORD_DATA}返回第一段音频数据。此消息一般在
	 * {@link RecognizerListener#onBeginOfSpeech()}回调后到达。
	 * </p>
	 * 
	 * @see #EVENT_RECORD_DATA
	 * @see #EVENT_RECORD_STOP
	 * @see RecognizerListener#onBeginOfSpeech()
	 */
	public static final int EVENT_SPEECH_START= 22002;
	
	/**
	 * <h4>录音结束</h4>
	 * <p>
	 * 		在停止录音后，返回此消息。无论是SDK通过{@link SpeechConstant#VAD_EOS}检测到尾端点，
	 * 还是应用层主动结束录音，都会返回此消息。
	 * </p>
	 * 
	 * @see #EVENT_SPEECH_START
	 * @see RecognizerListener#onEndOfSpeech()
	 */
	public static final int EVENT_RECORD_STOP= 22003;

	
	/**
	 * <h4>服务端音频url</h4>
	 * <p>
	 * 		在消息到达时,{@link com.iflytek.cloud.RecognizerListener#onEvent}
	 * 第4个参数obj,包含数据,通过{@link android.os.Bundle#getString}
	 * 指定KEY为{@link #KEY_EVENT_AUDIO_URL}获取.
	 * </p>
	 */
	public static final int EVENT_AUDIO_URL = 23001;
	
	/**
	 * <h4>服务端音频url key</h4>
	 * <p>用于在{@link #EVENT_AUDIO_URL}事件到达时，获取数据的KEY。</p>
	 * @see #EVENT_AUDIO_URL
	 */
	public static final String KEY_EVENT_AUDIO_URL = "audio_url";
	
	
	
	
	
	/**
	 * <h4>转写音频文件消息</h4>
	 * 
	 * <p>转写功能暂未开放，相关参数设计预留</p>
	 * <p>
	 * 		在录音模式下，成功创建音频文件时返回。可通过
	 * {@link com.iflytek.cloud.TranscripterListener#onEvent}
	 * 第4个参数obj调用{@link android.os.Bundle#getString}
	 * 指定Key为{@link com.iflytek.cloud.SpeechConstant#IST_AUDIO_PATH},获取音频文件绝对路径.
	 * 或通过{@link com.iflytek.cloud.SpeechTranscripter#getParameter},
	 * 指定Key为{@link com.iflytek.cloud.SpeechConstant#IST_AUDIO_PATH},获取音频文件绝对路径.
	 * </p>
	 */
	public static final int EVENT_IST_AUDIO_FILE = 10004;
	
	/**
	 * <h4>转写已上传字节消息</h4>
	 * 
	 * <p>转写功能暂未开放，相关参数设计预留</p>
	 * <p>
	 * 		在消息到达时,通过{@link com.iflytek.cloud.TranscripterListener#onEvent}
	 * 的第二个参数arg1,获取已确认上传到服务器的字节数.
	 * </p>
	 * <p>
	 * 		若当前音频源为非写音频模式,还可通过{@link com.iflytek.cloud.TranscripterListener#onEvent}
	 * 的第三个参数arg2,获取当前所有音频的字节大小.录音模式时，由于所有音频字节大小会变。
	 * 当停止音频输入后（等待录音时间超时
	 * {@link com.iflytek.cloud.SpeechConstant#KEY_SPEECH_TIMEOUT}，
	 * ，或调用{@link com.iflytek.cloud.SpeechTranscripter#stopTranscripting}），
	 * 且服务器收到所有音频时，第四个参数obj，将包含完成标记的布尔值(true)，可通过obj调用
	 * {@link android.os.Bundle#getBoolean}指定KEY为{@link #KEY_EVENT_IST_UPLOAD_COMPLETE}获取。
	 * </p>
	 * <p>此消息可能多次返回.</p>
	 * 
	 * @see com.iflytek.cloud.SpeechConstant#IST_AUDIO_UPLOADED
	 * @see com.iflytek.cloud.SpeechTranscripter#writeAudio
	 * @see com.iflytek.cloud.SpeechTranscripter#startTranscripting
	 * @see com.iflytek.cloud.SpeechTranscripter#stopTranscripting
	 */
	public static final int EVENT_IST_UPLOAD_BYTES = 10006;
	
	/**
	 * <h4>转写缓存剩余</h4>
	 * 
	 * <p>转写功能暂未开放，相关参数设计预留</p>
	 * <p>
	 * 		此消息仅在音频源为-1时需要关注。
	 * </p>
	 * <p>
	 * 		在调用 {@link SpeechTranscripter#writeAudio}写音频时，应该关注此事件。
	 * 此事件在调用写音频接口、及音频最后被写入底层库时分别回调一次。当事件回调时，通过
	 * {@link com.iflytek.cloud.TranscripterListener#onEvent}
	 * 的第二个参数arg1,获取当前剩余的缓存大小，当缓存小于要写入的音频时，应该先暂停写音
	 * 频数据，直到下次缓存大小大于要写入的音频时.
	 * </p>
	 * <p>最大缓存为128KByte。</p>
	 * 
	 * @see SpeechTranscripter#writeAudio(byte[], int, int)
	 */
	public static final int EVENT_IST_CACHE_LEFT = 10007;
	
	/**
	 * <h4>转写是否已上传完标记KEY</h4>
	 * 
	 * <p>转写功能暂未开放，相关参数设计预留</p>
	 * <p>用于在{@link #EVENT_IST_UPLOAD_BYTES}事件中，获取是否已上传完成。</p>
	 * @see #EVENT_IST_UPLOAD_BYTES
	 */
	public static final String KEY_EVENT_IST_UPLOAD_COMPLETE = "ist_upload_complete";
	
	/**
	 * <h4>转写结果等待时间消息</h4>
	 * 
	 * <p>转写功能暂未开放，相关参数设计预留</p>
	 * <p>
	 * 		在转写所有音频都上传完后，服务器可能返回结果处理需要的时间，但也可能不返回。
	 * </p>
	 * <p>
	 * 		在消息到达时,通过
	 * {@link com.iflytek.cloud.TranscripterListener#onEvent}
	 * 的第二个参数arg1,获取当前结果需要的时间.
	 * 此消息可能多次返回，返回时间不定，且不一定会返回。若应用层已经获取到最后的结果，可直接忽略此参数。
	 * 否则，应用层应考虑等待结果时间太长时，结束当次的等待，一段时间后，通过下一次会话中使用续传。关于
	 * 续传的介绍，请参考{@link SpeechTranscripter}类说明。
	 * </p>
	 * 
	 * @see SpeechConstant#IST_SESSION_ID
	 * @see SpeechConstant#IST_AUDIO_UPLOADED
	 * @see SpeechConstant#IST_SYNC_ID
	 */
	public static final int EVENT_IST_RESULT_TIME = 10008;
	
	/**
	 * <h4>转写转写音频同步ID消息</h4>
	 * 
	 * <p>转写功能暂未开放，相关参数设计预留</p>
	 * <p>
	 * 		在消息到达时,通过 {@link com.iflytek.cloud.TranscripterListener#onEvent}
	 * 的第二个参数arg1,获取当前写音频同步ID.
	 * </p>
	 * <p>此消息可能多次返回.</p>
	 * 
	 * @see com.iflytek.cloud.SpeechConstant#IST_SYNC_ID
	 */
	public static final int EVENT_IST_SYNC_ID = 10009;
	
	/**
	 * <h4>会话开始消息</h4>
	 * <p>在会话开始成功后返回</p>
	 * 
	 * @see com.iflytek.cloud.SpeechTranscripter#startTranscripting
	 * @see com.iflytek.cloud.SpeechTranscripter#writeAudio
	 */
	public static final int EVENT_SESSION_BEGIN = 10010;
	
	/**
	 * <h4>会话结束消息</h4>
	 * <p>在会话结束前返回</p>
	 *  
	 * @see com.iflytek.cloud.SpeechTranscripter#stopTranscripting
	 * @see com.iflytek.cloud.SpeechTranscripter#cancel
	 */
	public static final int EVENT_SESSION_END = 10011;
	
	/**
	 * <h4>音量消息</h4>
	 * <p>在得到音量时抛出，暂时只有身份验证的声纹业务用到</p>
	 */
	public static final int EVENT_VOLUME = 10012;
	
	/**
	 * <h4>VAD后端点消息</h4>
	 * <p>在检测到VAD后端点时抛出，暂时只有身份验证的声纹业务用到</p>
	 */
	public static final int EVENT_VAD_EOS = 10013;
	
}
