package com.iflytek.cloud;

import android.os.Parcel;
import android.os.Parcelable;

/**
 * <h3>唤醒结果类</h3>
 * <p>
 * 		唤醒结果：以json格式的字符串返回，其中各键值说明如下：
 * <br>sst: 业务类型，包括wakeup（唤醒）, enroll（注册）;参考{@link SpeechConstant#IVW_SST}
 * <br>id: 唤醒词id；
 * <br>score: 得分；参考{@link SpeechConstant#IVW_THRESHOLD}
 * <br>bos: 唤醒音频的前端点；
 * <br>eos: 唤醒音频的后端点；
 * <br>angle: 唤醒角度（仅在{@link SpeechConstant#IVW_CHANNEL_NUM}大于0时出现）；
 * </p>
 * <p>
 * 		唤醒注册结果：暂不支持。
 * </p>
 * @author <script>gShowAuthor();</script>
 * @since version:1073
 * @see WakeuperListener
 */
public class WakeuperResult implements Parcelable {
	/**
	 * 唤醒结果，请参考文档中唤醒结果定义。
	 */
	private String mResult = "";
	
	/**
	 * 唤醒音频仅在{@link SpeechConstant#IVW_CHANNEL_NUM}大于0时，有意义，其他时侯为null。
	 */
	private byte[] mBuffer = null;
	
	/*
	 * @hide
	 */
	public WakeuperResult( String result, byte[] buffer ){
		this( result );
		mBuffer = buffer;
	}

	/*
	 *@hide 
	 */
	public WakeuperResult(Parcel in) {
		mResult = in.readString();
	}

	/**
	 * <h4>构造函数</h4>
	 * <p>应用层只通过{@link WakeuperListener#onResult}得到结果，可忽略此类的构造函数。</p>
	 * @param result json格式的结果
	 */
	public WakeuperResult(String result) {
		if (null != result) {
			mResult = result;
		}
	}

	/**
	 * @hide
	 */
	public int describeContents() {
		return 0;
	}

	/**
	 * <h4>获取唤醒结果</h4>
	 * <p>
	 * 		结果各参数请参考类说明。
	 * </p>
	 * @return 唤醒结果
	 */
	public String getResultString() {
		return mResult;
	}
	
	/**
	 * <h4>获取唤醒音频</h4>
	 * <p>
	 * 		唤醒音频，仅在{@link SpeechConstant#IVW_CHANNEL_NUM}大于0时，
	 * 有意义，其他时侯为null。
	 * </p>
	 * 
	 * @return 唤醒音频
	 */
	public byte[] getBuffer(){
		return mBuffer;
	}

	/**
	 * @hide
	 */
	public void writeToParcel(Parcel dest, int flags) {
		dest.writeString(mResult);
	}

	/**
	 * @hide
	 */
	public static final Parcelable.Creator<WakeuperResult> CREATOR = new Parcelable.Creator<WakeuperResult>() {

		@Override
		public WakeuperResult createFromParcel(Parcel in) {
			return new WakeuperResult(in);
		}

		@Override
		public WakeuperResult[] newArray(int size) {
			return new WakeuperResult[size];
		}
	};
}
