package com.iflytek.cloud.msc.ivw.aimic.impl;

import java.util.LinkedList;
import java.util.Map;

/*************************************  begin LengthLimitedBuffer class  ******************************************/
public class LengthLimitedBuffer{
	private static final int INFINITE_LEN = -1;
	private static final int DEF_PER_BUF_LEN = 512;
	private static final int DEF_BUF_LEN = DEF_PER_BUF_LEN * 10;
	private static final boolean DEF_IS_BUF_REF = false;
	
	private static final int BUF_RESERVE = 2;
	private static final boolean DEF_OVERLAY = true;
	
	private final LinkedList<BufferEntry> mDatas = new LinkedList<BufferEntry>();
	private final LinkedList<BufferEntry> mFreeBufs = new LinkedList<BufferEntry>();
	private int mInitPerBufLen = DEF_PER_BUF_LEN;
	private long mInitBufLen = DEF_BUF_LEN;
	
	private long mMaxLen = INFINITE_LEN;
	private long mCurBufLen = 0;
	private long mCurStart = 0;
	private long mCurEnd = 0;
	
	private boolean mIsBufRef = DEF_IS_BUF_REF;
	private boolean mIsOverlay = DEF_OVERLAY;
	private long mMaxBufLen = 0;
	private final Object mSync = new Object();
	
	public LengthLimitedBuffer( final long maxLen, final int initPerBufLen, final long initBufLen, final boolean isBufRef, final boolean isOverlay ){
		mMaxLen = maxLen;
		mInitBufLen = initBufLen;
		mInitPerBufLen = initPerBufLen;
		mIsBufRef = isBufRef;
		
		mIsOverlay = isOverlay;
		mMaxBufLen = mMaxLen + mInitPerBufLen*BUF_RESERVE ;
		
		if( !mIsBufRef ){
			initBuffers();
		}
	}
	
	public LengthLimitedBuffer( final long maxLen ){
		this( maxLen, DEF_PER_BUF_LEN, maxLen/2, DEF_IS_BUF_REF, DEF_OVERLAY );
	}
	
	public LengthLimitedBuffer( final long maxLen, final int initPerBufLen ){
		this( maxLen, initPerBufLen, maxLen/2, DEF_IS_BUF_REF, DEF_OVERLAY );
	}
	
	public void raiseMaxLength( final long increaseLength ){
		synchronized( mSync ){
			this.mMaxLen += increaseLength;
			mMaxBufLen = mMaxLen + mInitPerBufLen*BUF_RESERVE ;
		}//end of synchronized
	}
	
	public long getMaxLength(){
		long value = 0;
		
		synchronized( mSync ){
			value = mMaxLen;
		}//end of synchronized
		
		return value;
	}
	
	public long getCurrentStart(){
		long value = 0;
		
		synchronized( mSync ){
			value = mCurStart;
		}
		
		return value;
	}
	
	public long getCurrentEnd(){
		long value = 0;
		
		synchronized( mSync ){
			value = mCurEnd;
		}
		
		return value;
	}
	
	public long getCurrentLength(){
		long value = 0;
		
		synchronized( mSync ){
			value = mCurEnd - mCurStart;
		}
		
		return value;
	}
	
	public int getCurrnetBlockUsed(){
		int ret = 0;
		
		synchronized( mSync ){
			ret = mDatas.size();
		}//end of synchronized
		
		return ret;
	}
	
	public void clear(){
		synchronized( mSync ){
			mCurStart = 0;
			mCurEnd = 0;
			BufferEntry bufferEntry = null; 

			while( !mDatas.isEmpty() ){
				bufferEntry = mDatas.pop();
				if( mIsBufRef ){
					bufferEntry.clear();
				}
				mFreeBufs.add( bufferEntry );
			}
		}//end of synchronized
	}
	
	public boolean isEmpty(){
		boolean ret = true;
		synchronized( mSync ){
			ret = mDatas.isEmpty();
		}
		return ret;
	}
	
	public BufferEntry popData(){
		BufferEntry bufEntry = null;
		
		synchronized( mSync ){
			if( !mDatas.isEmpty() ){
				bufEntry = mDatas.pop();
				mCurStart += bufEntry.getValue();
				if( !mIsBufRef && null!=bufEntry.getKey() ){
					mCurBufLen -= bufEntry.getKey().length;
				}
			}//end of if datas not empty
		}//end of synchronized
		
		return bufEntry;
	}
	
	public void recycle( final BufferEntry bufferEntry ){
		synchronized( mSync ){
			if( null != bufferEntry ){
				if( mIsBufRef ){
					bufferEntry.clear();
					mFreeBufs.add( bufferEntry );
				}else if( mCurBufLen<mMaxLen && null!=bufferEntry.getKey() ){
					mFreeBufs.add( bufferEntry );
					mCurBufLen += bufferEntry.getKey().length;
				}
			}
		}//end of synchronized
	}
	
	public void put( final byte[] data, final int dataOffset, final int dataLen ) throws IllegalArgumentException, NullPointerException, OutOfMemoryError{
		final int dataEndOffset = dataOffset + dataLen;
		if( null==data 
				|| data.length<dataEndOffset 
				|| 0>dataLen 
				|| (mIsBufRef && 0<dataOffset) ){
			throw new IllegalArgumentException();
		}else{
			synchronized( mSync ){
				if( 0<mMaxLen && 0<dataLen ){
					if( !mIsOverlay && mCurEnd-mCurStart+dataLen > mMaxLen ){
						throw new OutOfMemoryError( "current start="+mCurStart+", current end="+mCurEnd
								+", data len="+dataLen+", has over max len="+mMaxLen );
					}//end of if matched max len
					
					int offset = dataOffset;
					int saveLen = 0;
					int entryOffset = 0;
					
					BufferEntry entry = null;
					if( !mIsBufRef ){
						entry = mDatas.peekLast();
					}
					
					if( null == entry || entry.getKey().length==entry.getValue() ){
						entry = getBuffer();
					}else{
						entryOffset = entry.getValue();
					}

					while( offset < dataEndOffset ){
						if( mIsBufRef ){
							saveLen = dataLen;
							entry.setKey( data );
						}else{
							saveLen = Math.min( dataEndOffset-offset, entry.getKey().length-entryOffset );
							System.arraycopy( data, offset, entry.getKey(), entryOffset, saveLen );
						}
						
						offset += saveLen;
						entry.setValue( entryOffset+saveLen );
						this.mCurEnd += saveLen;
						
						if( !entry.equals(mDatas.peekLast()) ){
							mDatas.add( entry );
						}
						
						recycle();
						
						if( offset < dataEndOffset ){
							entry = getBuffer();
							entryOffset = 0;
						}
					}//end of while save all					
				}//end of len > 0
			}//end of synchronized
		}//end of if-else argument invalid
	}
	
	public Object getSync(){
		return mSync;
	}
	
	/**
	 * 使用整个队列时，通过{@link #getSync()}同步锁住，以免出现异常
	 * @return
	 */
	public LinkedList<BufferEntry> get(){
		return this.mDatas;
	}
	
	public boolean get( final byte[] buffer, final int dstOffset, final long srcStart, final int dataLen ){
		boolean ret = false;
		
		synchronized( mSync ){
			if( null!=buffer
					&& dataLen > 0
					&& buffer.length-dstOffset>=dataLen 
					&& srcStart >= mCurStart
					&& mCurEnd-srcStart>=dataLen ){
				
				int cpyLen = 0;
				long curOffset = mCurStart;
				final long endOffset = srcStart + dataLen;
				int bufOffset = dstOffset;
				for( LengthLimitedBuffer.BufferEntry entry : mDatas ){
					if( curOffset >= srcStart ){
						cpyLen = (int) Math.min( entry.getValue(), endOffset-curOffset);
						System.arraycopy( entry.getKey(), 0, buffer, bufOffset, cpyLen);
						bufOffset += cpyLen;
					}else if( curOffset+entry.getValue()>srcStart ){
						final int srcOffset = (int)( srcStart - curOffset );
						cpyLen = Math.min(dataLen, entry.getValue()-srcOffset);
						System.arraycopy( entry.getKey(), srcOffset, buffer, bufOffset, cpyLen );
						bufOffset +=cpyLen;
					}//end of if-else offset value
					
					curOffset += entry.getValue();
					if( curOffset >= endOffset ){
						break;
					}
				}//end of for entries
				
				ret = dstOffset+dataLen==bufOffset;
			}//end of arg is valid
		}//end of synchronized
		
		return ret;
	}
	
	public long getAvailable(){
		long len = 0;
		
		synchronized( mSync ){
			len = mFreeBufs.size() * mInitPerBufLen;
		}//end of synchronized
		
		return len;
	}
	
	public void gc(){
		boolean gcEnable = false;
		synchronized( mSync ){
			while( mInitBufLen<mCurBufLen && !mFreeBufs.isEmpty() ){
				mFreeBufs.remove();
				mCurBufLen -= mInitPerBufLen;
				gcEnable = true;
			}//end of while
		}//end of synchronized
		
		if( gcEnable ){
			System.gc();
		}
	}
	
	private BufferEntry getBuffer() throws OutOfMemoryError{
		BufferEntry entry = null;
		
		if( !mFreeBufs.isEmpty() ){
			entry =  mFreeBufs.remove();
		}
		
		if( null==entry ){
			if( mIsBufRef || !isMatchedMaxBufLen() ){
				int len = 0;
				byte[] buf = null;
				if( !mIsBufRef ){
					len = mInitPerBufLen;
					buf = new byte[ mInitPerBufLen ];
					mCurBufLen += mInitPerBufLen;
				}
				entry = new BufferEntry( buf, len );
			}else{
				throw new OutOfMemoryError( "current buffer len="+mCurBufLen+", has match max len: "+mMaxLen );
			}//end of if-else matched max length
		}//end of if entry is null
		
		return entry;
	}
	
	private boolean isMatchedMaxBufLen(){
		boolean ret = INFINITE_LEN != mMaxLen 
				&& mMaxBufLen <= mCurBufLen;
		return ret;
	}
	
	private void recycle() throws OutOfMemoryError{
		BufferEntry headEntry = mDatas.peek();
		if( null != headEntry ){
			final long newStart = this.mCurStart + headEntry.getValue();
			if( getMaxLength() <= this.mCurEnd-newStart ){
				BufferEntry entry = mDatas.remove();
				if( mIsBufRef ){
					entry.clear();
				}
				mFreeBufs.add( entry );
				
				this.mCurStart = newStart;
			}
		}//end of head not null
	}
	
	private void initBuffers(){
		if( 0 < mInitBufLen ){
			final int bufCount = (int)( mInitBufLen / mInitPerBufLen + BUF_RESERVE );
			for( int i=0; i<bufCount; ++i ){
				byte[] buf = new byte[ mInitPerBufLen ];
				BufferEntry entry = new BufferEntry( buf, mInitPerBufLen );
				mFreeBufs.add( entry );
				mCurBufLen += mInitPerBufLen;
			}
		}//end of if mInitBufLen > 0
	}
	
	
	private static class InternalEntry<K,V> implements Map.Entry<K,V>{
		private K key = null;
		private V value = null;
		
		public InternalEntry( K key, V value ){
			this.key = key;
			this.value = value;
		}
		
		@Override
		public K getKey() {
			return key;
		}
		
		public K setKey( K key ){
			K lastKey = key;
			this.key = key;
			return lastKey;
		}

		@Override
		public V getValue() {
			return value;
		}

		@Override
		public V setValue(V object) {
			V lastValue = value;
			value = object;
			return lastValue;
		}
		
		public void clear(){
			key = null;
			value = null;
		}
		
	}//end of class InternalEntry
	

	public static class BufferEntry extends InternalEntry<byte[], Integer>{

		public BufferEntry(byte[] key, Integer value) {
			super(key, value);
		}
		
	}//end of class BufferEntry
}
/*************************************  begin LengthLimitedBuffer class  ******************************************/
