package com.iflytek.cloud.msc.mfv;

import java.io.UnsupportedEncodingException;

import android.content.Context;
import android.os.SystemClock;
import android.text.TextUtils;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.msc.module.MscHandler;
import com.iflytek.cloud.msc.module.MscSession;
import com.iflytek.cloud.msc.util.DataUtil;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.msc.util.log.PerfLogger;
import com.iflytek.cloud.param.ParamBuilder;
import com.iflytek.msc.MSC;
import com.iflytek.msc.MSCSessionInfo;

public class MfvSession extends MscSession {

	private final static int	   MFV_DATA_BLOCK_FIRST	   = 0x01;
    private final static int       MFV_DATA_BLOCK_CONTINUE = 0x02;
    private final static int       MFV_DATA_BLOCK_LAST     = 0x04;

    /** member properties */
    private MSCSessionInfo         mSessionOut               = new MSCSessionInfo();
    private MSCSessionInfo         mWriteout                 = new MSCSessionInfo();
    // 通过QISRGetResult获取到的识别结果
    private byte[]                 mResult                   = null;
    @Override
    public int sessionBegin(final Context context, final String grammar,
            final MscHandler handler) throws SpeechError,
            UnsupportedEncodingException {
        String params = ParamBuilder.composeMfvSessionParam(context, grammar,handler);
        DebugLog.LogS("sessionBegin Params:" + params);
        
        long d1 = SystemClock.elapsedRealtime();
        //加锁防止外部多线程造成同时ssb成功，之后回调造成的崩溃。
    	PerfLogger.appendInfo(PerfLogger.MSC_SESSION_BIGNE, null);
        synchronized (MfvSession.class) {
        	mClientID = MSC.QMFVSessionBegin(params.getBytes(handler.getParamEncoding()), mSessionOut);
        }
    	PerfLogger.appendInfo(PerfLogger.SESSION_BEGIN_END, null);
        DebugLog.LogD("sessionBegin ErrCode:" + mSessionOut.errorcode
                + " time:" + (SystemClock.elapsedRealtime() - d1));
        int errorCode = mSessionOut.errorcode;
        if (0 != errorCode) {
            // 外面做超时处理，防止10129直接返回
            if (errorCode != ErrorCode.MSP_ERROR_CREATE_HANDLE
                    && errorCode != ErrorCode.MSP_ERROR_OVERFLOW
                    && errorCode != ErrorCode.MSP_ERROR_INVALID_OPERATION) {
                throw new SpeechError(errorCode);
            }
        }
        return errorCode;
    }

    @Override
    public void sessionEnd(final String reason) {
        if (null == mClientID) {
            return;
        }
        DebugLog.LogD("sessionEnd enter ");
        long d1 = System.currentTimeMillis();
        boolean bRet = 0 == MSC.QMFVSessionEnd(mClientID, reason.getBytes());
        DebugLog.LogD("sessionEnd leavel:" + bRet + " time:"
                + (System.currentTimeMillis() - d1));
        mClientID = null;
        mSessionID = null;
    }

    synchronized public void pushAudioData(StringBuffer param, final byte[] waveData,
            final int waveLen, boolean firstBlock) throws SpeechError {
    	StringBuffer paramBuffer = new StringBuffer(param.toString());
    	paramBuffer.append(",data_status=");
    	if (firstBlock) {
    		paramBuffer.append(MFV_DATA_BLOCK_FIRST);
		} else {
			paramBuffer.append(MFV_DATA_BLOCK_CONTINUE);
		}
		
		DebugLog.LogS("pushAudioData, param:" + paramBuffer.toString());
		writeData(paramBuffer.toString(), waveData, waveLen);
    }

    synchronized public void pushEndFlag(String ssub) throws SpeechError {
    	
    	PerfLogger.appendInfo(PerfLogger.LAST_DATA_FLAG, null);
		byte[] data = new byte[0];
		StringBuffer param = new StringBuffer();
		param.append("ssub=" + ssub);
		param.append(",data_status=" + MFV_DATA_BLOCK_LAST);
		DebugLog.LogD("mfv pushEndFlag, param:" + param.toString());
		writeData(param.toString(), data, 0);
    }

    /**
     * upload data to server
     * @param param 传入参数
     * @param waveData 音频数据
     * @param waveLen 数据长度
     * @throws SpeechError MSC error 信息
     */

    synchronized private void writeData(String param ,final byte[] waveData,
            final int waveLen) throws SpeechError {
        int nRet = MSC.QMFVDataWrite(mClientID, param.getBytes(), waveData, waveLen, mWriteout);
        mSessionOut.sesstatus = mWriteout.sesstatus;
        DebugLog.LogI("QISRAudioWrite length:" + waveLen);
        if (0 != nRet) {
            throw new SpeechError(mWriteout.errorcode);
        }
    }

    /**
     * 
     * @return true if find record end point
     */
    synchronized public int getEpStatus() {
        int eps = mWriteout.epstatues;
        return eps;
    }

    /**
     * get audio data volume value
     * 
     * @return
     */
    synchronized public int getAudioVolume() {
        int ret = 0;
        int vadret = 0;
        try {
            vadret = MSC.QMFVGetParam(mClientID, "volume".getBytes(), mWriteout);
            if (vadret == 0) {
                String volume = new String(mWriteout.buffer);
                // MscLog.Log(TAG,"VAD CHECK RET = " + vadret + " vad=" +
                // mWriteout.getQisrParamValue().length + new
                // String(mWriteout.getQisrParamValue()));
                ret = Integer.parseInt(new String(volume));
            } else {
                DebugLog.LogI("VAD CHECK FALSE");
            }
        } catch (Exception e) {
            DebugLog.LogI("getAudioVolume Exception vadret = " + vadret);
        }
        return ret;
    }

    /**
     * get int value
     * 
     * @return
     */
    synchronized public int getIntValue(final String tag) {
        if (mClientID == null) {
            return 0;
        }
        int ret = 0;
        try {
            String value = getStringValue(tag);
            if (!TextUtils.isEmpty(value)) {
                ret = Integer.parseInt(new String(value));
            } else {
            }
        } catch (Exception e) {
            DebugLog.LogE(e);
        }
        return ret;
    }

    /**
     * 设置会话参数，如rec_start、reco_ustop信息
     * 
     * @return
     */
    synchronized public boolean setParam(final String key, final String value) {
        if (TextUtils.isEmpty(key) || TextUtils.isEmpty(value)
                || mClientID == null) {
            return false;
        }
        int ret = -1;
        try {
            ret = MSC.QMFVSetParam(mClientID, key.getBytes(DataUtil.UTF8),value.getBytes(DataUtil.UTF8));
        } catch (UnsupportedEncodingException e) {
            DebugLog.LogE(e);
        }
        return (ret == 0);
    }

    /**
     * get String value
     * 
     * @return
     */
    synchronized public String getStringValue(final String tag) {
        if (mClientID == null) {
            return null;
        }
        int code = 0;
        try {
            code = MSC.QMFVGetParam(mClientID, tag.getBytes(), mSessionOut);
            if (code == 0) {
                return new String(mSessionOut.buffer);
            } else {
            }
        } catch (Exception e) {
        }
        return null;
    }

    /**
     * 上传、下载流量信息获取
     * 
     * @param tag
     * @return
     */
    public static int getIntByTag(final String tag) {
        try {
            String val = getStringByTag(tag);
            if (TextUtils.isEmpty(val)) {
                return 0;
            }
            return Integer.parseInt(val);
        } catch (Exception e) {
        }
        return 0;
    }

    /**
     * 根据tag获取msc相关的值
     * 
     * @param tag
     * @return
     */
    public static String getStringByTag(final String tag) {
        int code = 0;
        try {
            MSCSessionInfo info = new MSCSessionInfo();
            code = MSC.QMFVGetParam(null, tag.getBytes(), info);
            if (code == 0) {
                return new String(info.buffer).trim();
            } else {
            }
        } catch (Exception e) {
        }
        return null;
    }

    /**
     * if engine has result,return true
     * 
     * @return
     */
    synchronized public boolean hasResult() {
        return mSessionOut.sesstatus == 0;
    }

    public byte[] getResultData() {
        return mResult;
    }

    /**
     * 获得服务端ID
     */
    protected String getSessionID() {
        if (null == mSessionID) {
            mSessionID = getStringValue(TAG_SID);
        }
        return mSessionID;
    }
    
    /**
     * 获得音频url
     */
    protected String getAudioUrl() {
    	return  getStringValue(TAG_AUDIO_URL);
    }
}
