package com.iflytek.cloud.msc.util;

import java.lang.reflect.Field;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import android.app.ActivityManager;
import android.content.ComponentName;
import android.content.Context;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.res.Configuration;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.os.Build;
import android.os.Handler;
import android.os.Handler.Callback;
import android.os.Message;
import android.provider.Settings.Secure;
import android.telephony.TelephonyManager;
import android.text.TextUtils;
import android.util.DisplayMetrics;
import android.webkit.WebSettings;
import android.webkit.WebView;

import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.param.HashParam;

public class AppInfoUtil {

	//应用名称.
	public static final String APP_NAME			= "app.name";
	//应用安装路径.
	public static final String APP_PATH			= "app.path";
	//应用包名.
	public static final String APP_PKG_NAME		= "app.pkg";
	//应用版本名称.
	public static final String APP_VER_NAME		= "app.ver.name";
	//应用版本号.
	public static final String APP_VER_CODE		= "app.ver.code";
	
	// 系统名称
	public static final String OS_SYSTEM		= "os.system";
	// 屏幕分辨率
	public static final String OS_RESOLUTION	= "os.resolution";
	
	// 屏幕密度
	public static final String OS_DENSITY		= "os.density";
	// 运营商
	public static final String CARRIER		= "carrier";
	
	// MAC地址
	public static final String NET_MAC		= "net.mac";
	// IMEI
	public static final String OS_IMEI		= "os.imei";
	// IMSI
	public static final String OS_IMSI		= "os.imsi";
	
	// ANDROID_ID
	public static final String OS_ANDROID_ID	= "os.android_id";
	
	// 系统版本,10
	public static final String OS_VERSION     = "os.version";
	// 系统版本名称，2.3.7
	public static final String OS_RELEASE     = "os.release";
	// 系统进阶版本，2.4.13
	public static final String OS_INCREMENTAL = "os.incremental";
	
	// 浏览器 USER AGENT
	public static final String BROWSER_UA = "user_agent";
	
	// 设备类型：handset, tablet
	public static final String DVC_TYPE = "device_type";
	public static final String DVC_TYPE_HANDSET = "handset";
	public static final String DVC_TYPE_TABLET = "tablet";
	public static final String DVC_TYPE_UNKNOW = "unknow";
	
	// 与参数分隔符冲突要被替换的符号
	public static String[][] REPLACED_SYMBOLS = {
			{ "=", ":" }
			,{ ",", "_" }
	};
	
	public static String BUILD_INFO[][] = {
	    //设备厂商
		{"os.manufact", Build.MANUFACTURER}		
		//设备型号
		, {"os.model",    Build.MODEL}
	    //设备版本
		, {"os.product",  Build.PRODUCT}
	    //主题，示例：MIUI
		, {"os.display",  Build.DISPLAY}
	    //HOST,
	  , {"os.host",     Build.HOST}
	    //ID,Either a change list number, or a label like "M4-rc20". 
	  , {"os.id",       Build.ID}
	    //DEVICE
	  , {"os.device", 	Build.DEVICE}
	    //The name of the underlying board, like "goldfish". 
	  , {"os.board",  	Build.BOARD}
	    //The brand (e.g., carrier) the software is customized for, if any. 
	  , {"os.brand", 	Build.BRAND}
	    //os user,like:builder
	  , {"os.user",     Build.USER}
	    //os type,like:underdebug
	  , {"os.type",		Build.TYPE}
	};
	// 需要获取的设备信息表
	private static String BUILD_INFO_KEYS[][] = {
	    //CPU类型
		{"os.cpu",      "CPU_ABI"}
	  , {"os.cpu2",     "CPU_ABI2"}
	    //A hardware serial number, if available. Alphanumeric only, case-insensitive. 
	  , {"os.serial",   "SERIAL"}
	    // Hardware,like:vivo
	  , {"os.hardware", "HARDWARE"}
	    // bootloader version,like:2.00.0000
	  , {"os.bootloader", "BOOTLOADER"}
	    // The radio firmware version number. 
	  , {"os.radio", "RADIO"}
	  };
	
	
	// 设备信息列表
	private static HashParam initInfo = new HashParam();
	// 是否已经获取成功
	private static boolean initOK = false;
	/**
	 * 返回应用、设备信息.
	 * @param context
	 * @return HashMap<String, Object>
	 */
	public static synchronized HashParam getAppInfo(Context context)
	{
		if(initOK)
			return initInfo;
		initialDevInfo(context);		
		return initInfo;
	}
	
	/**
	 * 初始化设备信息数据 
	 * @throws SpeechError 
	 */
	private static void initialDevInfo(Context context)
	{		
		try {
			initInfo.clear();
			
			// 系统信息
			initInfo.putParam(OS_SYSTEM, "Android");

			// 获取应用版本号、版本名称信息
			initAppVerInfo(initInfo, context);
			// 获取分辨率信息
			DisplayMetrics dms = null;
			dms = context.getResources().getDisplayMetrics();
			initInfo.putParam(OS_RESOLUTION, dms.widthPixels + "*" + dms.heightPixels);
			initInfo.putParam(OS_DENSITY, ""+dms.density);

			// 获取IMEI、IMSI号
			TelephonyManager telMgr = (TelephonyManager) context
					.getSystemService(Context.TELEPHONY_SERVICE);
			
			try {
				initInfo.putParam(OS_IMEI, telMgr.getDeviceId());
			} catch (SecurityException e) {
				DebugLog.LogD("Failed to get did Info:SecurityException");
			} catch (Throwable e) {
				DebugLog.LogD("Failed to get did Info:other exception");
			}
			try {
				initInfo.putParam(OS_IMSI, telMgr.getSubscriberId());
			} catch (SecurityException e) {
				DebugLog.LogD("Failed to get sbid Info:SecurityException");
			} catch (Throwable e) {
				DebugLog.LogD("Failed to get sbid Info:other exception");
			}
			
			// Android_ID added by yjzhao@2014.4.16
			String androidID = Secure.getString(context.getContentResolver(),Secure.ANDROID_ID);   
			if(!TextUtils.isEmpty(androidID))
				initInfo.putParam(OS_ANDROID_ID, androidID);
	        
	        // 系统版本信息
	        initInfo.putParam(OS_VERSION, Build.VERSION.SDK);
	        initInfo.putParam(OS_RELEASE, Build.VERSION.RELEASE);
	        initInfo.putParam(OS_INCREMENTAL, Build.VERSION.INCREMENTAL);
	        
	        // 直接获取设备信息
	        for(int i = 0; i < BUILD_INFO.length;i++)
	        {
	        	initInfo.putParam(BUILD_INFO[i][0],BUILD_INFO[i][1]);
	        }
			// 获取设备信息
	        for(int i = 0; i < BUILD_INFO_KEYS.length;i++)
	        {
	        	String value = getBuildInfo(BUILD_INFO_KEYS[i][1]);
	        	initInfo.putParam(BUILD_INFO_KEYS[i][0], value);
	        }
	        
	        //增加mac地址
	        initInfo.putParam(NET_MAC, getMac(context));
	        
	        //增加运营商
	        initInfo.putParam(CARRIER, getCarrier(context));
	        
	        // 防止异常字符的出现,不可频繁调用，有效率问题
	        initInfo.trimParam();
	        
			initOK = true;
		} catch (Exception e) {
			DebugLog.LogD("Failed to get prop Info");
			initOK = false;
		} catch (Throwable e) {
			DebugLog.LogD("Failed to get property Info");
			initOK = false;
		}
	}
	
	/**
	 * 获取应用程序版本名称、版本号
	 * @param param
	 * @param context
	 * @throws  
	 */
	public static void initAppVerInfo(HashParam param,Context context)
	{
		try {
			PackageInfo pkginfo = context.getPackageManager().getPackageInfo(context.getPackageName(), 0);
			ApplicationInfo appinfo = context.getPackageManager().getApplicationInfo(context.getPackageName(), 0);
			param.putParam(AppInfoUtil.APP_VER_NAME, pkginfo.versionName);
			param.putParam(AppInfoUtil.APP_VER_CODE, "" + pkginfo.versionCode);
			
			param.putParam(AppInfoUtil.APP_PKG_NAME, appinfo.packageName);
			param.putParam(AppInfoUtil.APP_PATH, appinfo.dataDir);
			param.putParam(AppInfoUtil.APP_NAME, appinfo.loadLabel(context.getPackageManager()).toString());
		} catch (Exception e) {
		}
	}
	
	/**
	 * 获取精简 应用、设备信息.
	 * @param context
	 * @return HashMap<String, Object>
	 */
	public static HashParam getShortAppInfo(Context context)
	{
		HashParam apparam = getAppInfo(context);
		HashParam param = new HashParam();
		// 插入所有其他设备信息数据
		// 由于msc目前使用明文传输信息，因此需要进行精简，获取基本信息
		param.putParam(apparam, AppInfoUtil.APP_NAME);
		param.putParam(apparam, AppInfoUtil.APP_PATH);
		param.putParam(apparam, AppInfoUtil.APP_PKG_NAME);
		param.putParam(apparam, AppInfoUtil.APP_VER_NAME);
		param.putParam(apparam, AppInfoUtil.APP_VER_CODE);
		param.putParam(apparam, AppInfoUtil.OS_SYSTEM);
		param.putParam(apparam, AppInfoUtil.OS_RESOLUTION);
		param.putParam(apparam, AppInfoUtil.OS_DENSITY);
		param.putParam(apparam, AppInfoUtil.NET_MAC);
		param.putParam(apparam, AppInfoUtil.OS_IMEI);
		param.putParam(apparam, AppInfoUtil.OS_IMSI);
		param.putParam(apparam, AppInfoUtil.OS_VERSION);
		param.putParam(apparam, AppInfoUtil.OS_RELEASE);
		param.putParam(apparam, AppInfoUtil.OS_INCREMENTAL);
		param.putParam(apparam, AppInfoUtil.OS_ANDROID_ID);
		param.putParam(apparam, AppInfoUtil.CARRIER);
		
		param.putParam(apparam, AppInfoUtil.BUILD_INFO[0][0]);
		param.putParam(apparam, AppInfoUtil.BUILD_INFO[1][0]);
		param.putParam(apparam, AppInfoUtil.BUILD_INFO[2][0]);
		param.putParam(apparam, AppInfoUtil.BUILD_INFO[3][0]);
	
		return param;
	}
	
	/**
	 * 获取运营商,默认值空字符串
	 * @param context 上下文
	 * @return String string类型运营商信息
	 */
	public static String getCarrier(Context context)
	{
		String result = "";
		try
		{
			String carrier = ((TelephonyManager)context
					.getSystemService(Context.TELEPHONY_SERVICE))
					.getNetworkOperatorName();
			if (!TextUtils.isEmpty(carrier)) {
				result = carrier;
			}
		} catch (Throwable e) {
//			Logging.d(TAG, "Get carrier failed. ", e);
		}
		return result;
	}
	
	/**
	 * 获取MAC地址
	 * @param context 上下文
	 * @return String string类型mac地址
	 */
	public static String getMac(Context context)
	{
		String result = "";
		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M){
			try{
				NetworkInterface networkInterface = NetworkInterface.getByName("wlan0");
				if(networkInterface == null)
					return result;
				byte[] address = networkInterface.getHardwareAddress();
				StringBuffer buf = new StringBuffer();
				for (byte b:address) {
					buf.append(String.format("%02x:",b));
				}
				if (buf.length()>0) {
					buf.deleteCharAt(buf.length() - 1);
				}

				String mac = buf.toString();
				if (mac != null && mac.length() >0) {
					result = mac;
					return result;
				}
			}catch (SocketException e) {
				DebugLog.LogW(e+"");
			}catch (NullPointerException e) {
				DebugLog.LogW(e+"");
			}

		}else {
			try {
				// MAC地址
				WifiManager wifi = (WifiManager) context.getSystemService(Context.WIFI_SERVICE);
				WifiInfo wiinfo = wifi.getConnectionInfo();
				result = wiinfo.getMacAddress();
			} catch (Throwable e) {
				DebugLog.LogD("Failed to get mac Info");
			}
		}

		return result;
	}
	/**
	 * 获取设备信息
	 * @param key 信息名称
	 * @return value
	 */
	private static String getBuildInfo(final String key) {
		try {
			Field fd = android.os.Build.class.getField(key);
			if (null != fd) 
			{
				return fd.get(new android.os.Build()).toString();
			}
		} catch (Exception e) 
		{
//			DebugLog.LogE(e);
			return "unknown";
		}
		return "unknown";
	}

	private static String sSafariUserAgent = null;
	public static String getBrowserUserAgent( final Context context ){		
		if( TextUtils.isEmpty(sSafariUserAgent) ){
			try{
				final int MSG_GET_UA  = 1;
				final Callback uiCallback = new Callback(){
					@Override
					public boolean handleMessage(Message msg) {
						switch( msg.what ){
						case MSG_GET_UA:
							WebView webView = new WebView( context );
							WebSettings settings = webView.getSettings();
							sSafariUserAgent = settings.getUserAgentString();
							DebugLog.LogS( "user agent: "+sSafariUserAgent );
							break;
						}
						
						return false;
					}
					
				};
				
				final Handler handler = new Handler( context.getMainLooper(), uiCallback );
				handler.sendEmptyMessage( MSG_GET_UA );
			}catch( Throwable e ){
				DebugLog.LogE( e );
			}
		}
		
		
		DebugLog.LogS( "get user agent: "+sSafariUserAgent );
		return sSafariUserAgent;
	}
	
	private static String sDeviceType = null;
	public static String getDeviceType( final Context context ){
		if( TextUtils.isEmpty(sDeviceType) ){
			try{
				final int screenLayout = context.getResources().getConfiguration().screenLayout;
				if( (Configuration.SCREENLAYOUT_SIZE_MASK&screenLayout) >= Configuration.SCREENLAYOUT_SIZE_LARGE ){
					sDeviceType = DVC_TYPE_TABLET;
				}else if( (Configuration.SCREENLAYOUT_SIZE_MASK&screenLayout) >= Configuration.SCREENLAYOUT_SIZE_SMALL ){
					sDeviceType = DVC_TYPE_HANDSET;
				}else{
					sDeviceType = DVC_TYPE_UNKNOW;
				}
			}catch( Throwable e ){
				DebugLog.LogE( e );
			}
		}//end of if sDeviceType empty
		
		DebugLog.LogS( "get device type: "+sDeviceType );
		return sDeviceType;
	}
	
	public static String checkAndReplaceIllegalSymbols( final String strInfo ){
		String newStr = null;
		DebugLog.LogS( "check and replace src: "+strInfo );
		
		if( null != strInfo ){
			newStr = strInfo;
			for( int i=0; i<REPLACED_SYMBOLS.length; ++i ){
				newStr = newStr.replace( REPLACED_SYMBOLS[i][0], REPLACED_SYMBOLS[i][1] );
			}//end of for
		}
		
		DebugLog.LogS( "check and replace result: "+newStr );
		return newStr;
	}

	/**
	 * 获取当前活动页名称
	 * @param context
	 * @return
	 */
	public static String getActivityMsg(Context context) {
		ActivityManager am = (ActivityManager) context.getSystemService(Context.ACTIVITY_SERVICE);
		    try {
                int currentapiVersion=android.os.Build.VERSION.SDK_INT;
                PackageManager pm = context.getPackageManager();
                boolean permission = (PackageManager.PERMISSION_GRANTED == pm.checkPermission("android.permission.GET_TASKS",context.getPackageName()));
		        if(currentapiVersion <23 && permission) {
                    ComponentName cn = am.getRunningTasks(1).get(0).topActivity;
                    return cn.getClassName();
                }else{
		            return "";
                }
            }catch(Exception e){
		        e.printStackTrace();
		        return "";
            }

	}
}