package com.iflytek.cloud.msc.util.log;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.util.Iterator;
import java.util.LinkedHashMap;

import android.text.TextUtils;

/**
 * 性能日志示例：
 * QISRAudioWrite=[1280:41;1280:64;1280:108;1280:136;1280:183;1280:220;1280:254
 * ......
 * ;1280:1195;1280:1229;1280:1280],errorCode=0,onEnd=1726,onFirstAudio=1307
 * ,onLastAudio=1298,File Name=./sdcard/msc/iatpcm/iat00000007_hf3e4304e7a7c2481d00.pcm
 * ,startListening=02:15:41.229,onAudioWrite=[1280:20;1280:58;1280:96;1280:133
 * ......
 * ;1280:1652;1280:1690],QISRSessionEnd=1514,QISRSessionBegin=7,QISRGetResult=1511,
 */
public class PerfLogger {
	
	public static LinkedHashMap<String, String> mTimes = new LinkedHashMap<String, String>();
	// 业务会话类性能日志
	public static final String SDK_SESSION_BIGNE = "SDKSessionBegin";
	public static final String MSC_SESSION_BIGNE = "MSCSessionBegin";
	public static final String SESSION_BEGIN_END = "SessionBeginEnd";
	public static final String LAST_DATA_FLAG = "LastDataFlag";
	public static final String GET_RESULT = "GetNotifyResult";
	public static final String SESSION_END_BEGIN = "SessionEndBegin";
	public static final String SESSION_END_END = "SessionEndEnd";
	// 请求类性能日志
	public static final String SENT_REQUEST = "SendRequest";
	public static final String REQUEST_RESULT = "RequestResult";
	// 下载类性能日志
	public static final String CREATE_DOWNLOAD = "CreateDownload";
	public static final String DOWNLOAD_ONSTART = "DownloadonStart";
	public static final String DOWNLOAD_ONFINISH = "DownloadonFinish";
	
	private static String SYMBOL_EQUAL = "=";
	
	private static String SYMBOL_COLON = ":";
	
	private static String SYMBOL_SEMICOLON = ";";
	
	private static String SYMBOL_DIVISION = "========================" +
			"=================================" + "\r\n";
	
	// 是否保存性能日志,不允许通过编译时修改，只能通过外部设置
	private static boolean isLogSaved = false;
	
	/**
	 * 设置是否保存性能日志
	 */
	public static void setLogSaved(boolean logSaved)
	{
		isLogSaved = logSaved;
	}
	
	public static final String TYPE_MSC = "msc";
	public static final String TYPE_PRE = "pre";
	
	/**
	 * 保存日志文件，追加写入.
	 * @param filename
	 */
	public static synchronized void saveLogFile(String filename)
	{
		if(!isLogSaved || mTimes.size() < 1)
			return;
		
		FileWriter writer = null;
		BufferedWriter bfwriter = null;
		
		try 
		{	
			writer = new FileWriter(filename, true);
			bfwriter = new BufferedWriter(writer);
			bfwriter.write(SYMBOL_DIVISION);
			Iterator<String> iterator= mTimes.keySet().iterator();
			while(iterator.hasNext())
			{
				String key = iterator.next();
				String value = mTimes.get(key);
				if(value == null) {
					value = "null";
				}
				if (value.contains(SYMBOL_SEMICOLON))
					value = "[" + value + "]";
				else if (value.contains(SYMBOL_EQUAL))
					bfwriter.write(key + value);
				else
					bfwriter.write(key + SYMBOL_EQUAL + value);
				bfwriter.write("\r\n");
			}
			bfwriter.write(SYMBOL_DIVISION);
			bfwriter.close();
			writer.close();
		}catch (Exception e) {
			DebugLog.LogE(e);
		}
		finally
		{
			try {
				if(bfwriter != null)
					bfwriter.close();
				else if(writer != null)
					writer.close();
			} catch (Exception e) {
				DebugLog.LogE(e);
			}
		}
	}
	
	public static synchronized void clear()
	{
		if(!isLogSaved || mTimes == null)
			return;
		mTimes.clear();
	}
	
	/**
	 * 添加日志信息，分别将value和与time写入mVaules和mTime两个列表中
	 * 当首次写入信息，向mTime中写入当前时间，以后均写入时间差值
	 * @param key
	 * @param value
	 * @param extension
	 */
	public static synchronized void appendInfo(String key, String extension)	
	{
		if(!isLogSaved)
			return;
		DebugLog.LogS("appendInfo:" + key + "," + extension);
	
		//若extension不为空，在其后面加一个分号(;)以便与后边的时间结合
		if(!TextUtils.isEmpty(extension))
			extension += SYMBOL_COLON ;
		else
			extension = "";
		
		//第一次写入信息，直接写入当前时间，以后均写入时间差值.
		mTimes.put(key, extension + System.currentTimeMillis());
	}
	
	/**
	 * 直接输入键值
	 * lastaudio =11111;
	 * @param key
	 * @param value
	 */
	 public static synchronized void appendInfoByValue(String key, String value)
	  {
	    if (!isLogSaved)
	      return;
	    if ((!mTimes.containsKey(key)) || (TextUtils.isEmpty((CharSequence)mTimes.get(key))))
	    {
	      mTimes.put(key, value);
	    }
	    else {
	      String str = (String)mTimes.get(key);
	      str = str + SYMBOL_SEMICOLON + value;
	      mTimes.put(key, str);
	    }
	  }
}
