/*
 * 文件名：VoiceWakeuperImpl.java 5.1.1041.1019 July 9,2014
 * 描述 ：唤醒对象具体实现.
 * 作者:iFLytek
 * 版权：科大讯飞
 */
package com.iflytek.cloud.speech.impl;

import java.io.File;

import org.json.JSONObject;

import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.text.TextUtils;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.RequestListener;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.WakeuperListener;
import com.iflytek.cloud.WakeuperResult;
import com.iflytek.cloud.msc.ivw.AimicWakeuper;
import com.iflytek.cloud.msc.ivw.MscWakeuper;
import com.iflytek.cloud.msc.ivw.OptimizeManager;
import com.iflytek.cloud.msc.module.SpeechModule;
import com.iflytek.cloud.msc.util.Config;
import com.iflytek.cloud.msc.util.DataUtil;
import com.iflytek.cloud.msc.util.FileUtil;
import com.iflytek.cloud.msc.util.FuncAdapter;
import com.iflytek.cloud.msc.util.NetworkUtil;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.msc.util.log.PerfLogger;
import com.iflytek.cloud.param.MscKeys;
import com.iflytek.cloud.util.FileDownloadListener;
import com.iflytek.cloud.util.ResourceUtil;
import com.iflytek.cloud.util.ResourceUtil.RESOURCE_TYPE;

public class VoiceWakeuperImpl extends SpeechModule {

	// 是否会话过程中暂停后台音乐播放
	private boolean mRequestFocus = false;

	private String mUserPath = null;
	private String mUserCommonPath = null; 
	private String mUserCmdPath = null;
	private String mUserThreshold = null;
	private String mCfgPath = null;
	private String mCfgThreshold = null;
	private boolean isUserPath = false;

	private static String RESPATH = "respath";

	private OptimizeManager mOptimizeManager = null;
	
	private Config mConfig = null;

	public VoiceWakeuperImpl(Context context) {
		super(context);
		mConfig = Config.getConfig(mContext);
	}

	public int queryResource(String resourcePath, boolean isExternalCall,RequestListener listener) {
		synchronized (mSynObj) {
			if(TextUtils.isEmpty(resourcePath)) {
				DebugLog.LogE("respath is null. please set respath before startlisten");
				return ErrorCode.ERROR_IVW_INVALID_RESOURCE;
			}
			// 获取配置文件路径
			String configPath = null;
			if(!isExternalCall) {
				// 外部强制请求，不与缓存文件进行比较
				configPath = ResourceUtil.generateResourcePath(mContext,
					RESOURCE_TYPE.path, mConfig.getString(MscKeys.IVW_CONFIG_PATH, null));
			}
			
			if(mOptimizeManager != null) {
				mOptimizeManager.cancelRequest();
				mOptimizeManager = null;
			}
			mOptimizeManager = new OptimizeManager(mContext);
			JSONObject object = OptimizeManager.getResRequestInfo(resourcePath, configPath);
			
			// 当传入错误资源路径时，进行非空判断 
			if(object == null) {
				DebugLog.LogE("ivw invalid resource");
				return ErrorCode.ERROR_IVW_INVALID_RESOURCE;
			}
			
			// 移除路径字段
			String path = (String) object.remove(RESPATH);
			if (!TextUtils.isEmpty(path) && !path.equals(resourcePath)) {
				mCfgPath = path;
				mCfgThreshold = mConfig.getString(MscKeys.IVW_CONFIG_THRESHOLD, null);
			} else {
				// 不符合条件，移除缓存配置
				mCfgPath = null;
				mConfig.removeBean(MscKeys.IVW_CONFIG_PATH);
				mConfig.removeBean(MscKeys.IVW_CONFIG_THRESHOLD);
			}

			PerfLogger.appendInfo(PerfLogger.SENT_REQUEST, null);
			DebugLog.LogD( object.toString() ); 
			return mOptimizeManager.queryRequest(object, 
					new InternalRequestListener(isExternalCall, listener));
		}
	}

	public int downloadResource(String url, String filePath, String md5, boolean isExternalCall, FileDownloadListener listener) {
		synchronized (mSynObj) {
			if (TextUtils.isEmpty(url) 
					|| TextUtils.isEmpty(filePath)
					|| TextUtils.isEmpty(md5)) {
				return ErrorCode.ERROR_INVALID_PARAM;
			}

			PerfLogger.appendInfo(PerfLogger.CREATE_DOWNLOAD, null);
			
			if(mOptimizeManager != null) {
				mOptimizeManager.cancelRequest();
				mOptimizeManager = null;
			}
			mOptimizeManager = new OptimizeManager(mContext);
			return mOptimizeManager.downloadResource(url, filePath, md5,
					new InternalDownloadListener(isExternalCall, listener));
		}
	}

	/**
	 * 
	 * @param listener
	 *            回调监听
	 * @return void
	 */
	public int startListening(WakeuperListener listener) {
		int ret = ErrorCode.SUCCESS;
		
		synchronized (mSynObj) {
			try{
				mUserPath = mSessionParams.remove(SpeechConstant.IVW_RES_PATH);
				mUserThreshold = mSessionParams.remove(SpeechConstant.IVW_THRESHOLD); 
				mUserCommonPath = null;
				if( !TextUtils.isEmpty(mUserPath) ){
					final int startIdx = mUserPath.indexOf( MscKeys.VAL_SEP );
					if( 0<startIdx && mUserPath.length()>startIdx ){
						mUserCommonPath = mUserPath.substring( 0, startIdx );
						mUserCmdPath = mUserPath.substring(startIdx+1);
					}else{
						mUserCmdPath = mUserPath;
					}
				}
				
				// 判断ivw_net_mode参数是否可用
				if ( isQueryEnable() ) {
					mCfgPath = ResourceUtil.generateResourcePath(mContext,
							RESOURCE_TYPE.path, mConfig.getString(MscKeys.IVW_CONFIG_PATH, null));
					
					final Config config = mConfig;
					final long curTimeMs = System.currentTimeMillis(); 
					final long lastQueryTime = config.getLong(MscKeys.KEY_IVW_QUERY_LASTTIME
							, 0 );
					final long queryPeriod = mSessionParams.getLong( MscKeys.KEY_IVW_QUERY_PERIOD
							, MscKeys.IVW_DEF_QUERY_PERIOD );
					DebugLog.LogD( "query ivw res period: "+queryPeriod );
					
					if( curTimeMs-lastQueryTime >= queryPeriod ){
						String newResPath = null;
						if( isNetResourceNew() ){
							newResPath = mCfgPath;
						}else{
							newResPath = mUserCmdPath;
						}
						DebugLog.LogD( "begin resource query res path: "+newResPath );
						
						queryResource(newResPath, false, null);
						config.putLong( MscKeys.KEY_IVW_QUERY_LASTTIME, curTimeMs );
					}else if( curTimeMs == lastQueryTime ){
						config.putLong( MscKeys.KEY_IVW_QUERY_LASTTIME, curTimeMs );
					}//end of time valid
				}//end of if query
				
				ret = startWorking(listener);
			}catch( Throwable e ){
				ret = ErrorCode.ERROR_UNKNOWN;
				DebugLog.LogE( e );
			}//end of try-catch
		}//end of synchronized
		
		return ret;
	}

	private int startWorking(WakeuperListener listener) {
		int ret = ErrorCode.SUCCESS;
		
		synchronized (mSynObj) {
			try{
				// 根据配置路径状态，设置唤醒参数
				if ( isQueryEnable() && isNetResourceNew() ) {
					DebugLog.LogD( "ivw use resource from server" );
					String newResPath = null;
					if( TextUtils.isEmpty(mUserCommonPath) ){
						newResPath = mCfgPath;
					}else{
						newResPath = mUserCommonPath + MscKeys.VAL_SEP + mCfgPath;
					}
					mSessionParams.putParam(SpeechConstant.IVW_RES_PATH, newResPath);
					mSessionParams.putParam(SpeechConstant.IVW_THRESHOLD, null );	//cleanup hreshold, MSC will get it by resource file 
					setIsUserPath(false);
					
				} else {
					mSessionParams.putParam(SpeechConstant.IVW_RES_PATH, mUserPath);
					mSessionParams.putParam(SpeechConstant.IVW_THRESHOLD, mUserThreshold);
					setIsUserPath(true);
				}//end of if-else use net resource
				
				mRequestFocus = mSessionParams.getBoolean(
						SpeechConstant.KEY_REQUEST_FOCUS, false);
				// 唤醒过程中调用则结束上一次唤醒
				if (mscer != null && mscer.isRunning()) {
					((MscWakeuper) mscer).cancel(false);
				}

				final int channel = mSessionParams.getInt( SpeechConstant.IVW_CHANNEL_NUM, MscKeys.IVW_CHANNEL_SINGLE );
				if( MscKeys.IVW_CHANNEL_SINGLE == channel ){
					mscer = new MscWakeuper(mContext, mSessionParams,
							getHandlerThread("wakeuper"));
				}else{
					mscer = new AimicWakeuper(mContext, mSessionParams,
							getHandlerThread("wakeuper"));
				}

				// 会话开始，暂停后台音乐播放.
				FuncAdapter.Lock(mContext, mRequestFocus, null);
				((MscWakeuper) mscer)
						.startListening(new InternalListener(listener));
			}catch(SpeechError e){
				ret = e.getErrorCode();
				DebugLog.LogE( e );
			}catch( Throwable e ){
				ret = ErrorCode.ERROR_UNKNOWN;
				DebugLog.LogE( e );
			}//end of try-catch
		}//end of synchronized
		
		return ret;
	}
	
	/**
	 * 用于性能测试工具方法
	 * @param configPath
	 * @return
	 */
	public boolean restartWorking(String configPath) {
		boolean ret = false;
		synchronized (mSynObj) {
			if(TextUtils.isEmpty(configPath)) {
				return ret;
			}
			mConfig.putString(MscKeys.IVW_CONFIG_PATH, configPath);
			restartWorking(false);
			ret = true;
		}
		return ret;
	}

	private void restartWorking(boolean isError) {
		DebugLog.LogD("restart wake ,isError:"+isError);
		synchronized (mSynObj) {
			if(isError) {
				// 重启用户唤醒资源，配置路径置空及删除唤醒相关信息
				mCfgPath = null;
//				// 设置用户唤醒词路径
//				mSessionParams.putParam(SpeechConstant.IVW_RES_PATH, mUserPath);
//				mSessionParams.putParam(SpeechConstant.IVW_THRESHOLD, mUserThreshold);
//				setIsUserPath(true);
				// 启动唤醒
				startWorking(((MscWakeuper) mscer).getListener());
			} else if(mscer.isRunning()){
				// 下载成功，使用配置唤醒资源，重启唤醒
				// 从唤醒中获取配置资源路径，并设置唤醒词路径
				mCfgPath = ResourceUtil.generateResourcePath(mContext,
						RESOURCE_TYPE.path, mConfig.getString(MscKeys.IVW_CONFIG_PATH, null));
				mCfgThreshold = mConfig.getString(MscKeys.IVW_CONFIG_THRESHOLD, null);
//				mSessionParams.putParam(SpeechConstant.IVW_RES_PATH, mCfgPath);
//				mSessionParams.putParam(SpeechConstant.IVW_THRESHOLD, "");
//				setIsUserPath(false);
				// 启动唤醒
				startWorking(((MscWakeuper) mscer).getListener());
			}
		}
	}

	/**
	 * 写入录音数据
	 * 
	 * @param buffer
	 * @return
	 */
	public int writeAudio(byte[] buffer, int offset, int length) {
		synchronized (mSynObj) {
			if (mscer == null) {
				DebugLog.LogD("writeAudio error, no active session.");
				return ErrorCode.ERROR_ENGINE_CALL_FAIL;
			}

			if (buffer == null || buffer.length <= 0) {
				DebugLog.LogD("writeAudio error,buffer is null.");
				return ErrorCode.MSP_ERROR_INVALID_DATA;
			}

			if (buffer.length < (length + offset)) {
				DebugLog.LogD("writeAudio error,buffer length < length.");
				return ErrorCode.MSP_ERROR_INVALID_DATA;
			}

			final int audioSource = ((MscWakeuper) mscer).getAudioSource();
			if ( MscKeys.VALUE_AUDIO_EXTERNAL != audioSource ){
				return ErrorCode.MSP_ERROR_INVALID_PARA;
			}

			return ((MscWakeuper) mscer).writeAudio(buffer, offset, length);
		}
	}

	/**
	 * Stops listening for speech.
	 */
	public void stopListening() {
		synchronized (mSynObj) {
			if (mscer != null) {
				((MscWakeuper) mscer).stopListening(true);
			}
		}
	}

	/**
	 * 取消消息，保存底层事件
	 */
	@Override
	public void cancel(boolean notifyError) {
		synchronized (mSynObj) {
			if(mOptimizeManager != null) {
				mOptimizeManager.cancelRequest();
				mOptimizeManager = null;
			}
			// 会话取消，后台音乐继续播放.
			FuncAdapter.UnLock(mContext, mRequestFocus, null);
			super.cancel(notifyError);
		}
	}

	@Override
	public boolean destroy() {
		synchronized (mSynObj) {
			if(mOptimizeManager != null) {
				mOptimizeManager.cancelRequest();
				mOptimizeManager = null;
			}
			return super.destroy();
		}
	}

	public boolean isListening() {
		synchronized (mSynObj) {
			return isRunning();
		}
	}

	private synchronized void setIsUserPath(boolean isTrue) {
		isUserPath = isTrue;
	}

	private synchronized boolean getIsUserPath() {
		return isUserPath;
	}
	
//	public int updateWords(String action, String params) {
//		synchronized (mSynObj) {
//			if (action.equals("delete")) {
//				// 删除资源
//				return -1;
//				// return MscWakeuper.deleteSource(params);
//			} else if (action.equals("merge")) {
//				// 合并资源
//				return MscWakeuper.mergeSource(params);
//				// return MscWakeuper.mergeSource(mContext,params);
//			} else {
//				return -1;
//				// return ErrorCode.ERROR_INVALID_PARAM;
//			}
//		}
//	}
	
	private boolean isQueryEnable(){
		final int netMode = mSessionParams.getInt(SpeechConstant.IVW_NET_MODE,
				MscKeys.IVW_MODE_CLOSE);
		return MscKeys.IVW_NORMAL_QUERY==netMode 
				|| (MscKeys.IVW_WIFI_QUERY == netMode && NetworkUtil.isWifiConnect(mContext));
	}

	private boolean isNetResourceNew(){
		boolean ret = false;
		
		if( !TextUtils.isEmpty(mCfgPath) ){
			if( TextUtils.isEmpty(mUserPath) ){
				ret = true;
			}else{
				final JSONObject netResInfo = OptimizeManager.getIvwResInfo( mCfgPath, "" );
				final JSONObject userResInfo = OptimizeManager.getIvwResInfo( mUserCmdPath, "" );
				ret = OptimizeManager.isResNew( netResInfo, userResInfo );
			}//end of if-else user res is null or not
		}//end of if net res not null
		
		return ret;
	}
	
	/**
	 * 资源查询内部监听
	 * 
	 * @author sryang
	 */
	private class InternalRequestListener implements RequestListener {
		private RequestListener mOutListener = null;

		private final int BASE_MSG_EVENT = 0;
		private final int BASE_MSG_DATA = 1;
		private final int BASE_MSG_END = 2;
		// 外部请求标志
		private boolean isExternalCall = false;

		public InternalRequestListener(boolean externalCall, RequestListener outlistener) {
			isExternalCall = externalCall;
			mOutListener = outlistener;
		}

		private Handler mUiHandler = new Handler(Looper.getMainLooper()) {
			@Override
			public void handleMessage(Message msg) {
				if (mOutListener == null)
					return;
				switch (msg.what) {
				case BASE_MSG_EVENT:
					mOutListener.onEvent(msg.arg1, (Bundle) msg.obj);
					break;
				case BASE_MSG_DATA:
					mOutListener.onBufferReceived((byte[]) msg.obj);
					break;
				case BASE_MSG_END:
					mOutListener.onCompleted((SpeechError) msg.obj);
					break;
				default:
					break;
				}
				super.handleMessage(msg);
			}
		};

		@Override
		public void onEvent(int eventType, Bundle params) {
			PerfLogger.appendInfo(PerfLogger.REQUEST_RESULT, null);
			
			Message msg = mUiHandler.obtainMessage(BASE_MSG_EVENT, eventType,
					0, params);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onCompleted(SpeechError error) {
			PerfLogger.appendInfo(PerfLogger.REQUEST_RESULT, null);
			
			Message msg = mUiHandler.obtainMessage(BASE_MSG_END, error);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onBufferReceived(byte[] buffer) {
			DebugLog.LogD("onCompleted");
			try {
				// 仅在模式2或者模式4下启动下载
				if ( !isExternalCall && isQueryEnable() ) {
					// 解析出下载链接，进行下载
					String updateInfo = new String(buffer, DataUtil.UTF8);
					JSONObject obj = new JSONObject(updateInfo);

					// 获取下载链接
					String dlurl = obj.getString("dlurl");
					String md5 = obj.getString("md5");
					/**
					 * 默认下载路径
					 * 存放位置：sd卡或者/data/data 用户文件路径下
					 * 存放文件夹：/msc/ivw/
					 */
					// 解出链接文件名
					String resName = dlurl.substring(dlurl.lastIndexOf(File.separator)+1, dlurl.lastIndexOf("."));
					DebugLog.LogD("resName:"+resName);
					// 获取sdk资源下载路径
					String desPath = FileUtil.getResFilePath(mContext, resName);
					DebugLog.LogD("auto download path:" + desPath);
					// 启动自动下载
					downloadResource(dlurl, desPath, md5, isExternalCall, null);
					
					// 保存临时推荐门限值
					mConfig.putString(MscKeys.IVW_CONFIG_THRESTMEP, obj.getString("thresholder"));
				}
			} catch (Exception e) {
				onCompleted(new SpeechError(ErrorCode.ERROR_INVALID_DATA));
			}

			Message msg = mUiHandler.obtainMessage(BASE_MSG_DATA, buffer);
			mUiHandler.sendMessage(msg);
		}
	}

	/**
	 * 内部下载监听回调
	 * 
	 * @author sryang
	 */
	private final class InternalDownloadListener implements FileDownloadListener {
		private FileDownloadListener mOutListener = null;

		private final int MSG_START = 0;
		private final int MSG_PROGRESS = 1;
		private final int MSG_SUCCESS = 2;
		private final int MSG_FIALED = 3;
		// 外部请求标志
		private boolean isExternalCall = false;

		public InternalDownloadListener(boolean externalCall, FileDownloadListener listener) {
			isExternalCall = externalCall;
			mOutListener = listener;
		}

		private Handler mUiHandler = new Handler(Looper.getMainLooper()) {
			public void handleMessage(Message msg) {
				if (mOutListener == null)
					return;
				switch (msg.what) {
				case MSG_SUCCESS:
					mOutListener.onCompleted((String) msg.obj, null);
					break;
				case MSG_FIALED:
					mOutListener.onCompleted(null, (SpeechError) msg.obj);
					break;
				case MSG_START:
					mOutListener.onStart();
					break;
				case MSG_PROGRESS:
					mOutListener.onProgress(msg.arg1);
					break;
				default:
					break;
				}
				super.handleMessage(msg);
			};
		};

		@Override
		public void onStart() {
			PerfLogger.appendInfo(PerfLogger.DOWNLOAD_ONSTART, null);
			
			DebugLog.LogD("onStart");
			Message msg = mUiHandler.obtainMessage(MSG_START, null);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onProgress(int percent) {
			Message msg = mUiHandler.obtainMessage(MSG_PROGRESS, percent, 0,
					null);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onCompleted(String filePath, SpeechError error) {
			PerfLogger.appendInfo(PerfLogger.DOWNLOAD_ONFINISH, null);
			
			if (error == null) {
				DebugLog.LogD("onCompleted:filePath:" + filePath);

				if (!isExternalCall) {
					// 保存下载路径及推荐门限
					if (!TextUtils.isEmpty(filePath)) {
						mConfig.putString(MscKeys.IVW_CONFIG_PATH, filePath);
						mConfig.putString(MscKeys.IVW_CONFIG_THRESHOLD, 
								mConfig.getString(MscKeys.IVW_CONFIG_THRESTMEP, null));
					}
					restartWorking(false);
				}
				
				Message msg = mUiHandler.obtainMessage(MSG_SUCCESS, filePath);
				mUiHandler.sendMessage(msg);
			} else {
				DebugLog.LogD("onCompleted:errorcode:" + error.getErrorCode());
				
				Message msg = mUiHandler.obtainMessage(MSG_FIALED, error);
				mUiHandler.sendMessage(msg);
			}
		}

	}

	private final class InternalListener implements WakeuperListener {
		private WakeuperListener mOutListener = null;
		private static final int MSG_ERROR = 0;
		private static final int MSG_AUDIO_BUFFER = 1;
		private static final int MSG_SPEECH_BEGIN = 2;
		private static final int MSG_SPEECH_END = 3;
		private static final int MSG_RESULT = 4;
		private static final int MSG_VOLUME = 5;
		// EVENT自定义消息
		private static final int MSG_EVENT = 6;

		public InternalListener(WakeuperListener outlistener) {
			mOutListener = outlistener;
		}

		private Handler mUiHandler = new Handler(Looper.getMainLooper()) {
			@Override
			public void handleMessage(Message msg) {
				if (mOutListener == null)
					return;
				switch (msg.what) {
				case MSG_ERROR:
					mOutListener.onError((SpeechError) msg.obj);
					break;
				case MSG_AUDIO_BUFFER:
					break;
				case MSG_SPEECH_BEGIN:
					// 判断是否为重启操作
					if(mOutListener instanceof InternalListener) {
						break;
					}
					mOutListener.onBeginOfSpeech();
					break;
				case MSG_SPEECH_END:
					break;
				case MSG_RESULT:
					mOutListener.onResult((WakeuperResult) msg.obj);
					break;
				case MSG_VOLUME:
					mOutListener.onVolumeChanged(msg.arg1);
					break;
				case MSG_EVENT:
					Message event = (Message) msg.obj;
					if (event != null) {
						mOutListener.onEvent(event.what, event.arg1,
								event.arg2, (Bundle) event.obj);
					}
					break;
				default:
					break;
				}
				super.handleMessage(msg);
			}
		};

		@Override
		public void onResult(WakeuperResult result) {
			// try {
			// JSONObject object = new JSONObject(result.getResultString());
			// String sst = object.getString(SpeechConstant.IVW_SST);
			// if (sst.equals(MscKeys.IVW_ENROLL)) {
			// String status = object.getString(MscWakeuper.IVW_STATUS);
			// if (status.equals(MscWakeuper.IVW_STATUS_DONE)) {
			// sessionEnd();
			// }
			// }else {
			if (!mSessionParams.getBoolean(SpeechConstant.KEEP_ALIVE, true)) {
				sessionEnd();
			}
			// }
			//
			// } catch (JSONException e) {
			// e.printStackTrace();
			// }
			Message msg = mUiHandler.obtainMessage(MSG_RESULT, 1, 0, result);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onBeginOfSpeech() {
			DebugLog.LogD("onBeginOfSpeech");
			Message msg = mUiHandler
					.obtainMessage(MSG_SPEECH_BEGIN, 0, 0, null);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onError(SpeechError error) {
			DebugLog.LogE("error:" + error.getErrorCode());
			if (!getIsUserPath()) {
				restartWorking(true);
			} else {
				sessionEnd();
				Message msg = mUiHandler.obtainMessage(MSG_ERROR, error);
				mUiHandler.sendMessage(msg);
			}
		}

		/**
		 * 会话结束 判断是否需要保存音频或继续音乐播放 该接口不对外开放.
		 */
		protected void sessionEnd() {
			// 识别会话结束，后台音乐继续播放.
			FuncAdapter.UnLock(mContext, mRequestFocus, null);
		}

		@Override
		public void onEvent(int eventType, int arg1, int arg2, Bundle obj) {
			// 构造一个Event Message作为发送Message的obj
			Message msg = Message.obtain();
			msg.what = eventType;
			msg.arg1 = arg1;
			msg.arg2 = arg2;
			msg.obj = obj;
			Message sendMsg = mUiHandler.obtainMessage(MSG_EVENT, 0, 0, msg);
			mUiHandler.sendMessage(sendMsg);
		}

		@Override
		public void onVolumeChanged(int volume) {
			DebugLog.LogD("onVolumeChanged");
			Message msg = mUiHandler.obtainMessage(MSG_VOLUME, volume, 0, null);
			mUiHandler.sendMessage(msg);

		}
	}
}
