/*
 * 文件名：RecognizerView.java 2.5.1027.1040 Dec 11,2013
 * 描述 ：识别状态界面，包括录音中、识别状态、返回错误/取消/返回识别结果几种状态，
 * 界面主要包括以下几个部分：标题界面，仅在录音状态下显示；识别中界面，包括识别动画和提示；
 * 错误界面，包括错误动画和提示；
 * 作者:iFLytek
 * 版权：科大讯飞
 */

package com.iflytek.cloud.ui;
import android.content.Context;
import android.content.Intent;
import android.graphics.Paint;
import android.net.Uri;
import android.os.Bundle;
import android.os.SystemClock;
import android.provider.Browser;
import android.text.Html;
import android.text.Spannable;
import android.text.SpannableStringBuilder;
import android.text.Spanned;
import android.text.TextPaint;
import android.text.method.LinkMovementMethod;
import android.text.style.AbsoluteSizeSpan;
import android.text.style.ClickableSpan;
import android.text.style.ForegroundColorSpan;
import android.text.style.URLSpan;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.animation.Animation;
import android.view.animation.LinearInterpolator;
import android.view.animation.RotateAnimation;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.iflytek.cloud.ErrorCode;
import com.iflytek.cloud.InitListener;
import com.iflytek.cloud.RecognizerListener;
import com.iflytek.cloud.RecognizerResult;
import com.iflytek.cloud.SpeechError;
import com.iflytek.cloud.msc.util.Encrypter;
import com.iflytek.cloud.msc.util.FuncAdapter;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.param.MscKeys;
import com.iflytek.cloud.param.ParamBuilder;
import com.iflytek.cloud.resource.Resource;
import com.iflytek.cloud.SpeechRecognizer;
import com.iflytek.cloud.ui.control.MscView;
import com.iflytek.cloud.ui.control.DrawableMgr;
import com.iflytek.cloud.ui.control.StateRecordingDrawer;

public final class RecognizerView extends MscView implements OnClickListener{
	/** View的根目录layout，视图元素容器 */
	private LinearLayout mContainer;
	/** 录音状态动画 */
	private StateRecordingDrawer mRecordingDraw = null;
	/** 识别中状态动画 */
	private RotateAnimation mRotateAnimation = null;
	/** 识别对象 */
	private SpeechRecognizer mRecognizer;
    /** 识别结果监听器 */
	private RecognizerDialogListener mResultListener;
	
	/**
	 * 初始化识别界面和对象
	 * @param context
	 */
	public RecognizerView(Context context, InitListener listener) {
		super(context.getApplicationContext());
		
		mRecognizer = SpeechRecognizer.createRecognizer(context.getApplicationContext(),listener); 
		initView();
	}
	
	/**
	 * 初始化识别界面
	 */
	public void initView() {
		try {
			/* 使用getApplicationContext避免Activity的Context无法释放  */
			final Context context = getContext().getApplicationContext();
			View contentView = DrawableMgr.extractView(context, DrawableMgr.TAG_RECOGNIZE
					, this);
			contentView.setBackgroundDrawable(DrawableMgr.getDrawable
					(context.getApplicationContext(), DrawableMgr.TAG_RECOGNIZE_BG));

			final TextView link= (TextView)contentView.findViewWithTag
					(DrawableMgr.TAG_LINK);
			link.getPaint().setFlags(Paint. UNDERLINE_TEXT_FLAG );
			link.setText("语音识别能力由讯飞输入法提供");

			link.setLinksClickable(true);
			link.setOnClickListener(new OnClickListener() {
				@Override
				public void onClick(View v) {
					try {
						//加密处理appid
						String ending = Encrypter.cut16MD5(ParamBuilder.getAppid());
						Uri uri = Uri.parse("http://www.xunfei.cn/?appid="+ ending);
						DebugLog.LogD(uri.toString());
						Intent intent = new Intent(Intent.ACTION_VIEW, uri);
						intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);

						mState = STATE_RECORDING;
						updateUI();
						mRecognizer.stopListening();
						enterStateRecording();

						context.getApplicationContext().startActivity(intent);
					}catch (Exception e) {
						DebugLog.LogW("failed");
					}


				}
			});
			mContainer = (LinearLayout)contentView.findViewWithTag(DrawableMgr.TAG_CONTAINER);
			/* 不使用硬件加速，否则截取圆形不支持 */
			FuncAdapter.CloseHardWareAccelerate(this);
			
			/* 初始化识别中状态动画 */
			mRecordingDraw = new StateRecordingDrawer(context.getApplicationContext());
			LayoutParams lp = new LayoutParams(LayoutParams.FILL_PARENT, 0, 1);
			lp.bottomMargin = 20;
			mContainer.addView(mRecordingDraw, 1, lp);
			
			/* 初始化识别等待中状态动画*/
			FrameLayout recly = (FrameLayout)mContainer.findViewWithTag(DrawableMgr.TAG_WAITING);			
			View stateview = (View)recly.findViewWithTag(DrawableMgr.TAG_CONTROL);
			stateview.setBackgroundDrawable(DrawableMgr.getDrawable(getContext()
					, DrawableMgr.TAG_WAITING));
			
			mRotateAnimation = new RotateAnimation(0f, 360f, Animation.RELATIVE_TO_SELF
					, 0.5f, Animation.RELATIVE_TO_SELF, 0.5f); 
			mRotateAnimation.setRepeatCount(-1);	//永不停止
			mRotateAnimation.setInterpolator(new LinearInterpolator());  //匀速旋转
			mRotateAnimation.setDuration(700);  //一秒钟转2圈
			
		} catch (Exception e) {
			DebugLog.LogE(e);  //识别界面加载错误.
		}
	}
	
	/**
	 * 设置识别结果监听器
	 * @param resultListener
	 */
	public void setResultListener(RecognizerDialogListener resultListener) {
		mResultListener = resultListener;
		setOnClickListener(this);
	}
	
	/**
	 * 设置识别中使用到的参数.
	 * @param key
	 * @param value
	 */
	public void setParameter(String key, String value) {
		mRecognizer.setParameter(key, value);
	}
	
	private long mLastRequest = 0;
	
	/**
	 * 在部分手机上，按钮会连续两次响应开始事件，300ms内不允许连续点击两次
	 */
	private void startRecognizing() {
		DebugLog.LogD("startRecognizing");
		long lastRequest = mLastRequest;
		mLastRequest = SystemClock.elapsedRealtime();
		if (mLastRequest - lastRequest < 300) {
			return;
		}
		// 默认UI
		mRecognizer.setParameter(MscKeys.MSC_SKIN, "default");
		// 错误码
		int code = mRecognizer.startListening(mInternaListener);
		if(code != ErrorCode.SUCCESS)
			enterStateError(new SpeechError(code));
		else  /** start recognizing */
			enterStateRecording();
	}
	
	/**
	 * 初始化识别界面元素
	 */
	public static int SDK_GINGERBREAD = 9;
	
	/**
	 * 逆初始化动画所需资源
	 */
	private void unInitView() {
		if(mContainer != null) {
			mContainer.destroyDrawingCache();
			mContainer = null;
		}
		
		mRecordingDraw = null;
		/* 显示调用系统回收方法.*/
		System.gc();
	}
	
	/**
	 * 设置识别界面的标题
	 * @param text
	 */
	public void setTitle(CharSequence text) {
		TextView txtview = (TextView)mContainer
		    .findViewWithTag(DrawableMgr.TAG_TITLE);
		txtview.setText(text);
	}
	
	/**
	 * 设置识别回调监听器
	 */
	private RecognizerListener mInternaListener = new RecognizerListener() {

		/**
		 * 开始识别回调接口.
		 */
		public void onBeginOfSpeech() {
//			enterStateRecording();
		}

		/**
		 * 显示音量界面
		 * @param volume
		 */
		public void onVolumeChanged(int volume , byte[] data) {
			if(mState == STATE_RECORDING && null != mRecordingDraw) {
				// 音量返回值0-30,当音量28时返回满格音量值.
				volume = (volume + 2) / 5;
				// 不说话的情况下，音量进行随机
//				if(volume == 0 && DataUtil.RandomInt(0,4) < 2)
//					volume = 1;

				mRecordingDraw.setVolume(volume);
				mRecordingDraw.invalidate();
			} 
		}

		/**
		 * 结束识别回调接口.
		 */
		public void onEndOfSpeech() {
			enterStateWaiting();
		}

		/**
		 * 返回识别结果回调
		 * results,islast
		 */
		public void onResult(RecognizerResult result,boolean isLast) {
			if(isLast)
				dismiss();
			if (null != mResultListener) {
				mResultListener.onResult(result,isLast);
			}
		}

		/**
		 * 会话结束
		 */
		public void onError(SpeechError error) {			
			if(error == null || !mShow_error_view)
			{  
				dismiss();
			}
			else
				enterStateError(error);
			if (null != mResultListener) {
				mResultListener.onError(error);
			}
		}

		/**
		 * 回调网络状态
		 * @eventType
		 */
		public void onEvent(int eventType, int arg1, int arg2, Bundle obj) {
			
		}
	};
	
	/**
	 * 进入录音状态.
	 */
	private void enterStateRecording() {
		if(mRecordingDraw == null) {
			mRecordingDraw = new StateRecordingDrawer(getContext().getApplicationContext());
		}
		mState = STATE_RECORDING;
		updateUI();
	}

	/**
	 * 进入识别中状态.
	 */
	private void enterStateWaiting() {
		try {
			FrameLayout recly = (FrameLayout)mContainer
					.findViewWithTag(DrawableMgr.TAG_WAITING);			
			View stateview = (View)recly.findViewWithTag(DrawableMgr.TAG_CONTROL);
			stateview.startAnimation(mRotateAnimation);			
			
			/** update state */
			mState = STATE_WAITING;
			updateUI();
		} catch (Exception e) {
			DebugLog.LogE(e);
		}
	}
	
	/**
	 * 进入错误提示界面.
	 * @param error
	 */
	private void enterStateError(SpeechError error) {
		try {						
			//设置错误界面.
			LinearLayout errly = (LinearLayout)mContainer.findViewWithTag
					(DrawableMgr.TAG_ERROR);
			TextView linkview = (TextView)errly.findViewWithTag
					(DrawableMgr.TAG_ERROR_TXT);
			updateLinkItem(linkview, error);

			View errview = (View)errly.findViewWithTag(DrawableMgr.TAG_ERROR_VIEW);
			errview.setBackgroundDrawable(DrawableMgr.getDrawable(getContext()
					, DrawableMgr.TAG_WARNING));
			
			setTag(error);			
			mState = STATE_ERROR;
			updateUI();
		} catch(Exception e) {
			DebugLog.LogE(e);
		}
	}
	
	/**
	 * 获取新的linkItem，更新广告链接。
	 */
	public void updateLinkItem(TextView txtview, SpeechError error)
	{
		TextView linkview = txtview;
		
		boolean isPlainTips = true;
		String value = mRecognizer.getParameter( MscKeys.VIEW_TIPS_PLAIN );
		if( null != value
				&& (value.equalsIgnoreCase("false") || value.equalsIgnoreCase("0")) ){
			isPlainTips = false;
		}

		linkview.setText(Html.fromHtml(error.getHtmlDescription(isPlainTips)));
		linkview.setMovementMethod(LinkMovementMethod.getInstance());
		linkview.bringToFront();

		/**
		 * 获取linkView的URL点击事件
		 */
		CharSequence text = linkview.getText();
        if (text instanceof Spannable) 
        {
            int end = text.length();
            Spannable sp = (Spannable) linkview.getText();
            URLSpan[] urls = sp.getSpans(0, end, URLSpan.class);
            SpannableStringBuilder style = new SpannableStringBuilder(text);
            style.clearSpans();
            for (URLSpan url : urls) 
            {
            	LinkClickSpan myURLSpan = new LinkClickSpan(url.getURL());
            	style.setSpan(myURLSpan, sp.getSpanStart(url), sp.getSpanEnd(url), Spannable.SPAN_EXCLUSIVE_INCLUSIVE);
            }
            
            int errdesc = error.getHtmlDescription(false).length();
            int errcode = error.getHtmlDescription(true).length() 
            		- DrawableMgr.TAG_INTERVAL_LINE.length();
            
            /*
             * 设置错误码、错误描述、广告的字体和颜色值.
             */
            style.setSpan(new ForegroundColorSpan(DrawableMgr.getErrorTextColor()[0])
            	, 0, errdesc, Spannable.SPAN_EXCLUSIVE_INCLUSIVE);
            style.setSpan(new AbsoluteSizeSpan(DrawableMgr.getErrorTextSize()[0],true)
            	, 0, errdesc, Spanned.SPAN_EXCLUSIVE_EXCLUSIVE);
            
            if( isPlainTips ){
            	style.setSpan(new ForegroundColorSpan(DrawableMgr.getErrorTextColor()[1])
            		, errdesc + 1, errcode + 1, Spannable.SPAN_EXCLUSIVE_INCLUSIVE);
            	
            	style.setSpan(new AbsoluteSizeSpan(DrawableMgr.getErrorTextSize()[1],true)
        			, errdesc + 1, end, Spannable.SPAN_EXCLUSIVE_INCLUSIVE);
            }
            
            linkview.setText(style);
        }
	}
	
	/**
	 * URL点击事件处理
	 */
    public class LinkClickSpan extends ClickableSpan {
    	private String mUrl;
    	public LinkClickSpan(String url) {
    		mUrl = url;
    	}
    	
    	@Override
    	public void onClick(View widget) {    		
    		try{
    			Context context = widget.getContext();
				Uri uri = Uri.parse(mUrl);
				Intent intent = new Intent(Intent.ACTION_VIEW, uri);
				intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
				intent.putExtra(Browser.EXTRA_APPLICATION_ID, context.getPackageName());
				context.startActivity(intent);
			}catch(Exception e) {
				//若未安装浏览器，无点击事件.
				DebugLog.LogE(e);
			}finally {
			}
    	}
    	
    	 @Override  
	    public void updateDrawState(TextPaint textpaint) {  
	        super.updateDrawState(textpaint);  
//	        ds.setUnderlineText(false);  //取消下划线  
//	        ds.setColor(0xff0066ff);     //指定文字颜色  
	    }  
    }
	
	/**
	 * 更新界面元素，设置动画停止，界面是否可见.
	 */
	private void updateUI() {
		FrameLayout recly = (FrameLayout)mContainer.findViewWithTag(DrawableMgr.TAG_WAITING);
		TextView txtview = (TextView)mContainer.findViewWithTag(DrawableMgr.TAG_TITLE);
		LinearLayout errly = (LinearLayout)mContainer.findViewWithTag(DrawableMgr.TAG_ERROR);
		TextView tipsview = (TextView)recly.findViewWithTag(DrawableMgr.TAG_TIPS);
		
		if(mState == STATE_RECORDING) {
			//识别状态，设置错误View不可见 .
			errly.setVisibility(View.GONE);
			txtview.setVisibility(View.VISIBLE);
			recly.setVisibility(View.GONE);
			
	        txtview.setText(Resource.getTitle(Resource.TITLE_RECORDING));

			mRecordingDraw.setVolume(0);
	        mRecordingDraw.invalidate();
	        mRecordingDraw.setVisibility(View.VISIBLE);
		}else if(mState == STATE_WAITING) {       
			txtview.setVisibility(View.GONE);
			mRecordingDraw.setVisibility(View.GONE);
			recly.setVisibility(View.VISIBLE);
			
			tipsview.setVisibility(View.VISIBLE);
	        tipsview.setText(Resource.getTitle(Resource.TITLE_RECOGNIZE_WAITING));
		}else if(mState == STATE_ERROR) {
			txtview.setVisibility(View.GONE);
			mRecordingDraw.setVisibility(View.GONE);
			recly.setVisibility(View.GONE);
			
			errly.setVisibility(View.VISIBLE);
		}
	}
	
	private static final int STATE_RECORDING = 1;
	private static final int STATE_WAITING = 2;
	private static final int STATE_ERROR = 3;

	private volatile int mState;

	/**
	 * 识别View点击事件，当识别错误时点击（1）无网络状态下，跳转到网络设置界面  （2）否则跳转到识别界面
	 * 当时候过程中点击，停止录音进入识别中状态.
	 */
	public void onClick(View view) {
		switch (mState) {
			case STATE_ERROR:
				if(view.getTag() != null && ((SpeechError)view.getTag())
				.getErrorCode() == ErrorCode.ERROR_NO_NETWORK) {
					//打开网络设置界面
					openWireLessView();
				} else {	
					startRecognizing();
				}
				break;
			case STATE_RECORDING:
				mRecognizer.stopListening();
				enterStateWaiting();
				break;
		}
	}
	
	/**
	 * 开始识别
	 */
	public void start() {
		super.start();
		startRecognizing();
	}

	/**
	 * 取消识别
	 */
	public void cancel() {
		/** cancel recognizing */
		if(mRecognizer.isListening()){
			mRecognizer.cancel();
		}
		super.cancel();
	}
	
	/**
	 * 销毁识别对象.
	 */
	protected boolean destroy() {
		if( super.destroy() )
		{
			unInitView();
			return mRecognizer.destroy();
		}
		return false;
	}
}
