package com.iflytek.cloud.util;

import com.iflytek.cloud.SpeechError;

/**
 * <h3>文件监听器</h3>
 * <p>通过实现此接口，获取当前文件下载的进度、状态和结果</p>
 *
 * @author <script>gShowAuthor();</script>
 * @since version:1088
 */
public interface FileDownloadListener {

	/**
	 * <h4>下载启动回调</h4>
	 * <p>
	 * 		回调此函数时，下载任务已启动下载
	 * </p>
	 */
	public void onStart();

	/**
	 * <h4>下载进度回调</h4>
	 * <p>
	 * 		回调此函数时，下将返回载任务的进度信息，用于显示当前任务进度，已换算成百分制
	 * </p>
	 * @param percent 下载任务进度，已换算百分制。
	 */
	public void onProgress(int percent);

	/**
	 * <h4>下载完成回调</h4>
	 * <p>
	 * 		回调此函数时，下载任务完成。若下载成功回到文件绝对路径，若会话有错误，
	 * 则通过参数2返回错误信息。
	 * </p>
	 * @param filePath 下载文件绝对路径
	 * @param error 错误信息，会话正常时，error值为null。
	 * 
	 * @see SpeechError
	 */
	public void onCompleted(String filePath, SpeechError error);
}
